@@ -23,7 +23,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #endif
 
 /*  alloc.c  */
-char *alloc_page();
+void *alloc_page();
 object alloc_object();
 void *malloc(size_t);
 void *realloc(void *,size_t);
@@ -66,8 +66,8 @@ object big_plus();
 object big_times();
 object normalize_big_to_object();
 double big_to_double();
-EXTER  struct bignum big_fixnum1_body,big_fixnum2_body;
-EXTER object big_fixnum1,big_fixnum2;
+EXTER  struct bignum big_fixnum1_body,big_fixnum2_body,big_fixnum3_body,big_fixnum4_body;
+EXTER object big_fixnum1,big_fixnum2,big_fixnum3,big_fixnum4;
 object maybe_replace_big();
 
 
@@ -116,20 +116,14 @@ float object_to_float();
 double object_to_double();
 
 /*  error.c  */
-EXTER object sKerror;
-EXTER object sKwrong_type_argument;
 EXTER object sKcatch;
 EXTER object sKprotect;
 EXTER object sKcatchall;
-EXTER object sKtoo_few_arguments;
-EXTER object sKtoo_many_arguments;
-EXTER object sKunexpected_keyword;
-EXTER object sKinvalid_form;
-EXTER object sKunbound_variable;
-EXTER object sKinvalid_variable;
-EXTER object sKundefined_function;
-EXTER object sKinvalid_function;
-EXTER object sKpackage_error;
+EXTER object sKdatum;
+EXTER object sKexpected_type;
+EXTER object sKpackage;
+EXTER object sKformat_control;
+EXTER object sKformat_arguments;
 object wrong_type_argument();
 EXTER object sSuniversal_error_handler;
 
@@ -140,8 +134,8 @@ object simple_lispcall();
 object simple_lispcall_no_event();
 object simple_symlispcall();
 object simple_symlispcall_no_event();
-EXTER object Vevalhook;
-EXTER object Vapplyhook;
+EXTER object siVevalhook;
+EXTER object siVapplyhook;
 object ieval();
 object ifuncall(object,int,...);
 object ifuncall1();
@@ -185,6 +179,7 @@ EXTER object sLAterminal_ioA;
 EXTER object sLAtrace_outputA;
 EXTER object terminal_io;
 EXTER object standard_io;
+EXTER object standard_error;
 
 EXTER object sLAload_verboseA;
 EXTER object FASL_string;
@@ -212,6 +207,10 @@ frame_ptr frs_sch_catch();
 /*  gbc.c  */
 EXTER bool GBC_enable;
 
+#ifdef CAN_UNRANDOMIZE_SBRK
+EXTER bool gcl_unrandomized;
+#endif
+
 /*  let.c  */
 
 /*  lex.c  */
@@ -259,8 +258,8 @@ object cddddr();
 object nth();
 object nthcdr();
 object make_cons();
-object list(int,...);
-object listA(int,...);
+object list(fixnum,...);
+object listA(fixnum,...);
 object append();
 object copy_list();
 object make_list();
@@ -274,21 +273,14 @@ object macro_expand();
 
 /*  main.c  */
 EXTER char* system_directory;
-EXTER int ARGC;
-EXTER char **ARGV;
 void error();
-#ifdef UNIX
-EXTER char **ENVP;
-#endif
 object vs_overflow(void);
 
 EXTER object sSAsystem_directoryA;
 #ifdef UNIX
 EXTER char *kcl_self;
 #endif
-#if !defined(IN_MAIN) || !defined(ATT)
-EXTER bool initflag;
-#endif
+EXTER bool raw_image;
 char *merge_system_directory();
 
 
@@ -296,13 +288,13 @@ EXTER object sLquote;
 
 EXTER object sLlambda;
 
-EXTER object sLlambda_block;
-EXTER object sLlambda_closure;
-EXTER object sLlambda_block_closure;
+EXTER object sSlambda_block;
+EXTER object sSlambda_closure;
+EXTER object sSlambda_block_closure;
 
 EXTER object sLfunction;
-EXTER object sLmacro;
-EXTER object sLtag;
+EXTER object sSmacro;
+EXTER object sStag;
 EXTER object sLblock;
 
 
@@ -313,7 +305,7 @@ EXTER object sLblock;
 /*  number.c  */
 EXTER object shortfloat_zero;
 EXTER object longfloat_zero;
-#define make_fixnum(a) ((((FIXtemp=(a))+SMALL_FIXNUM_LIMIT)&(-2*SMALL_FIXNUM_LIMIT))==0?small_fixnum(FIXtemp):make_fixnum1(FIXtemp))
+/* #define make_fixnum(a) ({fixnum _a=(a);((_a+SMALL_FIXNUM_LIMIT)&(-2*SMALL_FIXNUM_LIMIT))==0?small_fixnum(_a):make_fixnum1(_a);}) */
 object make_fixnum1(long);
 object make_ratio();
 object make_shortfloat();
@@ -354,9 +346,6 @@ object shift_integer();
 /*  package.d  */
 EXTER object lisp_package;
 EXTER object user_package;
-#ifdef ANSI_COMMON_LISP
-EXTER object common_lisp_package;
-#endif
 EXTER object keyword_package;
 EXTER object system_package;
 EXTER object sLApackageA;
@@ -390,10 +379,11 @@ EXTER object sKname;
 EXTER object sKtype;
 EXTER object sKversion;
 EXTER object sKdefaults;
-EXTER object sKroot;
-EXTER object sKcurrent;
-EXTER object sKparent;
-EXTER object sKper;
+
+EXTER object sKabsolute;
+EXTER object sKrelative;
+EXTER object sKup;
+
 /* object parse_namestring(); */
 object coerce_to_pathname();
 /* object default_device(); */
@@ -401,14 +391,12 @@ object merge_pathnames();
 object namestring();
 object coerce_to_namestring();
 
-/*  prediate.c  */
-
-int eql(),equal(),eq();
-
 /*  print.d  */
 EXTER object sKupcase;
 EXTER object sKdowncase;
 EXTER object sKcapitalize;
+EXTER object sKpreserve;
+EXTER object sKinvert;
 EXTER object sKstream;
 EXTER object sKreadably;
 EXTER object sKescape;
@@ -461,7 +449,7 @@ EXTER object sLAread_default_float_formatA;
 EXTER object sLAread_baseA;
 EXTER object sLAread_suppressA;
 EXTER object READtable;
-/* EXTER object read_byte1();  */
+EXTER object read_byte1();
 EXTER int READdefault_float_format;
 EXTER int READbase;
 EXTER bool READsuppress;
@@ -564,25 +552,23 @@ EXTER object sSfunction_documentation;
 /*  typespec.c  */
 EXTER object sLcommon,sLnull,sLcons,sLlist,sLsymbol,sLarray,sLvector,sLbit_vector,sLstring;
 EXTER object sLsequence,sLsimple_array,sLsimple_vector,sLsimple_bit_vector,sLsimple_string;
-EXTER object sLcompiled_function,sLpathname,sLcharacter,sLnumber,sLrational,sLfloat,sLstring_char;
-EXTER object sLinteger,sLratio,sLshort_float,sLstandard_char,sLfixnum,sLpositive_fixnum, sLcomplex;
+EXTER object sLcompiled_function,sLpathname,sLcharacter,sLnumber,sLrational,sLfloat;
+EXTER object sLinteger,sLreal,sLratio,sLshort_float,sLstandard_char,sLfixnum,sLpositive_fixnum, sLcomplex;
 EXTER object sLsingle_float,sLpackage,sLbignum,sLrandom_state,sLdouble_float,sLstream,sLbit,sLreadtable;
 EXTER object sLlong_float,sLhash_table,sLstructure,sLboolean;
+EXTER object sLdivision_by_zero,sLfloating_point_inexact,sLfloating_point_invalid_operation;
+EXTER object sLfloating_point_overflow,sLfloating_point_underflow;
 
-#ifdef ANSI_COMMON_LISP
-/* new ansi types */
 EXTER object sLarithmetic_error,sLbase_char,sLbase_string,sLbroadcast_stream,sLbuilt_in_class;
-EXTER object sLcell_error,sLclass,sLconcatenated_stream,sLcondition,sLcontrol_error,sLdivision_by_zero;
+EXTER object sLcell_error,sLclass,sLconcatenated_stream,sLcondition,sLcontrol_error;
 EXTER object sLecho_stream,sLend_of_file,sLerror,sLextended_char,sLfile_error,sLfile_stream;
-EXTER object sLfloating_point_inexact,sLfloating_point_invalid_operation,sLfloating_point_overflow;
-EXTER object sLfloating_point_underflow,sLgeneric_function,sLlogical_pathname,sLmethod,sLpackage_error;
+EXTER object sLgeneric_function,sLlogical_pathname,sLmethod,sLpackage_error;
 EXTER object sLparse_error,sLprint_not_readable,sLprogram_error,sLreader_error,sLserious_condition;
 EXTER object sLsimple_base_string,sLsimple_condition,sLsimple_type_error,sLsimple_warning,sLstandard_class;
 EXTER object sLstandard_generic_function,sLstandard_method,sLstandard_object,sLstorage_condition;
 EXTER object sLstream_error,sLstring_stream,sLstructure_class,sLstyle_warning,sLsynonym_stream;
 EXTER object sLtwo_way_stream,sLtype_error,sLunbound_slot,sLunbound_variable,sLundefined_function,sLwarning;
 EXTER object sLmethod_combination,sLstructure_object;
-#endif
 
 EXTER object sLsatisfies;
 EXTER object sLmember;
@@ -593,10 +579,10 @@ EXTER object sLvalues;
 EXTER object sLmod;
 EXTER object sLsigned_byte;
 EXTER object sLunsigned_byte;
-EXTER object sLsigned_char;
-EXTER object sLunsigned_char;
-EXTER object sLsigned_short;
-EXTER object sLunsigned_short;
+EXTER object sSsigned_char;
+EXTER object sSunsigned_char;
+EXTER object sSsigned_short;
+EXTER object sSunsigned_short;
 EXTER object sLA;
 EXTER object sLplusp;
 EXTER object TSor_symbol_string;
@@ -614,12 +600,6 @@ EXTER object TSor_pathname_string_symbol_stream;
 EXTER int interrupt_flag;		/* console interupt flag */
 EXTER int interrupt_enable;		/* console interupt enable */
 
-/*  CMPtemp  */
-EXTER object CMPtemp;
-EXTER object CMPtemp1;
-EXTER object CMPtemp2;
-EXTER object CMPtemp3;
-
 EXTER object sLAlink_arrayA;
 
 /* nfunlink.c */
@@ -633,12 +613,10 @@ object list_vector_new();
 object Iapply_ap();
 object IisFboundp();
 object IapplyVector();
-object c_apply_n();
 EXTER object sSPmemory;
 EXTER object sSPinit;
 object sLfset();
 object MakeAfun();
-extern  object Cstd_key_defaults[];   
 extern object call_proc0();
 /* extern object call_proc(); */
 /* extern object call_vproc(); */
