@@ -13,6 +13,9 @@
 #include        <sys/file.h>
 #include        <sys/resource.h>
 */
+#ifndef plong
+#define plong int
+#endif
 
 #ifndef EXTER
 #define EXTER extern
@@ -30,7 +33,7 @@
 extern ulong ABS_MOST_NEGS[];
 extern ulong MOST_NEGS[];
 #undef ulong
-#define our_ulong unsigned long
+#define our_ulong unsigned plong
 
 
 GEN addii();
@@ -38,18 +41,7 @@ GEN icopy();
 GEN divss();
 GEN rcopy();
 
-EXTER int in_saved_avma ;
-
-/* #define DEBUG_AVMA */
-
-#ifdef DEBUG_AVMA
-#define save_avma long lvma = (in_saved_avma = 1, avma)
-#define restore_avma avma = (in_saved_avma = 0, lvma)
-#else
-#define save_avma long lvma = avma
-#define restore_avma avma = lvma
-#endif
-
+/* EXTER int in_saved_avma ; */
 
 
 
