@@ -7,7 +7,7 @@
 #  undef RSYM_COMMAND
 #  define SEPARATE_SFASL_FILE "sfaslbfd.c"
 #else
-#  undef SEPARATE_SFASL_FILE
+#  define SEPARATE_SFASL_FILE "sfaslcoff.c"
 #  define SPECIAL_RSYM "rsym_nt.c"
 #  define RSYM_COMMAND(command,system_directory,kcl_self,tmpfile1) \
       sprintf(command,"rsym %s %s",kcl_self,tmpfile1);
@@ -26,34 +26,12 @@
 #define f_nsyms NumberOfSymbols
 #define NO_PWD_H
 
-#define MAXPATHLEN 512
-
-/* alter pathToAlter to fit in with the Clibrary of the system.
-   and report error using name 'x' if you cant do it.
-   The result in pathToAlter should be less
-*/   
-#define FIX_FILENAME(x,pathToAlter) fix_filename(x,pathToAlter)
-
-#define MEMORY_SAVE(self,filename) \
-  do { char buf[MAXPATHLEN]; \
-       strcpy(buf,self); \
-       fix_filename(Cnil,buf); \
-       memory_save(buf,filename); \
-       } while (0)
-
 #define signals_pending *signalsPendingPtr
 
 #undef DBEGIN_TY
 #define DBEGIN_TY unsigned int
 extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
 
-/* define if there is no _cleanup,   do here what needs
-   to be done before calling unexec
-   */   
-#define CLEANUP_CODE \
-  setbuf(stdin,0); \
-  setbuf(stdout,0);
-
 #define NO_SYS_PARAM_H
 #define NO_SYS_TIMES_H
 
@@ -95,7 +73,6 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
 #define NEED_TO_REINSTALL_SIGNALS
 
 /*#define HAVE_SIGACTION*/
-#define SETUP_SIG_STACK
 #define SV_ONSTACK 0
 #define SA_RESTART 0
 
@@ -103,9 +80,10 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
    if the pointe/r is on the C stack or the 0 pointer
    in winnt our heap starts at DBEGIN
    */
-#define NULL_OR_ON_C_STACK(y) \
-    (((unsigned int)(y)) == 0 || \
-    (((unsigned int)(y)) > _stacktop && ((unsigned int)(y)) < _stackbottom))     
+/* #define NULL_OR_ON_C_STACK(y) \ */
+/*     (((unsigned int)(y)) == 0 || \ */
+/*     (((unsigned int)(y)) > _stacktop && ((unsigned int)(y)) < _stackbottom))      */
+/* #define NULL_OR_ON_C_STACK(x) (!(int *)x || ((int *)x>cs_limit && (int *)x<=cs_org)) */
       
 #if defined ( IN_FILE ) || defined ( IN_SOCKETS )
 #  define HAVE_NSOCKET
@@ -116,9 +94,10 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
 /* use the slightly older unexec */
 #define UNIXSAVE "unexnt.c"
     
-#define RECREATE_HEAP if (initflag) { recreate_heap1(); \
+#define RECREATE_HEAP { recreate_heap1(); \
      terminal_io->sm.sm_object1->sm.sm_fp=stdout; \
-     terminal_io->sm.sm_object0->sm.sm_fp=stdin; }
+     terminal_io->sm.sm_object0->sm.sm_fp=stdin; \
+     init_shared_memory();}
 
 #define HAVE_AOUT "wincoff.h"
 /* we dont need to worry about zeroing fp->_base , to prevent  */
@@ -139,14 +118,9 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
         ungetc(i, fp); \
     } while (0)
 		
-#define FCLOSE_SETBUF_OK 
 #define	IEEEFLOAT
 #define I386
 
-#undef SET_REAL_MAXPAGE  
-#define SET_REAL_MAXPAGE \
-	 init_shared_memory(); real_maxpage=MAXPAGE;
-
 /* include some low level routines for maxima */
 #define CMAC
 
@@ -166,7 +140,7 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
 	if (e - core_end < 0x10000 ) { \
 	  int i; \
 	  for (i=page(core_end); i < page(e); i++) { \
-	    type_map[i] = t_other; \
+	    \
 	  } \
 	  core_end = e; \
 	} \
@@ -213,3 +187,38 @@ extern char *GCLExeName ( void );
 
 /* End for cmpinclude */
 
+extern int mingwlisten(FILE *);
+#undef LISTEN_FOR_INPUT
+#define LISTEN_FOR_INPUT(fp) do {if (mingwlisten(fp)) return 0;} while (0)
+
+#define socklen_t int
+
+#undef DBEGIN
+#define DBEGIN _dbegin
+
+#define NOFREE_ERR
+
+#define FPE_CODE(i_,v_) make_fixnum((long)fSfpe_code((long)FFN(fSfnstsw)(),(long)FFN(fSstmxcsr)()))
+#define FPE_ADDR(i_,v_) make_fixnum(0)
+#define FPE_CTXT(v_) Cnil
+
+#define FPE_INIT Cnil
+
+#ifndef FE_INVALID
+#define FE_INVALID 1
+#define FE_DIVBYZERO 4
+#define FE_OVERFLOW 8
+#define FE_UNDERFLOW 16
+#define FE_INEXACT 32
+#endif
+
+#define FPE_FLTDIV 3
+#define FPE_FLTOVF 4
+#define FPE_FLTUND 5
+#define FPE_FLTRES 6
+#define FPE_FLTINV 7
+
+#include <limits.h>
+
+
+#define NO_FILE_LOCKING /*FIXME*/
