@@ -1,4 +1,5 @@
 
+
 #define	CHAR_CODE_LIMIT	256	
 #define	READ_TABLE_SIZE CHAR_CODE_LIMIT
 #define ARRAY_RANK_LIMIT 63
@@ -12,7 +13,7 @@ EXTER int GBC_enable;
 EXTER object sSAnotify_gbcA;
 
 /* symbols which are not needed in compiled lisp code */
-EXTER int interrupt_flag,initflag,interrupt_enable;
+EXTER int interrupt_flag,interrupt_enable;
 void install_default_signals();
 /* void sigint(),sigalrm(); */
 void segmentation_catcher();
@@ -21,19 +22,13 @@ void segmentation_catcher();
 EXTER int gc_enabled, saving_system;
 
 EXTER object lisp_package,user_package;
-#ifdef ANSI_COMMON_LISP
-EXTER object common_lisp_package;
-#endif
 EXTER char *core_end;
 EXTER int catch_fatal;
 EXTER long real_maxpage;
 char *getenv();
 EXTER char *this_lisp;
 
-#ifndef IN_MAIN
-EXTER
-char stdin_buf[], stdout_buf[];
-#endif
+EXTER char stdin_buf[],stdout_buf[];
 
 EXTER object user_package;
 
@@ -52,12 +47,6 @@ EXTER object user_package;
 			 else *__p++ = va_arg(ap,object);} \
   va_end(ap)
 
-/*  #undef endp */
-
-/*  #define	endp(obje)	((enum type)((endp_temp = (obje))->d.t) == t_cons ? \ */
-/*  			 FALSE : endp_temp == Cnil ? TRUE : \ */
-/*  			 endp1(endp_temp)) */
-
 #ifndef NO_DEFUN
 #undef DEFUN
 #define DEFUN(string,ret,fname,pack,min,max, flags, ret0a0,a12,a34,a56,doc) ret fname
@@ -81,8 +70,10 @@ void old(void) \
    pack == LISP ? LISP_makefun(string,fname,argd) : \
    error("Bad pack variable in MAKEFUN\n"))
 
-#define mjoin(a_,b_) a_ ## b_
-#define Mjoin(a_,b_) mjoin(a_,b_)
+#define MAKEFUNM(pack,string,fname,argd) \
+  (pack == SI ? SI_makefunm(string,fname,argd) : \
+   pack == LISP ? LISP_makefunm(string,fname,argd) : \
+   error("Bad pack variable in MAKEFUN\n"))
 
 #define SI 0
 #define LISP 1
@@ -92,6 +83,7 @@ void old(void) \
 #undef FFD
 #undef STATD
 #undef make_function
+#undef make_macro_function
 #undef make_si_function
 #undef make_si_sfun
 #undef make_special_form
@@ -100,8 +92,10 @@ void old(void) \
 #define LFD(a_) static void FFN(a_) (); void a_  () { FFN(a_)();} static void FFN(a_)
 #define FFD(a_) static void FFN(a_) (object); void a_  (object x) { FFN(a_)(x);} static void FFN(a_)
 #define make_function(a_,b_) make_function_internal(a_,FFN(b_))
+#define make_macro_function(a_,b_) make_macro_internal(a_,FFN(b_))
 #define make_si_function(a_,b_) make_si_function_internal(a_,FFN(b_))
 #define make_special_form(a_,b_) make_special_form_internal(a_,FFN(b_))
+#define make_si_special_form(a_,b_) make_si_special_form_internal(a_,FFN(b_))
 #define make_si_sfun(a_,b_,c_) make_si_sfun_internal(a_,FFN(b_),c_)
 #define STATD static
 #else
@@ -109,8 +103,10 @@ void old(void) \
 #define LFD(a_) void a_
 #define FFD(a_) void a_
 #define make_function(a_,b_) make_function_internal(a_,b_)
+#define make_macro_function(a_,b_) make_macro_internal(a_,b_)
 #define make_si_function(a_,b_) make_si_function_internal(a_,b_)
 #define make_special_form(a_,b_) make_special_form_internal(a_,b_)
+#define make_si_special_form(a_,b_) make_si_special_form_internal(a_,b_)
 #define make_si_sfun(a_,b_,c_) make_si_sfun_internal(a_,b_,c_)
 #define STATD
 #endif
@@ -121,6 +117,12 @@ void Mjoin(fname,_init) () {\
 }\
 STATD ret FFN(fname) args
 
+#define DEFUNM_NEW(string,ret,fname,pack,min,max, flags, ret0a0,a12,a34,a56,args,doc) STATD ret FFN(fname) args;\
+void Mjoin(fname,_init) () {\
+   MAKEFUNM(pack,string,(ret (*)())FFN(fname),F_ARGD(min,max,flags,ARGTYPES(ret0a0,a12,a34,a56)));\
+}\
+STATD ret FFN(fname) args
+
 /* eg.
    A function taking from 2 to 8 args
    returning object the first args is object, the next 6 int, and last defaults to object.
@@ -135,6 +137,16 @@ void Mjoin(fname,_init) () {\
    MAKEFUN(pack,string,(ret (*)())FFN(fname),F_ARGD(min,max,flags,ARGTYPES(ret0a0,a12,a34,a56)));\
 }\
 LFD(old)(void) \
+{   Iinvoke_c_function_from_value_stack((object (*)())FFN(fname),F_ARGD(min,max,flags,ARGTYPES(ret0a0,a12,a34,a56))); \
+    return;} \
+STATD  ret FFN(fname) args
+
+#define DEFUNOM_NEW(string,ret,fname,pack,min,max, flags, ret0a0,a12,a34,a56,oldret,old,args,doc) \
+STATD  ret FFN(fname) args; \
+void Mjoin(fname,_init) () {\
+   MAKEFUNM(pack,string,(ret (*)())FFN(fname),F_ARGD(min,max,flags,ARGTYPES(ret0a0,a12,a34,a56)));\
+}\
+LFD(old)(void) \
 {   Iinvoke_c_function_from_value_stack((object (*)())FFN(fname),F_ARGD(min,max,flags,ARGTYPES(ret0a0,a12,a34,a56))); \
     return;} \
 STATD  ret FFN(fname) args
@@ -166,7 +178,7 @@ TS_MEMBER(t0,TS(t1)|TS(t2)|TS(t3)...)
 #define TS(s) (1<<s)
 #define TS_MEMBER(t1,ts) ((TS(t1)) & (ts))
 
-#define ASSURE_TYPE(val,t) if(type_of(val)!=t) val= Icheck_one_type(val,t)
+#define ASSURE_TYPE(val,t) if (type_of(val)!=t) TYPE_ERROR(val,type_name(t))
 
 object IisArray();
 
@@ -219,7 +231,7 @@ EXTER  bool left_trim;
 EXTER bool right_trim;
 int  (*casefun)();
 
-#define	Q_SIZE		128
+#define	Q_SIZE		256
 #define IS_SIZE		256
 
 struct printStruct {
@@ -251,12 +263,10 @@ EXTER struct printStruct *printStructBufp;
 */
 #ifndef NULL_OR_ON_C_STACK
 
-
-#if (CSTACK_ADDRESS > 0)
-#define NULL_OR_ON_C_STACK(x) ((x)==0 || ((unsigned long)x) > (unsigned long)(pagetochar(MAXPAGE+1)))
-#else
-#define NULL_OR_ON_C_STACK(x) ((long)x <= 0)     
-#endif
+#define NULL_OR_ON_C_STACK(x) ({\
+      /* if ((void *)(x)<data_start && ((void *)(x)!=NULL) && ((object)(x))!=Cnil && ((object)(x))!=Ct) */ \
+      /* {pp(x);printf("%p %p\n",(void *)(x),data_start);}			*/ \
+      ((((void *)(x))<(void *)data_start || ((void *)(x))>=(void *)core_end));})
 
 #endif /* NULL_OR_ON_C_STACK */
 
@@ -276,12 +286,99 @@ gcl_init_cmp_anon(void);
 #ifdef SGC
 #define SAFE_READ(a_,b_,c_) \
    ({int _a=(a_),_c=(c_);char *_b=(b_);extern int sgc_enabled;\
-     if (sgc_enabled) memset(_b,0,_c);read(_a,_b,_c);})
+     if (sgc_enabled) memset(_b,0,_c); \
+     read(_a,_b,_c);})
 #define SAFE_FREAD(a_,b_,c_,d_) \
    ({int _b=(b_),_c=(c_);char *_a=(a_);FILE *_d=(d_);extern int sgc_enabled; \
-     if (sgc_enabled) memset(_a,0,_b*_c);fread(_a,_b,_c,_d);})
+     if (sgc_enabled) memset(_a,0,_b*_c); \
+     fread(_a,_b,_c,_d);})
 #else
 #define SAFE_READ(a_,b_,c_) read((a_),(b_),(c_))
 #define SAFE_FREAD(a_,b_,c_,d_) fread((a_),(b_),(c_),(d_))
 #endif
 
+#include "gmp_wrappers.h"
+
+char FN1[PATH_MAX],FN2[PATH_MAX],FN3[PATH_MAX],FN4[PATH_MAX],FN5[PATH_MAX];
+
+#define coerce_to_filename(a_,b_) coerce_to_filename1(a_,b_,sizeof(b_))
+
+#include <errno.h>
+#define massert(a_) ({errno=0;if (!(a_)) assert_error(#a_,__LINE__,__FILE__,__FUNCTION__);})
+
+extern bool writable_malloc;
+#define writable_malloc_wrap(f_,rt_,a_...) ({rt_ v;bool w=writable_malloc;writable_malloc=1;v=f_(a_);writable_malloc=w;v;})
+#define fopen(a_,b_) writable_malloc_wrap(fopen,FILE *,a_,b_)
+
+#define Mcar(x)	(x)->c.c_car
+#define Mcdr(x)	(x)->c.c_cdr
+#define Mcaar(x)	(x)->c.c_car->c.c_car
+#define Mcadr(x)	(x)->c.c_cdr->c.c_car
+#define Mcdar(x)	(x)->c.c_car->c.c_cdr
+#define Mcddr(x)	(x)->c.c_cdr->c.c_cdr
+#define Mcaaar(x)	(x)->c.c_car->c.c_car->c.c_car
+#define Mcaadr(x)	(x)->c.c_cdr->c.c_car->c.c_car
+#define Mcadar(x)	(x)->c.c_car->c.c_cdr->c.c_car
+#define Mcaddr(x)	(x)->c.c_cdr->c.c_cdr->c.c_car
+#define Mcdaar(x)	(x)->c.c_car->c.c_car->c.c_cdr
+#define Mcdadr(x)	(x)->c.c_cdr->c.c_car->c.c_cdr
+#define Mcddar(x)	(x)->c.c_car->c.c_cdr->c.c_cdr
+#define Mcdddr(x)	(x)->c.c_cdr->c.c_cdr->c.c_cdr
+#define Mcaaaar(x)	(x)->c.c_car->c.c_car->c.c_car->c.c_car
+#define Mcaaadr(x)	(x)->c.c_cdr->c.c_car->c.c_car->c.c_car
+#define Mcaadar(x)	(x)->c.c_car->c.c_cdr->c.c_car->c.c_car
+#define Mcaaddr(x)	(x)->c.c_cdr->c.c_cdr->c.c_car->c.c_car
+#define Mcadaar(x)	(x)->c.c_car->c.c_car->c.c_cdr->c.c_car
+#define Mcadadr(x)	(x)->c.c_cdr->c.c_car->c.c_cdr->c.c_car
+#define Mcaddar(x)	(x)->c.c_car->c.c_cdr->c.c_cdr->c.c_car
+#define Mcadddr(x)	(x)->c.c_cdr->c.c_cdr->c.c_cdr->c.c_car
+#define Mcdaaar(x)	(x)->c.c_car->c.c_car->c.c_car->c.c_cdr
+#define Mcdaadr(x)	(x)->c.c_cdr->c.c_car->c.c_car->c.c_cdr
+#define Mcdadar(x)	(x)->c.c_car->c.c_cdr->c.c_car->c.c_cdr
+#define Mcdaddr(x)	(x)->c.c_cdr->c.c_cdr->c.c_car->c.c_cdr
+#define Mcddaar(x)	(x)->c.c_car->c.c_car->c.c_cdr->c.c_cdr
+#define Mcddadr(x)	(x)->c.c_cdr->c.c_car->c.c_cdr->c.c_cdr
+#define Mcdddar(x)	(x)->c.c_car->c.c_cdr->c.c_cdr->c.c_cdr
+#define Mcddddr(x)	(x)->c.c_cdr->c.c_cdr->c.c_cdr->c.c_cdr
+
+#define gethash_with_check(a_,b_) \
+  ({object _b=(b_);while (type_of(_b)!=t_hashtable) _b=wrong_type_argument(sLhash_table,_b);gethash(a_,_b);})
+#define sethash_with_check(a_,b_,c_) \
+  ({object _b=(b_);while (type_of(_b)!=t_hashtable) _b=wrong_type_argument(sLhash_table,_b);sethash(a_,_b,c_);})
+
+#include "prelink.h"
+
+#include <signal.h>
+
+#ifdef GCL_GPROF
+#define prof_block(x) ({\
+      sigset_t prof,old;						\
+      int r;								\
+      sigemptyset(&prof);						\
+      sigaddset(&prof,SIGPROF);						\
+      sigprocmask(SIG_BLOCK,&prof,&old);				\
+      r=x;								\
+      sigprocmask(SIG_SETMASK,&old,NULL);				\
+      r;})
+#else
+#define prof_block(x) x
+#endif
+
+#define psystem(x) prof_block(vsystem(x))
+#define pfork() prof_block(fork())
+#define pvfork() prof_block(vfork())
+
+#include "error.h"
+
+#if __GNU_MP_VERSION > 4 || (__GNU_MP_VERSION == 4 && __GNU_MP_VERSION_MINOR >= 2)
+extern void __gmp_randget_mt ();
+extern void __gmp_randclear_mt ();
+extern void __gmp_randiset_mt ();
+
+typedef struct {void *a,*b,*c,*d;} gmp_randfnptr_t;
+EXTER gmp_randfnptr_t Mersenne_Twister_Generator_Noseed;
+#endif
+
+#define collect(p_,f_) (p_)=&(*(p_)=(f_))->c.c_cdr
+#define READ_STREAM_OR_FASD(strm_) \
+  type_of(strm_)==t_stream ? read_object_non_recursive(strm_) : fSread_fasd_top(strm_)
