@@ -12,10 +12,10 @@ there are tab.n_symbols pairs occurring.
 #endif
 
 
-typedef unsigned int addr;
+typedef unsigned long addr;
 
 struct node{
-  char *string;
+  const char *string;
   addr address;
 #ifdef AIX3
   unsigned short tc_offset;
@@ -26,8 +26,8 @@ struct lsymbol_table{
   unsigned int n_symbols ;
   unsigned int tot_leng;};
 
-#define SYM_ADDRESS(table,i) ((*(table).ptable))[i].address
-#define SYM_STRING(table,i) ((*(table).ptable))[i].string
+#define SYM_ADDRESS(table,i) table.ptable[i].address
+#define SYM_STRING(table,i)  table.ptable[i].string
 #define SYM_TC_OFF(table,i) ((*(table).ptable))[i].tc_offset
 
 /* typedef struct node *TABL;  */
@@ -35,11 +35,11 @@ struct lsymbol_table{
 
 typedef struct node TABL[]; 
 
-struct node * find_sym_ptable();
-
 struct  string_address_table
-{ TABL *ptable;
+{ struct node *ptable;
   unsigned int length;
+  struct node *local_ptable;
+  unsigned int local_length;
   unsigned int alloc_length;
 };
 
@@ -50,7 +50,7 @@ struct  string_address_table
 #ifdef SPECIAL_RSYM
 struct string_address_table c_table;
 #else
-static struct bfd_link_info link_info;
+struct bfd_link_info link_info;
 #endif
 struct string_address_table combined_table;
 
