@@ -24,6 +24,9 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 	cmpaux.c
 */
 
+#include <sys/types.h>
+#include <sys/stat.h>
+     
 #include <string.h>
 #include <signal.h>
 #include <stdlib.h>
@@ -31,6 +34,19 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "include.h"
 #define dcheck_type(a,b) check_type(a,b)
 
+#include "page.h"
+
+#ifdef HAVE_AOUT
+#undef ATT
+#undef BSD
+#ifndef HAVE_ELF
+#ifndef HAVE_FILEHDR
+#define BSD
+#endif
+#endif
+#include HAVE_AOUT
+#endif
+
 DEFUNO_NEW("SPECIALP",object,fSspecialp,SI
        ,1,1,NONE,OO,OO,OO,OO,void,siLspecialp,(object sym),"")
 {
@@ -43,7 +59,7 @@ DEFUNO_NEW("SPECIALP",object,fSspecialp,SI
 	RETURN1(sym);
 }
 
-DEF_ORDINARY("DEBUG",sSdebug,SI,"");
+DEF_ORDINARY("DEBUGGER",sSdebugger,SI,"");
 
 DEFUN_NEW("DEFVAR1",object,fSdefvar1,SI
        ,2,3,NONE,OO,OO,OO,OO,(object sym,object val,...),"")
@@ -66,10 +82,10 @@ DEFUN_NEW("DEFVAR1",object,fSdefvar1,SI
       }
 
 
-DEFUN_NEW("DEBUG",object,fSdebug,SI
+DEFUN_NEW("DEBUG",object,fLdebug,LISP
        ,2,2,NONE,OO,OO,OO,OO,(object sym,object val),"")
 { /* 2 args */
-  putprop(sym,val,sSdebug);
+  putprop(sym,val,sSdebugger);
   RETURN1(sym);
 }
 
@@ -202,6 +218,31 @@ object_to_int(object x)
 	return(i);
 }
 
+fixnum
+object_to_fixnum(object x)
+{
+	fixnum i=0;
+
+	switch (type_of(x)) {
+	case t_character:
+	  i = char_code(x);  break;
+	case t_fixnum:
+	  i = fix(x);  break;
+	case t_bignum:
+	  i = number_to_double(x);
+	  break;
+	case t_ratio:
+	  i = number_to_double(x);  break;
+	case t_shortfloat:
+	  i = sf(x);  break;
+	case t_longfloat:
+	  i = lf(x);  break;
+	default:
+	  FEerror("~S cannot be coerce to a C int.", 1, x);
+	}
+	return(i);
+}
+
 float 
 object_to_float(object x) 
 { 
@@ -253,26 +294,28 @@ object_to_double(object x)
    have a null character in the fillpointer position. */
 
 char *
-object_to_string(object x)
-{ unsigned int leng;
+object_to_string(object x) { 
+
+  unsigned int leng;
+  char *res;
+
   if (type_of(x)!=t_string) FEwrong_type_argument(sLstring,x);
   leng= x->st.st_fillp;
   /* user has thoughtfully provided a null terminated string ! */
-    if (leng > 0 && leng < x->st.st_dim && x->st.st_self[leng]==0)
+  if (leng > 0 && leng < x->st.st_dim && x->st.st_self[leng]==0)
     return x->st.st_self;
-  if (x->st.st_dim == leng
-      && ( leng % sizeof(object))
-     )
-    { x->st.st_self[leng] = 0;
-      return x->st.st_self;
-    }
-  else
-    {char *res=malloc(leng+1);
-     bcopy(x->st.st_self,res,leng);
-     res[leng]=0;
-     return res;
-   }}
 
+  if (x->st.st_dim == leng && leng % sizeof(object) && MAYBE_DATA_P(x->st.st_self)) {
+    x->st.st_self[leng] = 0;
+    return x->st.st_self;
+  }  
+  
+  res=malloc(leng+1);
+  bcopy(x->st.st_self,res,leng);
+  res[leng]=0;
+  return res;
+
+}
 
 /*  typedef int (*FUNC)(); */
 
@@ -292,64 +335,19 @@ object_to_string(object x)
 /* } */
 /* #endif */
 
+
 void
-call_init(int init_address, object memory, object fasl_vec, FUNC fptr)
-{object form;
- FUNC at;
-/* #ifdef CLEAR_CACHE */
-/*  static int n; */
-/*  static sigset_t ss; */
+call_init(int init_address,object memory,object faslfile) {
 
-/*  if (!n) { */
-/*      struct sigaction sa={{(void *)sigh},{{0}},SA_RESTART|SA_SIGINFO,NULL}; */
+  bds_bind(sSPmemory,memory);
+  bds_bind(sSPinit,faslfile);
+  ((FUNC)(memory->cfd.cfd_start+init_address))();
+  bds_unwind1;
+  bds_unwind1;
 
-/*      sigaction(SIGILL,&sa,NULL); */
-/*      sigemptyset(&ss); */
-/*      sigaddset(&ss,SIGILL); */
-/*      sigprocmask(SIG_BLOCK,&ss,NULL); */
-/*      n=1; */
-/*  } */
-/* #endif */
+}
 
 
-  check_type(fasl_vec,t_vector);
-  form=(fasl_vec->v.v_self[fasl_vec->v.v_fillp -1]);
-
- if (fptr) at = fptr;
-  else 
- at=(FUNC)(memory->cfd.cfd_start+ init_address );
- 
-#ifdef VERIFY_INIT
- VERIFY_INIT
-#endif
-   
- if (type_of(form)==t_cons &&
-     form->c.c_car == sSPinit)
-   {bds_bind(sSPinit,fasl_vec);
-    bds_bind(sSPmemory,memory);
-/* #ifdef CLEAR_CACHE */
-/*     sigprocmask(SIG_UNBLOCK,&ss,NULL); */
-/* #endif */
-    (*at)();
-/* #ifdef CLEAR_CACHE */
-/*     sigprocmask(SIG_BLOCK,&ss,NULL); */
-/* #endif */
-    bds_unwind1;
-    bds_unwind1;
-  }
- else
-   /* old style three arg init, with all init being done by C code. */
-   {memory->cfd.cfd_self = fasl_vec->v.v_self;
-    memory->cfd.cfd_fillp = fasl_vec->v.v_fillp;
-/* #ifdef CLEAR_CACHE */
-/*     sigprocmask(SIG_UNBLOCK,&ss,NULL); */
-/* #endif */
-    (*at)(memory->cfd.cfd_start, memory->cfd.cfd_size, memory);
-/* #ifdef CLEAR_CACHE */
-/*     sigprocmask(SIG_BLOCK,&ss,NULL); */
-/* #endif */
-}}
-
 /* statVV is the address of some static storage, which is used by the
    cfunctions to refer to global variables,..
    Initially it holds a number of addresses.   We also have sSPmemory->s.s_dbind
@@ -361,39 +359,50 @@ call_init(int init_address, object memory, object fasl_vec, FUNC fptr)
 
    */
 
-void
-do_init(object *statVV)
-{object fasl_vec=sSPinit->s.s_dbind;
- object data = sSPmemory->s.s_dbind;
- {object *p,*q,y;
-  int n=fasl_vec->v.v_fillp -1;
-  int i;
-  object form;
-  check_type(fasl_vec,t_vector);
-  form = fasl_vec->v.v_self[n];
-  dcheck_type(form,t_cons);  
+object *min_cfd_self=NULL;
 
+void
+do_init(object *statVV) {
+
+  object faslfile=sSPinit->s.s_dbind;
+  object data=sSPmemory->s.s_dbind;
+  object *p,*q,y;
+  int i,n;
+  object fasl_vec;
+  char ch;
+
+  ch=readc_stream(faslfile);
+  unreadc_stream(ch,faslfile);
+
+  if (ch!='\n') {
+    struct fasd * fd;
+    faslfile=fSopen_fasd(faslfile,sKinput,OBJNULL,Cnil);
+    fd=(struct fasd *)faslfile->v.v_self;
+    n=fix(fd->table_length);
+    fd->table->v.v_self=alloca(n*sizeof(object));
+    memset(fd->table->v.v_self,0,n*sizeof(object));
+    fd->table->v.v_dim=faslfile->v.v_self[1]->v.v_fillp=n;
+  }
+
+  n=fix(READ_STREAM_OR_FASD(faslfile));
+  sSPinit->s.s_dbind=fasl_vec=fSmake_vector1_1(n,aet_object,Cnil);
 
   /* switch SPinit to point to a vector of function addresses */
-     
-  fasl_vec->v.v_elttype = aet_fix;
-  fasl_vec->v.v_dim *= (sizeof(object)/sizeof(fixnum));
-  fasl_vec->v.v_fillp *= (sizeof(object)/sizeof(fixnum));
-  
-  /* swap the entries */
-  p = fasl_vec->v.v_self;
 
-  q = statVV;
-  for (i=0; i<=n ; i++)
-    {  y = *p;
-     *p++ = *q;
-     *q++ = y;
-     }
-  
+  fasl_vec->v.v_elttype = aet_fix;
+
+  /* swap the entries */
+  for (i=0,p=fasl_vec->v.v_self,q=statVV;i<n;i++) {
+    y=*p;
+    *p++=*q;
+    *q++=y;
+  }
+
   data->cfd.cfd_self = statVV;
-  data->cfd.cfd_fillp= n+1;
-  statVV[n] = data;
-  
+  if (!min_cfd_self || data->cfd.cfd_self<min_cfd_self)
+    min_cfd_self=data->cfd.cfd_self;
+  data->cfd.cfd_fillp= n;
+  statVV[n-1] = data;
 
   /* So now the fasl_vec is a fixnum array, containing random addresses of c
      functions and other stuff from the compiled code.
@@ -401,16 +410,20 @@ do_init(object *statVV)
   */
   /* Now we can run the forms f1 f2 in form= (%init f1 f2 ...) */
 
-  form=form->c.c_cdr;
-  {object *top=vs_top;
-   
-   for(i=0 ; i< form->v.v_fillp; i++)
-     { 
-       eval(form->v.v_self[i]);
-       vs_top=top;
-     }
- }
-}}
+  fSload_stream(faslfile,Cnil);
+  if (type_of(faslfile)!=t_stream)
+    fSclose_fasd(faslfile);
+
+}
+
+DEFUN_NEW("MARK-MEMORY-AS-PROFILING",object,fSmark_memory_as_profiling,SI,0,0,
+	  NONE,OO,OO,OO,OO,(void),"") {
+
+  sSPmemory->s.s_dbind->cfd.cfd_prof=1;
+
+  return Cnil;
+
+}
 
 #ifdef DOS
 #define PATH_LIM 8
@@ -435,25 +448,45 @@ char *s;
 	
 #endif
 
-void
-gcl_init_or_load1(void (*fn)(void),char *file)
-{int n=strlen(file);
- if (file[n-1]=='o')
-   { object memory;
-     object fasl_data;
-     file=FIX_PATH_STRING(file);
+object
+new_cfdata(void) {
 
-     memory=alloc_object(t_cfdata);
-     memory->cfd.cfd_self=0;
-     memory->cfd.cfd_fillp=0;
-     memory->cfd.cfd_size = 0;
-     printf("Initializing %s\n",file); fflush(stdout);
-     fasl_data = read_fasl_data(file);
-     memory->cfd.cfd_start= (char *)fn;
-     call_init(0,memory,fasl_data,0);
+  object memory=alloc_object(t_cfdata);
+
+  memory->cfd.cfd_size=0;
+  memory->cfd.cfd_fillp=0;
+  memory->cfd.cfd_prof=0;
+  memory->cfd.cfd_self=0;
+  memory->cfd.cfd_start=0;
+
+  return memory;
+
+}
+
+
+void
+gcl_init_or_load1(void (*fn)(void),const char *file) {
+
+  if (file[strlen(file)-1]=='o') {
+
+    object memory;
+    object faslfile;
+    file=FIX_PATH_STRING(file);
+    
+    memory=new_cfdata();
+    memory->cfd.cfd_start= (char *)fn;
+    printf("Initializing %s\n",file); fflush(stdout);
+    faslfile=open_stream(make_simple_string(file),smm_input,Cnil,sKerror);
+    SEEK_TO_END_OFILE(faslfile->sm.sm_fp);
+    call_init(0,memory,faslfile);
+    close_stream(faslfile);
+
+  } else {
+    printf("loading %s\n",file); 
+    fflush(stdout);  
+    load(file);
   }
- else
-  {printf("loading %s\n",file); fflush(stdout);  load(file);}
+
 }
 
 DEFUN_NEW("INIT-CMP-ANON", object, fSinit_cmp_anon, SI, 0, 0,
@@ -480,3 +513,67 @@ function.")
   return i ? Cnil : Ct;
 
 }
+
+object
+find_init_name1(char *s,unsigned len) {
+
+#ifdef _WIN32
+
+  char *tmp;
+
+  if (len) {
+    tmp=alloca(len+1);
+    memcpy(tmp,s,len);
+    tmp[len]=0;
+  } else
+    tmp=s;
+
+  return find_init_string(tmp);
+
+#else    
+/* These functions have no relevance on Windows
+ * as dlopen and friends don't exist in that part of Cyberspace. */
+
+  struct stat ss;
+  char *tmp,*q;
+  FILE *f;
+
+  if (len) {
+    tmp=alloca(len+1);
+    memcpy(tmp,s,len);
+    tmp[len]=0;
+  } else
+    tmp=s;
+  if (stat(tmp,&ss))
+    FEerror("File ~a does not exist",1,make_simple_string(tmp));
+  if (!(f=fopen(tmp,"rb")))
+    FEerror("Cannot open ~a for binary reading",1,make_simple_string(tmp));
+  tmp=alloca(ss.st_size+1);
+  if (fread(tmp,1,ss.st_size,f)!=ss.st_size)
+    FEerror("Error reading binary file",0);
+  fclose(f);
+  for (s=tmp+1;s<tmp+ss.st_size
+	 && (strncmp(s,"init_",5) || (s>tmp && (s[-1]=='_' ? (s>tmp+1 && s[-2]) : s[-1])));
+       q=strstr(s+1,"init_"),s=q ? q : s+strlen(s)+1);
+  if (strncmp(s,"init_",5))
+    FEerror("Init name not found",0);
+  return make_simple_string(s);
+#endif  /* _WIN32 */
+
+}
+ 
+
+DEFUN_NEW("FIND-INIT-NAME", object, fSfind_init_name, SI, 1, 1,
+	  NONE, OO, OO, OO,OO,(object namestring),"") 
+{
+
+  check_type_string(&namestring);
+  return find_init_name1(namestring->st.st_self,namestring->st.st_dim);
+
+}
+
+DEFUN_NEW("SEEK-TO-END-OFILE",object,fSseek_to_end_ofile,SI,1,1,NONE,OO,OO,OO,OO,(object sm),"") {
+  check_type_stream(&sm);
+  SEEK_TO_END_OFILE(sm->sm.sm_fp);
+  RETURN1(sm);
+}
