@@ -55,87 +55,59 @@ warn_avma()
 object sSAbreak_pointsA;
 object sSAbreak_stepA;
 
-
-/* This is a temporary workaround.  m68k cannot find the result 
-   of a function returning long when invoked via a function pointer
-   declared as a function returning a pointer, in this case, an 
-   object.  A proper fix will require rewriting sections of the lisp
-   compiler to separate the calling procedures for functions returning
-   an object from functions returning a long.  CM  20020801 */
-/*  #if defined(__mc68020__) */
-/*  #define LCAST(a) (object)(*(long(*)())a) */
-/*  #else */
-#define LCAST(a) (*a)
-/*  #endif */
-
-#define SET_TO_APPLY(res,f,n,x) \
- switch(n) {\
- case 0:   res=LCAST(f)(); break;\
-  case 1:  res=LCAST(f)(x[0]); break; \
-  case 2:  res=LCAST(f)(x[0],x[1]);break; \
-  case 3:  res=LCAST(f)(x[0],x[1],x[2]);break; \
-  case 4:  res=LCAST(f)(x[0],x[1],x[2],x[3]);break; \
-  case 5:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4]);break; \
-  case 6:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5]);  break;\
-  case 7:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5], x[6]); break;\
-  case 8:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5], x[6],x[7]); break;\
-  case 9:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5],x[6],x[7],x[8]);break;\
-  case 10: res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5],x[6],x[7],x[8],x[9]);break;\
-   default: res=c_apply_n(*f,n,x); break;}
-
-/*
-#undef SET_TO_APPLY
-#define SET_TO_APPLY(res,f,n,x)  res=c_apply_n(f,n,x);
-*/
+#include "apply_n.h"
 
 /* for t_sfun,t_gfun with args on vs stack */
 
 static void
-quick_call_sfun(object fun)
-{ DEBUG_AVMA
+quick_call_sfun(object fun) {
+
+  DEBUG_AVMA
   int i=fun->sfn.sfn_argd,n=SFUN_NARGS(i);
   enum ftype restype;
-  object *x,res,*base;
-  object *temp_ar=alloca(n*sizeof(object));
-/*   i=fun->sfn.sfn_argd; */
-/*   n=SFUN_NARGS(i); */
-  base = vs_base;
-  if (n != vs_top - base)
-    {check_arg_failed(n);}
+  object *x,*base;
+
+  if (n!=vs_top-vs_base)
+    check_arg_failed(n);
+
   restype = SFUN_RETURN_TYPE(i);
   SFUN_START_ARG_TYPES(i);
-  /* for moment just support object and int */
 #define COERCE_ARG(a,type)  (type==f_object ? a : (object)(fix(a)))
-  if (i==0)
-    x=vs_base;
-  else
-    {int j;
-     x=temp_ar;
-     for (j=0; j<n ; j++)
-       {enum ftype typ=SFUN_NEXT_TYPE(i);
-	x[j]=COERCE_ARG(vs_base[j],typ);}}
-  SET_TO_APPLY(res,fun->sfn.sfn_self,n,x);
-  base[0]=
-    (restype==f_object ?  res :
-     restype==f_fixnum ? make_fixnum((long)res)
-     :(object) (FEerror("Bad result type",0),Cnil));
-  vs_base = base;
-  vs_top=base+1;
-  CHECK_AVMA;
-  return;}
 
-/* only for sfun not gfun !!  Does not check number of args */
-static void
-call_sfun_no_check(object fun)
-{ DEBUG_AVMA
-  int n;
-  object *base=vs_base;
-  n=vs_top - base;
-  SET_TO_APPLY(base[0],fun->sfn.sfn_self,n,base);
+  x=vs_base;
+  if (i) {
+    int j;
+    x=alloca(n*sizeof(object));
+    for (j=0;j<n;j++) {
+      enum ftype typ=SFUN_NEXT_TYPE(i);
+      x[j]=COERCE_ARG(vs_base[j],typ);
+    }
+  }
+
+  base=vs_base;
+  *base=c_apply_n_fun(fun,n,x);
+  if (restype==f_fixnum)
+    *base=make_fixnum((fixnum)*base);
+
   vs_top=(vs_base=base)+1;
+
   CHECK_AVMA;
   return;
+
 }
+
+/* /\* only for sfun not gfun !!  Does not check number of args *\/ */
+/* static void */
+/* call_sfun_no_check(object fun) */
+/* { DEBUG_AVMA */
+/*   int n; */
+/*   object *base=vs_base; */
+/*   n=vs_top - base; */
+/*   base[0]=c_apply_n_fun(fun,n,base); */
+/*   vs_top=(vs_base=base)+1; */
+/*   CHECK_AVMA; */
+/*   return; */
+/* } */
 static void
 call_vfun(object fun)
 { DEBUG_AVMA
@@ -147,7 +119,7 @@ call_vfun(object fun)
   if (n > fun->vfn.vfn_maxargs)
     {FEtoo_many_arguments(base,vs_top); return;}
   VFUN_NARGS = n;
-  SET_TO_APPLY(base[0],fun->sfn.sfn_self,n,base);
+  base[0]=c_apply_n_fun(fun,n,base);
   vs_top=(vs_base=base)+1;
   CHECK_AVMA;
   return;
@@ -159,7 +131,7 @@ funcall(object fun)
 { 
         object temporary;
 	object x;
-	 object * VOL top;
+        object * VOL top;
 	object *lex;
 	bds_ptr old_bds_top;
 	VOL bool b;
@@ -174,15 +146,23 @@ funcall(object fun)
 		CHECK_AVMA; return;
 	case t_gfun:	
 	case t_sfun:
-		ihs_check;ihs_push(fun);
-		quick_call_sfun(fun);
-		ihs_pop();
-		return;
+	  { 
+	    extern int Rset;
+	    int rset=Rset;
+	    if (!rset) {ihs_check;ihs_push(fun);}
+	    quick_call_sfun(fun);
+	    if (!rset) ihs_pop();
+	  }
+	  return;
         case t_vfun:
-		ihs_check;ihs_push(fun);
-		call_vfun(fun);
-		ihs_pop();
-		return;
+	  { 
+	    extern int Rset;
+	    int rset=Rset;
+	    if (!rset) {ihs_check;ihs_push(fun);}
+	    call_vfun(fun);
+	    if (!rset) ihs_pop();
+	  }
+	  return;
          case t_afun:
 	 case t_closure:
 	   { object res,*b = vs_base;
@@ -194,26 +174,13 @@ funcall(object fun)
 	     while (--n> 0 ) b[n] = fcall.values[n];
 	     b[0] = res;
 	     return;}		
-      case t_cclosure:
 
-	{
-		object *top, *base, l;
+        case t_cclosure:
+	
+	  MMccall(fun);
+	  CHECK_AVMA;
+	  return;
 
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			CHECK_AVMA; return;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		CHECK_AVMA; return;
-	}
-		
 	case t_symbol:
 	     {object x = fun->s.s_gfdef;
 	      if (x) { fun = x; goto TOP;}
@@ -263,24 +230,24 @@ funcall(object fun)
 	  c = FALSE;
 	  fun = fun->c.c_cdr;
 
-	}else if (x == sLlambda_block) {
+	}else if (x == sSlambda_block) {
 	  b = TRUE;
 	  c = FALSE;
-	  if(sSlambda_block_expanded->s.s_dbind)
+	  if(sSlambda_block_expanded->s.s_dbind!=OBJNULL)
 	    fun = ifuncall1(sSlambda_block_expanded->s.s_dbind,fun);
 
 	  fun = fun->c.c_cdr;
 
 
 	
-	} else if (x == sLlambda_closure) {
+	} else if (x == sSlambda_closure) {
 		b = FALSE;
 		c = TRUE;
 		fun = fun->c.c_cdr;
 	} else if (x == sLlambda) {
 		b = c = FALSE;
 		fun = fun->c.c_cdr;
-	} else if (x == sLlambda_block_closure) {
+	} else if (x == sSlambda_block_closure) {
 		b = c = TRUE;
 		fun = fun->c.c_cdr;
 	} else
@@ -352,23 +319,8 @@ funcall_no_event(object fun)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		(*fun->cc.cc_self)(top);
-		break;
-	}
+	  (*fun->cc.cc_self)(fun);
+	  break;
 
 	case t_sfun:
 /*  		call_sfun_no_check(fun); return; */
@@ -400,26 +352,11 @@ lispcall(object *funp, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
+	  MMccall(fun);
+	  break;
 
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
-
-	      default:
-		funcall(fun);
+	default:
+	  funcall(fun);
 
 	}
   CHECK_AVMA;
@@ -442,27 +379,12 @@ lispcall_no_event(object *funp, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		(*fun->cc.cc_self)(top);
-		break;
-	}
 
+	  (*fun->cc.cc_self)(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
 
 	}
 	 CHECK_AVMA;
@@ -485,23 +407,8 @@ symlispcall(object sym, object *base, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
+	  MMccall(fun);
+	  break;
 
 	default:
 		funcall(fun);
@@ -526,26 +433,11 @@ symlispcall_no_event(object sym, object *base, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		(*fun->cc.cc_self)(top);
-		break;
-	}
+	  (*fun->cc.cc_self)(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
 
 	}
 	CHECK_AVMA;
@@ -569,26 +461,12 @@ simple_lispcall(object *funp, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
+	  MMccall(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
+
 	}
 	vs_top = sup;
 	CHECK_AVMA;
@@ -659,26 +537,11 @@ simple_symlispcall(object sym, object *base, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
+	  MMccall(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
 
 	}
 	vs_top = sup;
@@ -744,27 +607,34 @@ super_funcall(object fun)
 }
 
 void
-super_funcall_no_event(object fun)
-{
+super_funcall_no_event(object fun) {
+
 #ifdef DEBUGGING_AVMA
   funcall_no_event(fun); return;
 #endif 
-   if (type_of(fun)==t_cfun){(*fun->cf.cf_self)();return;}
-   if (type_of(fun)==t_sfun){call_sfun_no_check(fun); return;}
-   if (type_of(fun)==t_gfun)
-       {quick_call_sfun(fun); return;}
-   if (type_of(fun)==t_vfun)
-       {call_vfun(fun); return;}
-   if (type_of(fun) == t_symbol) {
-	  if (fun->s.s_sfdef != NOT_SPECIAL || fun->s.s_mflag)
-			FEinvalid_function(fun);
-		if (fun->s.s_gfdef == OBJNULL)
-			FEundefined_function(fun);
-		fun = fun->s.s_gfdef;
-		if (type_of(fun)==t_cfun){(*fun->cf.cf_self)();
-					  return;}
-	}
-	funcall_no_event(fun);
+
+  switch(type_of(fun)) {
+  case t_cfun:
+    (*fun->cf.cf_self)(); return;
+  case t_cclosure:
+    (*fun->cc.cc_self)(fun); return;
+  case t_sfun:
+    /* call_sfun_no_check(fun); return; */
+  case t_gfun:
+    quick_call_sfun(fun); return;
+  case t_vfun:
+    call_vfun(fun); return;
+  case t_symbol:
+    if (fun->s.s_sfdef != NOT_SPECIAL || fun->s.s_mflag)
+      FEinvalid_function(fun);
+    if (fun->s.s_gfdef == OBJNULL)
+      FEundefined_function(fun);
+    super_funcall_no_event(fun->s.s_gfdef);
+    return;
+  default:
+    funcall(fun);
+  }
+
 }
 
 #ifdef USE_BROKEN_IEVAL
@@ -784,13 +654,13 @@ EVAL:
 
 	vs_check;
 
-	if (Vevalhook->s.s_dbind != Cnil && eval1 == 0)
+	if (siVevalhook->s.s_dbind != Cnil && eval1 == 0)
 	{
 		bds_ptr old_bds_top = bds_top;
-		object hookfun = symbol_value(Vevalhook);
+		object hookfun = symbol_value(siVevalhook);
 		/*  check if Vevalhook is unbound  */
 
-		bds_bind(Vevalhook, Cnil);
+		bds_bind(siVevalhook, Cnil);
 		form = Ifuncall_n(hookfun,2,form,list(3,lex_env[0],lex_env[1],lex_env[2]));
 		bds_unwind(old_bds_top);
 		return form;
@@ -861,7 +731,7 @@ APPLICATION:
 	for (x = lex_env[1];  type_of(x) == t_cons;  x = x->c.c_cdr)
 		if (x->c.c_car->c.c_car == fun) {
 			x = x->c.c_car;
-			if (MMcadr(x) == sLmacro) {
+			if (MMcadr(x) == sSmacro) {
 				x = MMcaddr(x);
 				goto EVAL_MACRO;
 			}
@@ -895,10 +765,10 @@ EVAL_ARGS:
 	  vs_top = ++top;
 	  form = MMcdr(form);}
 	  n =top - base; /* number of args */
-	if (Vapplyhook->s.s_dbind != Cnil) {
+	if (siVapplyhook->s.s_dbind != Cnil) {
 	  base[0]= (object)n;
 	  base[0] = c_apply_n(list,n+1,base);
-	  x = Ifuncall_n(Vapplyhook->s.s_dbind,3,
+	  x = Ifuncall_n(siVapplyhook->s.s_dbind,3,
 			 x, /* the function */
 			 base[0], /* the arg list */
 			 list(3,lex_env[0],lex_env[1],lex_env[2]));
@@ -915,7 +785,7 @@ EVAL_ARGS:
 
 LAMBDA:
 	if (type_of(fun) == t_cons && MMcar(fun) == sLlambda) {
-	  x = listA(4,sLlambda_closure,lex_env[0],lex_env[1],lex_env[2],Mcdr(fun));
+	  x = listA(4,sSlambda_closure,lex_env[0],lex_env[1],lex_env[2],Mcdr(fun));
 	  goto EVAL_ARGS;
 	}
 	FEinvalid_function(fun);
@@ -945,22 +815,16 @@ EVAL:
 
 	vs_check;
 
-	if (Vevalhook->s.s_dbind != Cnil && eval1 == 0)
+	if (siVevalhook->s.s_dbind != Cnil && eval1 == 0)
 	{
 		bds_ptr old_bds_top = bds_top;
-		object hookfun = symbol_value(Vevalhook);
-		/*  check if Vevalhook is unbound  */
+		object hookfun = symbol_value(siVevalhook);
+		/*  check if siVevalhook is unbound  */
 
-		bds_bind(Vevalhook, Cnil);
+		bds_bind(siVevalhook, Cnil);
 		vs_base = vs_top;
 		vs_push(form);
-		vs_push(lex_env[0]);
-		vs_push(lex_env[1]);
-		vs_push(lex_env[2]);
-		vs_push(Cnil);
-		stack_cons();
-		stack_cons();
-		stack_cons();
+		vs_push(list(3,lex_env[0],lex_env[1],lex_env[2]));
 		super_funcall(hookfun);
 		bds_unwind(old_bds_top);
 		return;
@@ -1043,7 +907,7 @@ APPLICATION:
 	for (x = lex_env[1];  type_of(x) == t_cons;  x = x->c.c_cdr)
 		if (x->c.c_car->c.c_car == fun) {
 			x = x->c.c_car;
-			if (MMcadr(x) == sLmacro) {
+			if (MMcadr(x) == sSmacro) {
 				x = MMcaddr(x);
 				goto EVAL_MACRO;
 			}
@@ -1080,7 +944,7 @@ EVAL_ARGS:
 		form = MMcdr(form);
 	}
 	vs_base = base;
-	if (Vapplyhook->s.s_dbind != Cnil) {
+	if (siVapplyhook->s.s_dbind != Cnil) {
 		call_applyhook(fun);
 		return;
 	}
@@ -1099,7 +963,7 @@ LAMBDA:
 		temporary = make_cons(lex_env[2], fun->c.c_cdr);
 		temporary = make_cons(lex_env[1], temporary);
 		temporary = make_cons(lex_env[0], temporary);
-		x = make_cons(sLlambda_closure, temporary);
+		x = make_cons(sSlambda_closure, temporary);
 		vs_push(x);
 		goto EVAL_ARGS;
 	}
@@ -1110,27 +974,17 @@ static void
 call_applyhook(object fun)
 {
 	object ah;
-	object *v;
 
-	ah = symbol_value(Vapplyhook);
-	v = vs_base + 1;
-	vs_push(Cnil);
-	while (vs_top > v)
-		stack_cons();
+	ah = symbol_value(siVapplyhook);
+	Llist();
 	vs_push(vs_base[0]);
 	vs_base[0] = fun;
-	vs_push(lex_env[0]);
-	vs_push(lex_env[1]);
-	vs_push(lex_env[2]);
-	vs_push(Cnil);
-	stack_cons();
-	stack_cons();
-	stack_cons();
+	vs_push(list(3,lex_env[0],lex_env[1],lex_env[2]));
 	super_funcall(ah);
 }
 
 
-DEFUNO_NEW("FUNCALL",object,fLfuncall,LISP
+DEFUNOM_NEW("FUNCALL",object,fLfuncall,LISP
        ,1,MAX_ARGS,NONE,OO,OO,OO,OO,void,Lfuncall,(object fun,...),"")
 { va_list ap;
   object *new;
@@ -1143,10 +997,10 @@ DEFUNO_NEW("FUNCALL",object,fLfuncall,LISP
 }
 
 
-DEFUNO_NEW("APPLY",object,fLapply,LISP
+DEFUNOM_NEW("APPLY",object,fLapply,LISP
        ,2,MAX_ARGS,NONE,OO,OO,OO,OO,void,Lapply,(object fun,...),"")
 {	int m,n=VFUN_NARGS;
-	object list;
+	object l;
 	object buf[MAX_ARGS];
 	object *base=buf;
 	va_list ap;
@@ -1156,18 +1010,18 @@ DEFUNO_NEW("APPLY",object,fLapply,LISP
 	  {*base++ = va_arg(ap,object);
 	 }
 	m = n-2;
-	list = va_arg(ap,object);
+	l = va_arg(ap,object);
 	va_end(ap);
-	while (!endp(list))
+	while (!endp(l))
 	  { if (m >= MAX_ARGS) FEerror(" Lisps arglist maximum surpassed",0);
-	    *base++ = Mcar(list);
-	    list = Mcdr(list);
+	    *base++ = Mcar(l);
+	    l = Mcdr(l);
 	    m++;}
 	return IapplyVector(fun,m,buf);
       }
 	
 
-DEFUNO_NEW("EVAL",object,fLeval,LISP
+DEFUNOM_NEW("EVAL",object,fLeval,LISP
        ,1,1,NONE,OO,OO,OO,OO,void,Leval,(object x0),"")
 {
 	object *lex = lex_env;
@@ -1180,7 +1034,7 @@ DEFUNO_NEW("EVAL",object,fLeval,LISP
 	return Ivs_values();
 }
 
-LFD(Levalhook)(void)
+LFD(siLevalhook)(void)
 {
 	object env;
 	bds_ptr old_bds_top = bds_top;
@@ -1202,15 +1056,15 @@ LFD(Levalhook)(void)
 		vs_push(car(env));
 	} else
 		too_many_arguments();
-	bds_bind(Vevalhook, vs_base[1]);
-	bds_bind(Vapplyhook, vs_base[2]);
+	bds_bind(siVevalhook, vs_base[1]);
+	bds_bind(siVapplyhook, vs_base[2]);
 	eval1 = 1;
 	eval(vs_base[0]);
 	lex_env = lex;
 	bds_unwind(old_bds_top);
 }
 
-LFD(Lapplyhook)(void)
+LFD(siLapplyhook)(void)
 {
 
 	object env;
@@ -1234,8 +1088,8 @@ LFD(Lapplyhook)(void)
 		vs_push(car(env));
 	} else
 		too_many_arguments();
-	bds_bind(Vevalhook, vs_base[2]);
-	bds_bind(Vapplyhook, vs_base[3]);
+	bds_bind(siVevalhook, vs_base[2]);
+	bds_bind(siVapplyhook, vs_base[3]);
 	z = vs_top;
 	for (l = vs_base[1];  !endp(l);  l = l->c.c_cdr)
 		vs_push(l->c.c_car);
@@ -1356,7 +1210,7 @@ static object
 fcalln_cclosure(object first,va_list ap)
 {
 int i=fcall.argd;
- {object *base=vs_top;
+ {object *base=vs_top,*old_base=base;
   DEBUG_AVMA
     vs_base=base;
   if (i) {
@@ -1378,12 +1232,13 @@ int i=fcall.argd;
     default:
       FEerror("bad args",0);
     } vs_top=base;
-      base=base -i;
+      base=old_base;
   	do{object fun=fcall.fun;
 		object *top, *base, l;
 
+		massert(fun->cc.cc_turbo);
 		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
+			(*fun->cc.cc_self)(fun);
 			break;
 		}
 		top = vs_top;
@@ -1447,7 +1302,7 @@ fcalln_vfun(object first,va_list vl)
 {object *new,res;
  DEBUG_AVMA
  COERCE_VA_LIST_NEW(new,first,vl,fcall.argd);
- res = c_apply_n(fcall.fun->vfn.vfn_self,fcall.argd,new);
+ res = c_apply_n_fun(fcall.fun,fcall.argd,new);
  CHECK_AVMA;
  return res;
 }
@@ -1459,7 +1314,7 @@ fcalln1(object first,...)
    DEBUG_AVMA
    va_start(ap,first);
    if(type_of(fun)==t_cfun)
-     {object *base=vs_top;
+     {object *base=vs_top,*old_base=base;
       int i=fcall.argd;
       vs_base=base;
       if (i) {
@@ -1481,7 +1336,7 @@ fcalln1(object first,...)
       default:
 	FEerror("bad args",0);
       }  vs_top=base;
-      base=base -i;
+      base=old_base;
       (*fcall.fun->cf.cf_self)();
       vs_top=base;
       CHECK_AVMA;
@@ -1531,15 +1386,15 @@ gcl_init_eval(void)
         make_constant("CALL-ARGUMENTS-LIMIT", make_fixnum(64));
 
 
-	Vevalhook = make_special("*EVALHOOK*", Cnil);
-	Vapplyhook = make_special("*APPLYHOOK*", Cnil);
+	siVevalhook = make_si_special("*EVALHOOK*", Cnil);
+	siVapplyhook = make_si_special("*APPLYHOOK*", Cnil);
 
 
 	three_nils.nil3_self[0] = Cnil;
 	three_nils.nil3_self[1] = Cnil;
 	three_nils.nil3_self[2] = Cnil;
 
-	make_function("EVALHOOK", Levalhook);
-	make_function("APPLYHOOK", Lapplyhook);
+	make_si_function("EVALHOOK", siLevalhook);
+	make_si_function("APPLYHOOK", siLapplyhook);
 
 }
