@@ -35,22 +35,6 @@ object make_pathname ();
 
 static int needs_patching;
 
-
-struct fasd {
-  object stream;   /* lisp object of type stream */
-  object table;  /* hash table used in dumping or vector on input*/
-  object eof;      /* lisp object to be returned on coming to eof mark */
-  object direction;    /* holds Cnil or sKinput or sKoutput */
-  object package;  /* the package symbols are in by default */
-  object index;     /* integer.  The current_dump index on write  */
-  object filepos;   /* nil or the position of the start */ 
-  object table_length; /*    On read it is set to the size dump array needed
-		     or 0
-		     */
-  object evald_items;  /* a list of items which have been eval'd and must
-			  not be walked by fasd_patch_sharp */
-};
-
 struct fasd current_fasd;
 
 
@@ -149,7 +133,7 @@ enum dump_type {
 
 /* given SHORT extract top code (say 4 bits) and bottom byte */
 #define TOP(i) (i >> SIZE_BYTE)
-#define BOTTOM(i) (i &  ~(~0 << SIZE_BYTE))
+#define BOTTOM(i) (i &  ~(~0UL << SIZE_BYTE))
 
 #define FASD_VERSION 2
 
@@ -328,7 +312,7 @@ getd(str)
 
       
 #define D_TYPE_OF(byt) \
-  ((enum dump_type )((unsigned int) byt & ~(~0 << SIZE_D_CODE)))
+  ((enum dump_type )((unsigned int) byt & ~(~0UL << SIZE_D_CODE)))
 
 /* this field may be the top of a short for length, or part of an extended
    code */
@@ -345,14 +329,14 @@ getd(str)
 #define READ_BYTE1() getc(fas_stream)
 
 #define GET8(varx ) \
- do{unsigned long var=(unsigned long)READ_BYTE1();  \
-   var |=  ((unsigned long)READ_BYTE1() << SIZE_BYTE); \
-   var |=  ((unsigned long)READ_BYTE1() << (2*SIZE_BYTE)); \
-   var |=  ((unsigned long)READ_BYTE1() << (3*SIZE_BYTE)); \
-   var |=  ((unsigned long)READ_BYTE1() << (4*SIZE_BYTE)); \
-   var |=  ((unsigned long)READ_BYTE1() << (5*SIZE_BYTE)); \
-   var |=  ((unsigned long)READ_BYTE1() << (6*SIZE_BYTE)); \
-   var |=  ((unsigned long)READ_BYTE1() << (7*SIZE_BYTE)); \
+ do{unsigned long long var=READ_BYTE1();  \
+   var |=  ((unsigned long long)READ_BYTE1() << SIZE_BYTE); \
+   var |=  ((unsigned long long)READ_BYTE1() << (2*SIZE_BYTE)); \
+   var |=  ((unsigned long long)READ_BYTE1() << (3*SIZE_BYTE)); \
+   var |=  ((unsigned long long)READ_BYTE1() << (4*SIZE_BYTE)); \
+   var |=  ((unsigned long long)READ_BYTE1() << (5*SIZE_BYTE)); \
+   var |=  ((unsigned long long)READ_BYTE1() << (6*SIZE_BYTE)); \
+   var |=  ((unsigned long long)READ_BYTE1() << (7*SIZE_BYTE)); \
    DPRINTF("{8byte:varx= %ld}", var); \
      varx=var;} while (0)
 
@@ -379,11 +363,14 @@ getd(str)
 
 
 
-#define MASK ~(~0 << 8)
+#define MASK ~(~0UL << 8)
 #define WRITE_BYTEI(x,i)  putc((((x) >> (i*SIZE_BYTE)) & MASK),fas_stream)
 
+#define PUTFIX(v_) Join(PUT,SIZEOF_LONG)(v_)
+#define GETFIX(v_) Join(GET,SIZEOF_LONG)(v_)
+
 #define PUT8(varx ) \
- do{unsigned long var= varx ; \
+ do{unsigned long long var= varx ; \
      DPRINTF("{8byte:varx= %ld}", var); \
        WRITE_BYTEI(var,0); \
      WRITE_BYTEI(var,1); \
@@ -395,7 +382,7 @@ getd(str)
      WRITE_BYTEI(var,7);} while(0)
 
 #define PUT4(varx ) \
- do{int var= varx ; \
+ do{unsigned long var= varx ; \
      DPRINTF("{4byte:varx= %d}", var); \
        WRITE_BYTEI(var,0); \
      WRITE_BYTEI(var,1); \
@@ -403,14 +390,14 @@ getd(str)
      WRITE_BYTEI(var,3);} while(0)
 
 #define PUT2(var ) \
- do{int v=var; \
+ do{unsigned long v=var; \
      DPRINTF("{2byte:var= %d}", v); \
        WRITE_BYTEI(v,0); \
      WRITE_BYTEI(v,1); \
      } while(0)
 
 #define PUT3(var ) \
- do{int v=var; \
+ do{unsigned long v=var; \
      DPRINTF("{3byte:var= %d}", v); \
        WRITE_BYTEI(v,0); \
      WRITE_BYTEI(v,1); \
@@ -509,8 +496,9 @@ do_hash(object obj, int dot)
    }
  
 static void write_fasd(object obj);
-static object
-FFN(write_fasd_top)(object obj, object x)
+DEFUN_NEW("WRITE-FASD-TOP",object,fSwrite_fasd_top,SI,2,2,NONE,OO,OO,OO,OO,(object obj, object x),"")
+/* static object */
+/* FFN(write_fasd_top)(object obj, object x) */
 {struct fasd *fd = (struct fasd *) x->v.v_self;
   if (fd->direction == sKoutput)
     SETUP_FASD_IN(fd);
@@ -531,8 +519,9 @@ FFN(write_fasd_top)(object obj, object x)
 #define MAYBE_PATCH(result) \
   if (needs_patching)  result =fasd_patch_sharp(result,0)
 
-static object
-FFN(read_fasd_top)(object x)
+DEFUN_NEW("READ-FASD-TOP",object,fSread_fasd_top,SI,1,1,NONE,OO,OO,OO,OO,(object x),"")
+/* static object */
+/* FFN(read_fasd_top)(object x) */
 {  struct fasd *fd = (struct fasd *)  x->v.v_self;
    VOL int e=0;
    object result;
@@ -572,13 +561,21 @@ FFN(read_fasd_top)(object x)
      { RESTORE_FASD;
      return result;}
  }
+#ifdef STATIC_FUNCTION_POINTERS
+object
+fSread_fasd_top(object x) {
+  return FFN(fSread_fasd_top)(x);
+}
+#endif
+
 
 object sLeq;
 object sSPinit;
 void Lmake_hash_table();
 
-static object
-FFN(open_fasd)(object stream, object direction, object eof, object tabl)
+DEFUN_NEW("OPEN-FASD",object,fSopen_fasd,SI,4,4,NONE,OO,OO,OO,OO,(object stream, object direction, object eof, object tabl),"")
+/* static object */
+/* FFN(open_fasd)(object stream, object direction, object eof, object tabl) */
 {  object str=Cnil;
    object result;
    if(direction==sKinput)
@@ -589,11 +586,11 @@ FFN(open_fasd)(object stream, object direction, object eof, object tabl)
 	check_type(tabl,t_vector);}
    if(direction==sKoutput)
      {str=coerce_stream(stream,1);
-      if(tabl==Cnil) tabl=funcall_cfun(Lmake_hash_table,2,sKtest,sLeq);
+       if(tabl==Cnil) tabl=gcl_make_hash_table(sLeq);
       else
 	check_type(tabl,t_hashtable);}
-   check_type(str,t_stream);
-   result=alloc_simple_vector(sizeof(struct fasd)/sizeof(int),aet_object);
+   massert(str==stream);
+   result=alloc_simple_vector(sizeof(struct fasd)/sizeof(object),aet_object);
    array_allocself(result,1,Cnil);
    {struct fasd *fd= (struct fasd *)result->v.v_self;
     fd->table=tabl;
@@ -602,7 +599,7 @@ FFN(open_fasd)(object stream, object direction, object eof, object tabl)
     fd->eof=eof;
     fd->index=small_fixnum(0);
     fd->package=symbol_value(sLApackageA);
-    fd->filepos = make_fixnum(file_position(stream));
+    fd->filepos = make_fixnum(ftell(stream->sm.sm_fp));
     
     SETUP_FASD_IN(fd);
     if (direction==sKoutput){
@@ -625,30 +622,38 @@ FFN(open_fasd)(object stream, object direction, object eof, object tabl)
     fd->index=make_fixnum(dump_index);
     fd->filepos=current_fasd.filepos;
     fd->package=current_fasd.package;
+    fd->table_length=current_fasd.table_length;
     return result;
   }}
+#ifdef STATIC_FUNCTION_POINTERS
+object
+fSopen_fasd(object stream, object direction, object eof, object tabl) {
+  return FFN(fSopen_fasd)(stream,direction,eof,tabl);
+}
+#endif
 
-static object
-FFN(close_fasd)(object ar)
+DEFUN_NEW("CLOSE-FASD",object,fSclose_fasd,SI,1,1,NONE,OO,OO,OO,OO,(object ar),"")
+/* static object */
+/* FFN(close_fasd)(object ar) */
 {  struct fasd *fd= (struct fasd *)(ar->v.v_self);
    check_type(ar,t_vector);
    if (type_of(fd->table)==t_vector)
      /* input uses a vector */
      {if (fd->table->v.v_self)
-       gset(fd->table->v.v_self,0,fix(fd->index),aet_object);
+	 gset(fd->table->v.v_self,0,fix(fd->index),aet_object);
     }
    else
      if(fd->direction==sKoutput)
        {clrhash(fd->table);
 	SETUP_FASD_IN(fd);
 	PUT_OP(d_end_of_file);
-	{int i = file_position(fd->stream);
+	{int i = ftell(fd->stream->sm.sm_fp);
 	 if(type_of(fd->filepos) == t_fixnum)
-	  { file_position_set(fd->stream,fix(fd->filepos) +2);
+	   { fseek(fd->stream->sm.sm_fp,fix(fd->filepos)+2,SEEK_SET);
 	    /* record the length of array needed to read the indices */
 	    PUT4(fix(fd->index));
 	    /* move back to where we were */
-	    file_position_set(fd->stream,i);
+	    fseek(fd->stream->sm.sm_fp,i,SEEK_SET);
 	  }}
 	 
       }
@@ -657,6 +662,12 @@ FFN(close_fasd)(object ar)
    return ar;
   
  }
+#ifdef STATIC_FUNCTION_POINTERS
+object
+fSclose_fasd(object ar) {
+  return FFN(fSclose_fasd)(ar);
+}
+#endif
 
 
 #define HASHP(x) 1
@@ -665,7 +676,7 @@ FFN(close_fasd)(object ar)
 
 static void
 write_fasd(object obj)
-{  int j,leng;
+{  fixnum j,leng;
 
    /* hook for writing other data in fasd file */
 
@@ -778,7 +789,7 @@ write_fasd(object obj)
        else
 	 {PUT_OP(d_fixnum);
 	  j=leng;
-	  PUT4(j);}
+	  PUTFIX(j);}
        break;
      case DP(t_character:)
        PUT_OP(d_standard_character);
@@ -801,7 +812,7 @@ write_fasd(object obj)
      {int l = MP(obj)->_mp_size;
      int m = (l >= 0 ? l : -l);
       
-     unsigned long *u = (unsigned long *) MP(obj)->_mp_d;
+     mp_limb_t *u = MP(obj)->_mp_d;
      /* fix this */
      /* if (sizeof(mp_limb_t) != 4) { FEerror("fix for gmp",0);} */
      PUT4(l);
@@ -891,7 +902,7 @@ fasd_patch_sharp_cons(object x, int depth)
 		if (type_of(x->c.c_cdr) == t_cons)
 			x = x->c.c_cdr;
 		else {
-			x->c.c_cdr = fasd_patch_sharp(x->c.c_cdr,depth+1);
+                        x->c.c_cdr = SAFE_CDR(fasd_patch_sharp(x->c.c_cdr,depth+1));
 			break;
 		}
 	}
@@ -969,99 +980,13 @@ fasd_patch_sharp(object x, int depth)
 }
 
 object sharing_table;
-static enum circ_ind
-is_it_there(object x)
-{ struct htent *e;
-  object table=sharing_table;
-  switch(type_of(x)){
-  case t_cons:
-  case t_symbol:
-  case t_structure:
-  case t_array:
-  case t_vector:
-  case t_package:
-  e= gethash(x,table);
-    if (e->hte_key ==OBJNULL)
-      {sethash(x,table,make_fixnum(-1));
-       return FIRST_INDEX;
-     }
-    else
-      {int n=fix(e->hte_value);
-       if (n <0)
-	 e->hte_value=make_fixnum(n-1);
-       return LATER_INDEX;}
-  break;
- default:
-  return NOT_INDEXED;}}
 
-
-
-static void
-find_sharing(object x)
-{
-  cs_check(x);
- BEGIN:
-  if(is_it_there(x)!=FIRST_INDEX) return;  
-
-	switch (type_of(x)) {
-
-	case DP(t_cons:)
-
-	  find_sharing(x->c.c_car);
-	  x=x->c.c_cdr;
-	  goto BEGIN; 
-	  
-	  break;
-
-	case DP(t_vector:)
-	{
-		int i;
-
-		if ((enum aelttype)x->v.v_elttype != aet_object)
-		  break;
-
-		for (i = 0;  i < x->v.v_fillp;  i++)
-		  find_sharing(x->v.v_self[i]);
-		break;
-	}
-	case DP(t_array:)
-	{
-		int i, j;
-		
-		if ((enum aelttype)x->a.a_elttype != aet_object)
-		  break;
-
-		for (i = 0, j = 1;  i < x->a.a_rank;  i++)
-			j *= x->a.a_dims[i];
-		for (i = 0;  i < j;  i++)
-			find_sharing(x->a.a_self[i]);
-		break;
-	}
-	case DP(t_structure:)
-	  {object def = x->str.str_def;
-	 int i;
-	 i=S_DATA(def)->length;
-	 while (i--> 0)
-	        find_sharing(structure_ref(x,def,i));
-	 break;
-	  }
-	default:
-	  break;
-	}
-	return;
+DEFUN_NEW("FIND-SHARING-TOP",object,fSfind_sharing_top,SI,2,2,NONE,OO,OO,OO,OO,(object x, object table),"") {
+  sharing_table=table;
+  travel_find_sharing(x,table);
+  return Ct;
 }
 
-static object
-FFN(find_sharing_top)(object x, object table)
-{sharing_table=table;
- find_sharing(x);
- return Ct;
-}
-
-
-
-
-
 /* static object            */
 /* read_fasd(int i) */
 /* {object tem; */
@@ -1101,7 +1026,7 @@ grow_vector(object ar)
     ar->v.v_dim=	   ar->v.v_fillp=nl;
     while(--nl >=len)
       ar->v.v_self[nl]=Cnil;
-      END_NO_INTERRUPT;}}
+    END_NO_INTERRUPT;}}
   }
 
 static void
@@ -1122,8 +1047,7 @@ read_fasd1(int i, object *loc)
 	*loc=Cnil;return;
       case DP(d_cons:)
 	read_fasd1(GET_OP(),&tem);
-	*loc=make_cons(tem,Cnil);
-	loc= &((*loc)->c.c_cdr);
+        collect(loc,make_cons(tem,Cnil));
 	i=GET_OP();
 	goto BEGIN;
       case DP(d_list1:) i=1;goto READ_LIST;
@@ -1154,8 +1078,7 @@ read_fasd1(int i, object *loc)
 		read_fasd1(j,&tem);
 		DPRINTF("{Item=",(debug >= 2 ? pp(tem) : 0));
 		DPRINTF("}",0);
-		*loc=make_cons(tem,Cnil);
-		loc= &((*loc)->c.c_cdr);}}
+		collect(loc,make_cons(tem,Cnil));}}
 
       case DP(d_delimiter:)
       case DP(d_dot:)
@@ -1260,18 +1183,18 @@ read_fasd1(int i, object *loc)
 	{int leng=GETD("n=%d");
 	 if (leng & (1 << (SIZE_SHORT -1)))
 	   leng= leng - (1 << (SIZE_SHORT));
-	 *loc=make_fixnum(leng);
+	 *loc=SAFE_CDR(make_fixnum(leng));
 	 return;}
     
       case DP(d_fixnum:)
-	{int j;
-	 GET4(j);
-	 *loc=make_fixnum(j);       
+	{fixnum j;
+	 GETFIX(j);
+	 *loc=SAFE_CDR(make_fixnum(j));       
 	 return;}
       case DP( d_bignum:)
 	{int j,m;
 	 object tem;
-	 unsigned long *u;
+	 mp_limb_t *u;
 	 GET4(j);
 #ifdef GMP
 	 tem = new_bignum();
@@ -1279,7 +1202,7 @@ read_fasd1(int i, object *loc)
 	 _mpz_realloc(MP(tem),m);
 	 MP(tem)->_mp_size = j;
 	 j = m;
-	 u = (unsigned long *) MP(tem)->_mp_d;
+	 u = MP(tem)->_mp_d;
 #else	 
         { BEGIN_NO_INTERRUPT;
 	 tem = alloc_object(t_bignum);
@@ -1483,68 +1406,6 @@ clrhash(object table)
        table->ht.ht_self[i].hte_value = OBJNULL;}
    table->ht.ht_nent =0;}
 
-
-
-object read_fasl_vector1();
-object
-read_fasl_vector(object in)
-{char ch;
- object orig = in;
- object d;
- int tem;
- if (((tem=getc(in->sm.sm_fp)) == EOF) && feof(in->sm.sm_fp))
-   { d = coerce_to_pathname(in);
-     d = make_pathname(d->pn.pn_host,
-		       d->pn.pn_device,
-		       d->pn.pn_directory,
-		       d->pn.pn_name,
-		       make_simple_string("data"),
-		       d->pn.pn_version);
-     d = coerce_to_namestring(d);
-     in = open_stream(d,smm_input,Cnil,Cnil);
-     if (in == Cnil) 
-       FEerror("Can't open file ~s",1,d);
-   }
- else if (tem != EOF)
-   { ungetc(tem,in->sm.sm_fp);}
-  while (1)
-   { ch=readc_stream(in);
-     if (ch=='#')
-       {unreadc_stream(ch,in);
-	return read_fasl_vector1(in);}
-     if (ch== d_begin_dump){
-       unreadc_stream(ch,in);
-       break;}}
- {object ar=FFN(open_fasd)(in,sKinput,0,Cnil);
-  int n=fix(current_fasd.table_length);
-  object result,last;
-  { BEGIN_NO_INTERRUPT;
-#ifdef HAVE_ALLOCA
-  current_fasd.table->v.v_self
-    = (object *)alloca(n*sizeof(object));
-#else
-  current_fasd.table->v.v_self
-    = (object *)alloc_relblock(n*sizeof(object));
-#endif
-  current_fasd.table->v.v_dim=n;
-  current_fasd.table->v.v_fillp=n;
-  gset( current_fasd.table->v.v_self,0,n,aet_object);
-  END_NO_INTERRUPT;
-  }  
-  result=FFN(read_fasd_top)(ar);
-  if (type_of(result) !=t_vector) goto ERROR;
-  last=result->v.v_self[result->v.v_fillp-1];
-  if(type_of(last)!=t_cons || last->c.c_car !=sSPinit)
-    goto ERROR;
-  current_fasd.table->v.v_self = 0;
-  FFN(close_fasd)(ar);
-  if (orig != in)
-    close_stream(in);
-  return result;
- ERROR: FEerror("Bad fasd stream ~a",1,in);
-  return Cnil;
-}}
-
 object IfaslInStream;
 /* static void */
 /* IreadFasdData(void) */
@@ -1581,10 +1442,10 @@ object IfaslInStream;
 static void
 init_fasdump(void)
 {
-  make_si_sfun("READ-FASD-TOP",read_fasd_top,1);
-  make_si_sfun("WRITE-FASD-TOP",write_fasd_top,2);
-  make_si_sfun("OPEN-FASD",open_fasd,4);  
-  make_si_sfun("CLOSE-FASD",close_fasd,1);
-/*  make_si_sfun("FASD-I-DATA",fasd_i_macro,1); */
-  make_si_sfun("FIND-SHARING-TOP",find_sharing_top,2);
+/*   make_si_sfun("READ-FASD-TOP",read_fasd_top,1); */
+/*   make_si_sfun("WRITE-FASD-TOP",write_fasd_top,2); */
+/*   make_si_sfun("OPEN-FASD",open_fasd,4);   */
+/*   make_si_sfun("CLOSE-FASD",close_fasd,1); */
+/* /\*  make_si_sfun("FASD-I-DATA",fasd_i_macro,1); *\/ */
+/*   make_si_sfun("FIND-SHARING-TOP",find_sharing_top,2); */
 }
