@@ -170,6 +170,22 @@ object sSAindent_formatted_outputA;
 			fmt_string = old_fmt_string ; \
                         fmt_paramp = old_fmt_paramp 
 
+#define	fmt_old1	VOL object old_fmt_stream; \
+			VOL int old_ctl_origin; \
+			VOL int old_ctl_index; \
+			VOL int old_ctl_end; \
+			jmp_bufp   VOL old_fmt_jmp_bufp; \
+			VOL int old_fmt_indents; \
+			VOL object old_fmt_string ; \
+                        VOL format_parameter *old_fmt_paramp
+#define	fmt_save1	old_fmt_stream = fmt_stream; \
+			old_ctl_origin = ctl_origin; \
+			old_ctl_index = ctl_index; \
+			old_ctl_end = ctl_end; \
+			old_fmt_jmp_bufp = fmt_jmp_bufp; \
+			old_fmt_indents = fmt_indents; \
+			old_fmt_string = fmt_string ; \
+                        old_fmt_paramp = fmt_paramp
 #define	fmt_restore1	fmt_stream = old_fmt_stream; \
 			ctl_origin = old_ctl_origin; \
 			ctl_index = old_ctl_index; \
@@ -596,7 +612,7 @@ fmt_set_param(int i, int *p, int t, int v)
 static void
 fmt_ascii(bool colon, bool atsign)
 {
-	int mincol, colinc, minpad, padchar;
+	int mincol=0, colinc=0, minpad=0, padchar=0;
 	object x;
 	int l, i;
 
@@ -634,7 +650,7 @@ fmt_ascii(bool colon, bool atsign)
 static void
 fmt_S_expression(bool colon, bool atsign)
 {
-	int mincol, colinc, minpad, padchar;
+	int mincol=0, colinc=0, minpad=0, padchar=0;
 	object x;
 	int l, i;
 
@@ -672,7 +688,7 @@ fmt_S_expression(bool colon, bool atsign)
 static void
 fmt_decimal(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0;
 
 	fmt_max_param(3);
 	fmt_set_param(0, &mincol, fmt_int, 0);
@@ -685,7 +701,7 @@ fmt_decimal(bool colon, bool atsign)
 static void
 fmt_binary(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0;
 
 	fmt_max_param(3);
 	fmt_set_param(0, &mincol, fmt_int, 0);
@@ -698,7 +714,7 @@ fmt_binary(bool colon, bool atsign)
 static void
 fmt_octal(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0;
 
 	fmt_max_param(3);
 	fmt_set_param(0, &mincol, fmt_int, 0);
@@ -711,7 +727,7 @@ fmt_octal(bool colon, bool atsign)
 static void
 fmt_hexadecimal(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0;
 
 	fmt_max_param(3);
 	fmt_set_param(0, &mincol, fmt_int, 0);
@@ -724,7 +740,7 @@ fmt_hexadecimal(bool colon, bool atsign)
 static void
 fmt_radix(bool colon, bool atsign)
 {
-	int radix, mincol, padchar, commachar;
+	int radix=0, mincol=0, padchar=0, commachar=0;
 	object x;
 	int i, j, k;
 	int s, t;
@@ -1017,16 +1033,19 @@ fmt_character(bool colon, bool atsign)
 static void
 fmt_fix_float(bool colon, bool atsign)
 {
-	int w, d, k, overflowchar, padchar;
+        int w=0, d=0, k=0, overflowchar=0, padchar=0,dp;
 	double f;
 	int sign;
-	char buff[256], *b, buff1[256];
+	char *buff, *b, *buff1;
 	int exp;
 	int i, j;
 	object x;
 	int n, m;
 	vs_mark;
 
+	massert(buff=alloca(256)); /*from automatic array -- work around for persistent gcc alpha bug*/
+	massert(buff1=alloca(256));
+
 	b = buff1 + 1;
 
 	fmt_not_colon(colon);
@@ -1061,15 +1080,16 @@ fmt_fix_float(bool colon, bool atsign)
 		vs_reset;
 		return;
 	}
-	if (type_of(x) == t_longfloat)
-/* 		n = 16; */
-		n = 17;
-	else
-/* 		n = 7; */
-		n = 8;
+	if (type_of(x) == t_longfloat) {
+	  n = 17;
+	  dp=1;
+	} else {
+	  n = 8;
+	  dp=0;
+	}
 	f = number_to_double(x);
-	edit_double(n, f, &sign, buff, &exp);
-	if (exp + k > 100 || exp + k < -100 || d > 100) {
+	edit_double(n, f, &sign, buff, &exp, dp);
+	if (sign==2) {
 		prin1(x, fmt_stream);
 		vs_reset;
 		return;
@@ -1096,7 +1116,7 @@ fmt_fix_float(bool colon, bool atsign)
 			n = m = 0;
 	} else if (m < n) {
 		n = m;
-		edit_double(n, f, &sign, buff, &exp);
+		edit_double(n, f, &sign, buff, &exp, dp);
 	}
 	while (n >= 0)
 		if (buff[n - 1] == '0')
@@ -1209,7 +1229,7 @@ fmt_exponent1(int e)
 static void
 fmt_exponential_float(bool colon, bool atsign)
 {
-	int w, d, e, k, overflowchar, padchar, exponentchar;
+        int w=0, d=0, e=0, k=0, overflowchar=0, padchar=0, exponentchar=0,dp;
 	double f;
 	int sign;
 	char buff[256], *b, buff1[256];
@@ -1259,14 +1279,20 @@ fmt_exponential_float(bool colon, bool atsign)
 		vs_reset;
 		return;
 	}
-	if (type_of(x) == t_longfloat)
-/* 		n = 16; */
-		n = 17;
-	else
-/* 		n = 7; */
-		n = 8;
+	if (type_of(x) == t_longfloat) {
+	  n = 17;
+	  dp=1;
+	} else {
+	  n = 8;
+	  dp=0;
+	}
 	f = number_to_double(x);
-	edit_double(n, f, &sign, buff, &exp);
+	edit_double(n, f, &sign, buff, &exp, dp);
+	if (sign==2) {
+		prin1(x, fmt_stream);
+		vs_reset;
+		return;
+	}
 	if (d >= 0) {
 		if (k > 0) {
 			if (!(k < d + 2))
@@ -1299,7 +1325,7 @@ fmt_exponential_float(bool colon, bool atsign)
 			n = m = 0;
 	} else if (m < n) {
 		n = m;
-		edit_double(n, f, &sign, buff, &exp);
+		edit_double(n, f, &sign, buff, &exp, dp);
 	}
 	while (n >= 0)
 		if (buff[n - 1] == '0')
@@ -1413,7 +1439,7 @@ OVER:
 static void
 fmt_general_float(bool colon, bool atsign)
 {
-	int w, d, e, k, overflowchar, padchar, exponentchar;
+        int w=0, d=0, e=0, k, overflowchar, padchar=0, exponentchar,dp;
 	int sign, exp;
 	char buff[256];
 	object x;
@@ -1451,15 +1477,16 @@ fmt_general_float(bool colon, bool atsign)
 		vs_reset;
 		return;
 	}
-	if (type_of(x) == t_longfloat)
-/* 		q = 16; */
-		q = 17;
-	else
-/* 		q = 7; */
-		q = 8;
-	edit_double(q, number_to_double(x), &sign, buff, &exp);
+	if (type_of(x) == t_longfloat) {
+	  q = 17;
+	  dp=1;
+	} else {
+	  q = 8;
+	  dp=0;
+	}
+	edit_double(q, number_to_double(x), &sign, buff, &exp, dp);
 	n = exp + 1;
-	while (q >= 0)
+	while (q > 0)
 		if (buff[q - 1] == '0')
 			--q;
 		else
@@ -1505,7 +1532,7 @@ fmt_general_float(bool colon, bool atsign)
 static void
 fmt_dollars_float(bool colon, bool atsign)
 {
-	int d, n, w, padchar;
+        int d=0, n=0, w=0, padchar=0,dp;
 	double f;
 	int sign;
 	char buff[256];
@@ -1538,15 +1565,16 @@ fmt_dollars_float(bool colon, bool atsign)
 		vs_reset;
 		return;
 	}
-/* 	q = 7; */
 	q = 8;
-	if (type_of(x) == t_longfloat)
-/* 		q = 16; */
+	dp=0;
+	if (type_of(x) == t_longfloat) {
 		q = 17;
+		dp=1;
+	}
 	f = number_to_double(x);
-	edit_double(q, f, &sign, buff, &exp);
+	edit_double(q, f, &sign, buff, &exp, dp);
 	if ((q = exp + d + 1) > 0)
-		edit_double(q, f, &sign, buff, &exp);
+	  edit_double(q, f, &sign, buff, &exp, dp);
 	exp++;
 	if (w > 100 || exp > 100 || exp < -100) {
 		fmt_nparam = 6;
@@ -1592,7 +1620,7 @@ fmt_dollars_float(bool colon, bool atsign)
 static void
 fmt_percent(bool colon, bool atsign)
 {
-	int n, i;
+	int n=0, i;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1609,7 +1637,7 @@ fmt_percent(bool colon, bool atsign)
 static void
 fmt_ampersand(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1627,7 +1655,7 @@ fmt_ampersand(bool colon, bool atsign)
 static void
 fmt_bar(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1640,7 +1668,7 @@ fmt_bar(bool colon, bool atsign)
 static void
 fmt_tilde(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1668,7 +1696,7 @@ fmt_newline(bool colon, bool atsign)
 static void
 fmt_tabulate(bool colon, bool atsign)
 {
-	int colnum, colinc;
+	int colnum=0, colinc=0;
 	int c, i;
 	
 	fmt_max_param(2);
@@ -1704,7 +1732,7 @@ fmt_tabulate(bool colon, bool atsign)
 static void
 fmt_asterisk(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_not_colon_atsign(colon, atsign);
@@ -1773,7 +1801,7 @@ fmt_case(bool colon, bool atsign)
 {
 	VOL object x;
 	VOL int i, j;
-	fmt_old;
+	fmt_old1;
 	jmp_buf fmt_jmp_buf0;
 	int up_colon;
 	bool b;
@@ -1784,7 +1812,7 @@ fmt_case(bool colon, bool atsign)
 	j = fmt_skip();
 	if (ctl_string[--j] != ')' || ctl_string[--j] != '~')
 		fmt_error("~) expected");
-	fmt_save;
+	fmt_save1;
 	fmt_jmp_bufp = &fmt_jmp_buf0;
 	if ((up_colon = setjmp(*fmt_jmp_bufp)))
 		;
@@ -1794,42 +1822,46 @@ fmt_case(bool colon, bool atsign)
 	x = x->sm.sm_object0;
 	if (!colon && !atsign)
 		for (i = 0;  i < x->st.st_fillp;  i++) {
-			if (isUpper(j = x->st.st_self[i]))
-				j += 'a' - 'A';
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isUpper(j))
+		    j += 'a' - 'A';
+		  writec_stream(j, fmt_stream);
 		}
 	else if (colon && !atsign)
 		for (b = TRUE, i = 0;  i < x->st.st_fillp;  i++) {
-			if (isLower(j = x->st.st_self[i])) {
-				if (b)
-					j -= 'a' - 'A';
-				b = FALSE;
-			} else if (isUpper(j)) {
-				if (!b)
-					j += 'a' - 'A';
-				b = FALSE;
-			} else if (!isDigit(j))
-				b = TRUE;
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isLower(j)) {
+		    if (b)
+		      j -= 'a' - 'A';
+		    b = FALSE;
+		  } else if (isUpper(j)) {
+		    if (!b)
+		      j += 'a' - 'A';
+		    b = FALSE;
+		  } else if (!isDigit(j))
+		    b = TRUE;
+		  writec_stream(j, fmt_stream);
 		}
 	else if (!colon && atsign)
 		for (b = TRUE, i = 0;  i < x->st.st_fillp;  i++) {
-			if (isLower(j = x->st.st_self[i])) {
-				if (b)
-					j -= 'a' - 'A';
-				b = FALSE;
-			} else if (isUpper(j)) {
-				if (!b)
-					j += 'a' - 'A';
-				b = FALSE;
-			}
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isLower(j)) {
+		    if (b)
+		      j -= 'a' - 'A';
+		    b = FALSE;
+		  } else if (isUpper(j)) {
+		    if (!b)
+		      j += 'a' - 'A';
+		    b = FALSE;
+		  }
+		  writec_stream(j, fmt_stream);
 		}
 	else
 		for (i = 0;  i < x->st.st_fillp;  i++) {
-			if (isLower(j = x->st.st_self[i]))
-				j -= 'a' - 'A';
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isLower(j))
+		    j -= 'a' - 'A';
+		  writec_stream(j, fmt_stream);
 		}
 	vs_popp;
 	if (up_colon)
@@ -1841,9 +1873,9 @@ fmt_conditional(bool colon, bool atsign)
 {
 	int i, j, k;
 	object x;
-	int n;
+	int n=0;
 	bool done;
-	fmt_old;
+	fmt_old1;
 
 	fmt_not_colon_atsign(colon, atsign);
 	if (colon) {
@@ -1856,11 +1888,11 @@ fmt_conditional(bool colon, bool atsign)
 		if (ctl_string[--k] != ']' || ctl_string[--k] != '~')
 			fmt_error("~] expected");
 		if (fmt_advance() == Cnil) {
-			fmt_save;
+			fmt_save1;
 			format(fmt_stream, ctl_origin + i, j - i);
 			fmt_restore1;
 		} else {
-			fmt_save;
+			fmt_save1;
 			format(fmt_stream, ctl_origin + j + 2, k - (j + 2));
 			fmt_restore1;
 		}
@@ -1873,7 +1905,7 @@ fmt_conditional(bool colon, bool atsign)
 			;
 		else {
 			--fmt_index;
-			fmt_save;
+			fmt_save1;
 			format(fmt_stream, ctl_origin + i, j - i);
 			fmt_restore1;
 		}
@@ -1892,7 +1924,7 @@ fmt_conditional(bool colon, bool atsign)
 			for (k = j;  ctl_string[--k] != '~';)
 				;
 			if (n == 0) {
-				fmt_save;
+				fmt_save1;
 				format(fmt_stream, ctl_origin + i, k - i);
 				fmt_restore1;
 				done = TRUE;
@@ -1918,7 +1950,7 @@ fmt_conditional(bool colon, bool atsign)
 		if (ctl_string[--j] != ']' || ctl_string[--j] != '~')
 			fmt_error("~] expected");
 		if (!done) {
-			fmt_save;
+			fmt_save1;
 			format(fmt_stream, ctl_origin + i, j - i);
 			fmt_restore1;
 		}
@@ -1927,7 +1959,7 @@ fmt_conditional(bool colon, bool atsign)
 
 static void
 fmt_iteration(bool colon, bool atsign) {
-	int i,n;
+	int i,n=0;
 	VOL int j;
 	int o;
 	bool colon_close = FALSE;
@@ -2053,9 +2085,9 @@ fmt_iteration(bool colon, bool atsign) {
 static void
 fmt_justification(volatile bool colon, bool atsign)
 {
-	int mincol, colinc, minpad, padchar;
+	int mincol=0, colinc=0, minpad=0, padchar=0;
 	object fields[FORMAT_DIRECTIVE_LIMIT];
-	fmt_old;
+	fmt_old1;
 	jmp_buf fmt_jmp_buf0;
 	VOL int i,j,n,j0;
 	int k,l,m,l0;
@@ -2082,7 +2114,7 @@ fmt_justification(volatile bool colon, bool atsign)
 			;
 		fields[n] = make_string_output_stream(64);
 		vs_push(fields[n]);
-		fmt_save;
+		fmt_save1;
 		fmt_jmp_bufp = &fmt_jmp_buf0;
 		if ((up_colon = setjmp(*fmt_jmp_bufp))) {
 			--n;
@@ -2109,7 +2141,7 @@ fmt_justification(volatile bool colon, bool atsign)
 			special = 1;
 			for (j = j0;  ctl_string[j] != '~';  --j)
 				;
-			fmt_save;
+			fmt_save1;
 			format(fmt_stream, ctl_origin + j, j0 - j + 2);
 			fmt_restore1;
 			spare_spaces = fmt_spare_spaces;
@@ -2153,7 +2185,7 @@ fmt_justification(volatile bool colon, bool atsign)
 static void
 fmt_up_and_out(bool colon, bool atsign)
 {
-	int i, j, k;
+	int i=0, j=0, k=0;
 
 	fmt_max_param(3);
 	fmt_not_atsign(atsign);
