@@ -6,12 +6,20 @@
 MakeAfun(addr,F_ARGD(min,max,flags,ARGTYPES(a,b,c,d)),0);
 MakeAfun(addr,F_ARGD(2,3,NONE,ARGTYPES(OO,OO,OO,OO)),0);
 */
+
+static int mv;
+
 object MakeAfun(object (*addr)(object,object), unsigned int argd, object data)
-{int type = (F_ARG_FLAGS_P(argd,F_requires_fun_passed) ? t_closure : t_afun);
+{
+  ufixnum at=F_TYPES(argd)>>F_TYPE_WIDTH;
+  ufixnum ma=F_MIN_ARGS(argd);
+  ufixnum xa=F_MAX_ARGS(argd);
+  ufixnum rt=F_RESULT_TYPE(argd);
+  int type = (F_ARG_FLAGS_P(argd,F_requires_fun_passed) ? t_closure : (!at&&!rt&&ma==xa&&!mv ? t_sfun : t_afun));
   object x = alloc_object(type);
   x->sfn.sfn_name = Cnil;
   x->sfn.sfn_self = addr;
-  x->sfn.sfn_argd = argd;
+  x->sfn.sfn_argd = type==t_sfun ? ma : argd;
   if (type == t_closure)
     { x->cl.cl_env = 0;
       x->cl.cl_envdim=0;}
@@ -50,7 +58,7 @@ IsetClosure(object x, int n, va_list ap)
   if (type_of(x) != t_closure)
     { FEerror("Not a closure",0);}
   if (x->cl.cl_envdim < n)
-    {BEGIN_NO_INTERRUPT; x->cl.cl_env = (object *)alloc_contblock(n);
+    {BEGIN_NO_INTERRUPT; x->cl.cl_env = (object *)alloc_relblock(n);
      x->cl.cl_envdim = n;
      END_NO_INTERRUPT;
    }
@@ -82,12 +90,17 @@ closure.")
   return sym;
 }
 
+#include "apply_n.h"
+
 DEFUN_NEW("INITMACRO",object,fSinitmacro,SI,4,ARG_LIMIT,NONE,OO,OO,OO,OO,(object first,...),
       "Like INITFUN, but makes then sets the 'macro' flag on this symbol")
 {va_list ap;
  object res;
+ int n = VFUN_NARGS;
+ object *new;
  va_start(ap,first);
- res = Iapply_ap_new((object (*)())FFN(fSinitfun),first,ap);
+ COERCE_VA_LIST_NEW(new,first,ap,n);
+ res= c_apply_n_f((void *)FFN(fSinitfun),n,new,3,ARG_LIMIT);
  va_end(ap);
  res->s.s_mflag = 1;
  return res;
@@ -102,16 +115,100 @@ DEFUN_NEW("SET-KEY-STRUCT",object,fSset_key_struct,SI,1,1,NONE,OO,OO,OO,OO,(obje
 }
      
 
-void
-SI_makefun(char *strg, object (*fn) (/* ??? */), unsigned int argd)
-{ object sym = make_si_ordinary(strg);
- fSfset(sym, fSmakefun(sym,fn,argd));
+#define mcollect(top_,next_,val_) ({object _x=MMcons(val_,Cnil);\
+                                   if (top_==Cnil) top_=next_=_x; \
+                                   else next_=next_->c.c_cdr=_x;})
+
+
+static void
+put_fn_procls(object sym,fixnum argd,fixnum oneval,object def,object rdef) {
+
+  unsigned int atypes=F_TYPES(argd) >> F_TYPE_WIDTH;
+  unsigned int minargs=F_MIN_ARGS(argd);
+  unsigned int maxargs=F_MAX_ARGS(argd);
+  unsigned int rettype=F_RESULT_TYPE(argd);
+  unsigned int i;
+  object ta=Cnil,na=Cnil;
+
+  for (i=0;i<minargs;i++,atypes >>=F_TYPE_WIDTH) 
+    switch(maxargs!=minargs ? F_object : atypes & MASK_RANGE(0,F_TYPE_WIDTH)) {
+    case F_object:
+      mcollect(ta,na,def);
+      break;
+    case F_int:
+      mcollect(ta,na,sLfixnum);
+      break;
+    case F_shortfloat:
+      mcollect(ta,na,sLshort_float);
+      break;
+    case F_double_ptr:
+      mcollect(ta,na,sLlong_float);
+      break;
+    default:
+      FEerror("Bad sfn declaration",0);
+      break;
+    }
+  if (maxargs!=minargs)
+    mcollect(ta,na,sLA);
+  putprop(sym,ta,sSproclaimed_arg_types);
+  ta=na=Cnil;
+  if (oneval) 
+    switch(rettype) {
+    case F_object:
+      ta=rdef;
+      break;
+    case F_int:
+      ta=sLfixnum;
+      break;
+    case F_shortfloat:
+      ta=sLshort_float;
+      break;
+    case F_double_ptr:
+      ta=sLlong_float;
+      break;
+    default:
+      FEerror("Bad sfn declaration",0);
+      break;
+    }
+  else
+/*     ta=MMcons(sLA,Cnil); */
+    ta=sLA;
+  putprop(sym,ta,sSproclaimed_return_type);
+  if (oneval)
+    putprop(sym,Ct,sSproclaimed_function);
+
 }
 
 void
-LISP_makefun(char *strg, object (*fn) (/* ??? */), unsigned int argd)
+SI_makefun(char *strg, void *fn, unsigned int argd)
+{ object sym = make_si_ordinary(strg);
+ fSfset(sym, fSmakefun(sym,fn,argd));
+ put_fn_procls(sym,argd,1,Ct,Ct);
+}
+
+void
+LISP_makefun(char *strg, void *fn, unsigned int argd)
 { object sym = make_ordinary(strg);
  fSfset(sym, fSmakefun(sym,fn,argd));
+ put_fn_procls(sym,argd,1,Ct,Ct);
+}
+
+void
+SI_makefunm(char *strg, void *fn, unsigned int argd)
+{ object sym = make_si_ordinary(strg);
+  mv=1;
+  fSfset(sym, fSmakefun(sym,fn,argd));
+  mv=0;
+  put_fn_procls(sym,argd,0,Ct,Ct);
+}
+
+void
+LISP_makefunm(char *strg, void *fn, unsigned int argd)
+{ object sym = make_ordinary(strg);
+  mv=1;
+  fSfset(sym, fSmakefun(sym,fn,argd));
+  mv=0;
+  put_fn_procls(sym,argd,0,Ct,Ct);
 }
 
 
