@@ -4,10 +4,7 @@
 #include <string.h>
 
 #ifdef DODEBUG
-#define dprintf(s,arg) \
-  do {fprintf(stderr,s,arg); \
-    fflush(stderr); }\
-    while(0)
+#define dprintf(s,arg) emsg(s,arg)
 #else 
 #define dprintf(s,arg)
 #endif     
@@ -65,7 +62,7 @@
 #endif
 
 #define VOID void
-#define ERROR_MESSAGE(msg)     do{ fprintf(stderr,msg); exit(1) ; } while(0)    
+#define ERROR_MESSAGE(msg)     do{ emsg(msg); do_gcl_abort() ; } while(0)    
 
 #ifdef STAND
 
@@ -87,7 +84,7 @@ main(argc,argv)
   fd = doConnect(argv[1],atoi(argv[2]));
   if (fd < 0) {
     perror("cant connect");
-    exit(1);
+    do_gcl_abort();
   }
 
   while (1) { int high;
@@ -154,7 +151,12 @@ CreateSocketAddress(struct sockaddr_in *sockaddrPtr, char *host, int port)
     } else {
         addr.s_addr = inet_addr(host);
         if (addr.s_addr == -1) {
-            hostent = gethostbyname(host);
+            hostent = /* gethostbyname(host); */
+#ifdef STATIC_LINKING
+	      NULL;
+#else
+	    gethostbyname(host);
+#endif
             if (hostent != NULL) {
                 memcpy((VOID *) &addr,
                         (VOID *) hostent->h_addr_list[0],
@@ -199,7 +201,7 @@ CreateSocket(int port, char *host, int server, char *myaddr, int myport, int asy
                                  * attempt to do an async connect. Otherwise
                                  * do a synchronous connect or bind. */
 {
-    int status, sock, asyncConnect, curState, origState;
+    int status, sock, /* asyncConnect,  */curState, origState;
     struct sockaddr_in sockaddr;	/* socket address */
     struct sockaddr_in mysockaddr;	/* Socket address for client */
 
@@ -225,7 +227,7 @@ CreateSocket(int port, char *host, int server, char *myaddr, int myport, int asy
 
     fcntl(sock, F_SETFD, FD_CLOEXEC);
     
-    asyncConnect = 0;
+    /* asyncConnect = 0; */
     status = 0;
     if (server) {
 
@@ -280,7 +282,7 @@ CreateSocket(int port, char *host, int server, char *myaddr, int myport, int asy
                     sizeof(sockaddr));
             if (status < 0) {
                 if (errno == EINPROGRESS) {
-                    asyncConnect = 1;
+                    /* asyncConnect = 1; */
                     status = 0;
                 }
             }
@@ -325,19 +327,35 @@ doConnect(host,port)
 
 #define SOCKET_FD(strm) ((strm)->sm.sm_fp ? fileno((strm)->sm.sm_fp) : -1)
 
+static void
+check_socket(object x)
+{
+
+ if (type_of(x) != t_stream || x->sm.sm_mode != smm_socket) 
+   FEwrong_type_argument(sSsocket,x);
+  
+}
+
 DEFUN_NEW("GETPEERNAME",object,fSgetpeername,SI,1,1,NONE,OO,OO,OO,OO,(object sock),
  "Return a list of three elements: the address, the hostname and the port for the other end of the socket.  If hostname is not available it will be equal to the address.  Invalid on server sockets. Return NIL on failure.")
 {
  struct sockaddr_in peername;
- int size = sizeof(struct sockaddr_in);
+ socklen_t size = sizeof(struct sockaddr_in);
  struct hostent *hostEntPtr;
  object address,host;
  check_socket(sock);
  if (getpeername(SOCKET_FD(sock), (struct sockaddr *) &peername, &size)
 		>= 0) {
            address=make_simple_string(inet_ntoa(peername.sin_addr));
-           hostEntPtr = gethostbyaddr((char *) &(peername.sin_addr),
-                    sizeof(peername.sin_addr), AF_INET);
+           hostEntPtr = /* gethostbyaddr((char *) &(peername.sin_addr), */
+                    /* sizeof(peername.sin_addr), AF_INET); */
+#ifdef STATIC_LINKING
+	     NULL;
+#else
+	   gethostbyaddr((char *) &(peername.sin_addr),
+			 sizeof(peername.sin_addr), AF_INET);
+#endif
+
             if (hostEntPtr != (struct hostent *) NULL) 
                host = make_simple_string(hostEntPtr->h_name);
             else host = address;
@@ -351,7 +369,7 @@ DEFUN_NEW("GETPEERNAME",object,fSgetpeername,SI,1,1,NONE,OO,OO,OO,OO,(object soc
 DEFUN_NEW("GETSOCKNAME",object,fSgetsockname,SI,1,1,NONE,OO,OO,OO,OO,(object sock),
  "Return a list of three elements: the address, the hostname and the port for the socket.  If hostname is not available it will be equal to the address. Return NIL on failure. ")
 { struct sockaddr_in sockname;
- int size = sizeof(struct sockaddr_in);
+ socklen_t size = sizeof(struct sockaddr_in);
  struct hostent *hostEntPtr;
  object address,host;
 
@@ -359,8 +377,14 @@ DEFUN_NEW("GETSOCKNAME",object,fSgetsockname,SI,1,1,NONE,OO,OO,OO,OO,(object soc
  if (getsockname(SOCKET_FD(sock), (struct sockaddr *) &sockname, &size)
 		>= 0) {
   address= make_simple_string(inet_ntoa(sockname.sin_addr));
-  hostEntPtr = gethostbyaddr((char *) &(sockname.sin_addr),
-			     sizeof(sockname.sin_addr), AF_INET);
+  hostEntPtr = /* gethostbyaddr((char *) &(sockname.sin_addr), */
+	       /* 		     sizeof(sockname.sin_addr), AF_INET); */
+#ifdef STATIC_LINKING
+    NULL;
+#else
+  gethostbyaddr((char *) &(sockname.sin_addr),
+		sizeof(sockname.sin_addr), AF_INET);
+#endif
   if (hostEntPtr != (struct hostent *) NULL)
    host = make_simple_string(hostEntPtr->h_name);
   else host=address;
@@ -485,8 +509,7 @@ getOneChar(FILE *fp)
   int high;
   /*  fprintf(stderr,"<socket 0x%x>",fp);
   fflush(stderr); */
-  fprintf(stderr,"in getOneChar, fd=%d,fp=%p",fd,fp);
-  fflush(stderr);
+  emsg("in getOneChar, fd=%d,fp=%p",fd,fp);
   if (fd == 0)
    { joe(fd);
    return -1;
@@ -502,16 +525,14 @@ getOneChar(FILE *fp)
   if (high > 0)
     {
       int ch ;
-      fprintf(stderr,"in getOneChar, fd=%d,fp=%p",fd,fp);
-      fflush(stderr);
+      emsg("in getOneChar, fd=%d,fp=%p",fd,fp);
       ch = getc(fp);
       if ( ch != EOF || feof(fp) ) {
 	/*      fprintf(stderr,"< 0x%x returning %d,%c>\n",fp,ch,ch);
       fflush(stderr);
       */
       }
-      fprintf(stderr,"in getOneChar, ch= %c,%d\n",ch,ch);
-      fflush(stderr);
+      emsg("in getOneChar, ch= %c,%d\n",ch,ch);
       CHECK_INTERRUPT;	 
       if (ch != EOF) return ch;
       if (feof(fp)) return EOF;
@@ -521,10 +542,7 @@ getOneChar(FILE *fp)
 }
 
 #ifdef DODEBUG
-#define dprintf(s,arg) \
-  do {fprintf(stderr,s,arg); \
-    fflush(stderr); }\
-    while(0)
+#define dprintf(s,arg) emsg(s,arg)
 #else 
 #define dprintf(s,arg)
 #endif     
@@ -612,49 +630,43 @@ doReverse(char *s, int n)
   of the buffer may be changed.
  */
 int
-getCharGclSocket(object strm, object block)
-{
-  object bufp = SOCKET_STREAM_BUFFER(strm);
-  if (bufp->ust.ust_fillp > 0) {
-    dprintf("getchar returns (%c)\n",bufp->ust.ust_self[-1+(bufp->ust.ust_fillp)]);
+getCharGclSocket(object strm, object block) {
+
+  object bufp=SOCKET_STREAM_BUFFER(strm);
+  int fd=SOCKET_STREAM_FD(strm);
+
+  if (bufp->ust.ust_fillp > 0)
     return bufp->ust.ust_self[--(bufp->ust.ust_fillp)];
-  }
-  else {
+
+  if (fd>=0) {
+
     fd_set readfds;
-    struct timeval timeout;
-    int fd = SOCKET_STREAM_FD(strm);
-    if (1)
-      { int high;
-      AGAIN:      
-      /* under cygwin a too large timout like (1<<30) does not work */
-      timeout.tv_sec = (block != Ct ?  0 : 0);
-      timeout.tv_usec = 10000;
-      FD_ZERO(&readfds);
-      FD_SET(fd,&readfds);
-      high = select(fd+1,&readfds,NULL,NULL,&timeout);
-      if (high > 0)
-	{ object bufp = SOCKET_STREAM_BUFFER(strm);
-	int n;
-	n = SAFE_READ(fd,bufp->ust.ust_self ,bufp->ust.ust_dim);
-	doReverse(bufp->ust.ust_self,n);
+    struct timeval t,t1={0,10000},*tp=block==Ct ? NULL : &t;
+    int high,n;
+
+    FD_ZERO(&readfds);
+    FD_SET(fd,&readfds);
+
+    for (;(errno=0,t=t1,high=select(fd+1,&readfds,NULL,NULL,tp))==-1 && !tp && errno==EINTR;);
+
+    if (high > 0) {
+
+      massert((n=SAFE_READ(fd,bufp->st.st_self,bufp->ust.ust_dim))>=0);
+
+      if (n) {
+	doReverse(bufp->st.st_self,n);
 	bufp->ust.ust_fillp=n;
-	if (n > 0)
-	  {
-	    dprintf("getchar returns (%c)\n",bufp->ust.ust_self[-1+(bufp->ust.ust_fillp)]);
-	    return bufp->ust.ust_self[--(bufp->ust.ust_fillp)];
-	  }
-	else
-	  {
-	   return EOF;
-	  FEerror("select said there was stuff there but there was not",0);
-	  }
-	}
-      /* probably a signal interrupted us.. */
-      if (block == Ct)
-	goto AGAIN;
-      return EOF;
-      }
+      } else
+	SOCKET_STREAM_FD(strm)=-1;
+
+      return getCharGclSocket(strm,block);
+
+    }
+
   }
+
+  return EOF;
+
 }
 
 #else
