@@ -30,44 +30,52 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 void
 zero_divisor(void);
 
-object fixnum_add(int i, int j)
+object fixnum_add(fixnum i, fixnum j)
 {
 
   if (i>=0)
    { if (j<= (MOST_POSITIVE_FIX-i))
-      { MYmake_fixnum(return,i+j);
+      { return make_fixnum(i+j);
       }
    MPOP(return,addss,i,j);
    } else { /* i < 0 */
-     if ((MOST_NEG_FIXNUM -i) <= j) {
-       MYmake_fixnum(return,i+j);
+     if ((MOST_NEGATIVE_FIX -i) <= j) {
+       return make_fixnum(i+j);
      }
    MPOP(return,addss,i,j);
    }
 }
 /* return i - j */
-object fixnum_sub(int i, int j)
+object fixnum_sub(fixnum i, fixnum j)
 {  
 
   if (i>=0)
    { if (j >= (i - MOST_POSITIVE_FIX))
-      { MYmake_fixnum(return,i-j);
+      { return make_fixnum(i-j);
       }
    MPOP(return,subss,i,j);
    } else { /* i < 0 */
-     if ((MOST_NEG_FIXNUM -i) <= -j) {
-       MYmake_fixnum(return,i-j);
+     if ((MOST_NEGATIVE_FIX -i) <= -j) {
+       return make_fixnum(i-j);
      }
    MPOP(return,subss,i,j);
    }
 }
 
-/* static object */
-/* fixnum_times(int i, int j) */
-/* { */
+object 
+fixnum_times(fixnum i, fixnum j) {
+
+#ifdef HAVE_CLZL
+  if (i!=MOST_NEGATIVE_FIX && j!=MOST_NEGATIVE_FIX && fixnum_mul_safe(i,j))
+#else
+  if (i>=0 ? (j>=0 ? (!i || j<= (MOST_POSITIVE_FIX/i)) : (j==-1 || i<= (MOST_NEGATIVE_FIX/j))) :
+      (j>=0 ? (i==-1 || j<= (MOST_NEGATIVE_FIX/i)) : (i>MOST_NEGATIVE_FIX && -i<= (MOST_POSITIVE_FIX/-j))))
+#endif
+      return make_fixnum(i*j);
+  else
+    MPOP(return,mulss,i,j);
+}
 
-/*   MPOP(return,mulss,i,j); */
-/* } */
 
 static object
 number_to_complex(object x)
@@ -250,11 +258,7 @@ one_plus(object x)
 	switch (type_of(x)) {
 
 	case t_fixnum:
-	  
-	  if (fix(x)< MOST_POSITIVE_FIX-1) {
-	    MYmake_fixnum(return,fix(x)+1);
-	  }
-	  MPOP(return,addss,1,fix(x));
+	  return fixnum_add(fix(x),1);
 	case t_bignum:
 	  MPOP(return,addsi,1,MP(x));
 	case t_ratio:
@@ -296,7 +300,8 @@ number_minus(object x, object y)
 	case t_fixnum:
 		switch(type_of(y)) {
 		case t_fixnum:
-		  MPOP(return,subss,fix(x),fix(y));
+		  return fixnum_sub(fix(x),fix(y));
+		  /* MPOP(return,subss,fix(x),fix(y)); */
 		case t_bignum:
 		  MPOP(return, subsi,fix(x),MP(y));
 		case t_ratio:
@@ -437,7 +442,7 @@ one_minus(object x)
 	switch (type_of(x)) {
 
 	case t_fixnum:
-	  MPOP(return,addss,fix(x),-1);
+	  return fixnum_sub(fix(x),1);
 	case t_bignum:
 	  MPOP(return,addsi,-1,MP(x));
 	case t_ratio:
@@ -520,7 +525,8 @@ number_times(object x, object y)
 	case t_fixnum:
 		switch (type_of(y)) {
 		case t_fixnum:
-		  MPOP(return,mulss,fix(x),fix(y));
+		  return fixnum_times(fix(x),fix(y));
+		  /* MPOP(return,mulss,fix(x),fix(y)); */
 		case t_bignum:
 		  MPOP(return,mulsi,fix(x),MP(y));
 		case t_ratio:
@@ -544,7 +550,6 @@ number_times(object x, object y)
 		switch (type_of(y)) {
 		case t_fixnum:
  		  MPOP(return,mulsi,fix(y),MP(x));
-
 		case t_bignum:
 		  MPOP(return,mulii,MP(y),MP(x));
 		case t_ratio:
@@ -808,80 +813,60 @@ number_divide(object x, object y)
 }
 
 object
-integer_divide1(object x, object y)
-{
-	object q, r;
+integer_divide1(object x, object y,fixnum d) {
+  object q;
+
+  integer_quotient_remainder_1(x, y, &q, NULL,d);
+  return(q);
 
-	integer_quotient_remainder_1(x, y, &q, &r);
-	return(q);
 }
 
 object
-get_gcd(object x, object y)
-{
-	object	q, r;
+integer_divide2(object x, object y,fixnum d,object *r) {
+  object q;
 
-	if (number_minusp(x))
-		x = number_negate(x);
-	if (number_minusp(y))
-		y = number_negate(y);
+  integer_quotient_remainder_1(x, y, &q, r,d);
+  return(q);
+
+}
+
+object
+get_gcd_abs(object x,object y) {
+
+  object r;
+  
+  for (;;) {
+    
+    if (type_of(x) == t_fixnum && type_of(y) == t_fixnum)
+      return make_fixnum(fixnum_gcd(fix(x),fix(y)));
+    
+    if (number_compare(x, y) < 0) {
+      r = x;
+      x = y;
+      y = r;
+    }
+    if (type_of(y) == t_fixnum && fix(y) == 0)
+      return(x);
+
+    integer_quotient_remainder_1(x, y, NULL, &r,0);
+    x = y;
+    y = r;
+
+  }
+
+}
+
+
+object
+get_gcd(object x, object y) {
+  
+  return get_gcd_abs(number_abs(x),number_abs(y));
 
-L:
-	if (type_of(x) == t_fixnum && type_of(y) == t_fixnum) {
-	  /* LL: */
-	  /* 		if (i < j) { */
-	  /* 			k = i; */
-	  /* 			i = j; */
-	  /* 			j = k; */
-	  /* 		} */
-	  /* 		if (j == 0) { */
-	  /* 			return(make_fixnum(i)); */
-	  /* 		} */
-	  /* 		k = i % j; */
-	  /* 		i = j; */
-	  /* 		j = k; */
-	  /* 		goto LL; */
-	  
-	  register fixnum i, j, k, t;
-
-	  if (!(i = fix(x)))
-	    return y;
-	  if (!(j = fix(y)))
-	    return x;
-	  k=0;
-	  while(!(i&0x1) && !(j&0x1)) {
-	      k++;
-	      i>>=1;
-	      j>>=1;
-	  }
-	  t= i&0x1 ? -j : i>>1; 
-	  do {
-	    while(!(t&0x1)) t>>=1;
-	    if(t>0) i=t; else j=-t;
-	    t=i-j;
-	  } while (t);
-	  return make_fixnum(i<<k);
-
-	}
-
-	if (number_compare(x, y) < 0) {
-		r = x;
-		x = y;
-		y = r;
-	}
-	if (type_of(y) == t_fixnum && fix(y) == 0) {
-		return(x);
-	}
-	integer_quotient_remainder_1(x, y, &q, &r);
-	 x = y;
-	 y = r;
-	goto L;
 }
 
-/* (+          )   */
 LFD(Lplus)(void)
 {
-        int i, j;
+        fixnum i, j;
 	
 	j = vs_top - vs_base;
 	if (j == 0) {
@@ -897,7 +882,7 @@ LFD(Lplus)(void)
 
 LFD(Lminus)(void)
 {
-	int i, j;
+	fixnum i, j;
 
 	j = vs_top - vs_base;
 	if (j == 0)
@@ -915,7 +900,7 @@ LFD(Lminus)(void)
 
 LFD(Ltimes)(void)
 {
-	int i, j;
+	fixnum i, j;
 
 	j = vs_top - vs_base;
 	if (j == 0) {
@@ -931,7 +916,7 @@ LFD(Ltimes)(void)
 
 LFD(Ldivide)(void)
 {
-	int i, j;
+	fixnum i, j;
 
 	j = vs_top - vs_base;
 	if (j == 0)
@@ -978,59 +963,60 @@ LFD(Lconjugate)(void)
 	}
 }
 
-LFD(Lgcd)(void)
-{
-	int i, narg;
+LFD(Lgcd)(void) {
+
+  fixnum i, narg=vs_top-vs_base;
+  
+  if (narg == 0) {
+    vs_push(small_fixnum(0));
+    return;
+  }
+
+  for (i = 0;  i < narg;  i++)
+    check_type_integer(&vs_base[i]);
+
+  vs_top=vs_base;
+  vs_push(number_abs(vs_base[0]));
+  
+  for (i = 1;  i < narg;  i++)
+    vs_base[0] = get_gcd_abs(vs_base[0], number_abs(vs_base[i]));
 
-	narg = vs_top - vs_base;
-	if (narg == 0) {
-		vs_push(small_fixnum(0));
-		return;
-	}
-	for (i = 0;  i < narg;  i++)
-		check_type_integer(&vs_base[i]);
-	if (narg == 1) {
-		if (number_minusp(vs_base[0]))
-			vs_base[0] = number_negate(vs_base[0]);
-		return;
-	}
-	for (i = 1;  i < narg;  i++)
-		vs_base[0] = get_gcd(vs_base[0], vs_base[i]);
-	vs_top = vs_base+1;
 }
 
-LFD(Llcm)(void)
-{
-	object t, g;
-	int i, narg;
+object
+get_lcm_abs(object x,object y) {
+
+  object g=get_gcd_abs(x,y);
+
+  return number_zerop(g) ? g : number_times(x,integer_divide1(y,g,0));
+
+}
+
+object
+get_lcm(object x,object y) {
+
+  return get_lcm_abs(number_abs(x),number_abs(y));
+
+}
+
+LFD(Llcm)(void) {
+
+  fixnum i, narg;
+  
+  narg = vs_top - vs_base;
+
+  if (narg == 0)
+    too_few_arguments();
+
+  for (i = 0;  i < narg;  i++)
+    check_type_integer(&vs_base[i]);
+
+  vs_top=vs_base;
+  vs_push(number_abs(vs_base[0]));
+
+  for (i=1;i<narg && !number_zerop(vs_base[0]);i++)
+    vs_base[0]=get_lcm_abs(vs_base[0],number_abs(vs_base[i]));
 
-	narg = vs_top - vs_base;
-	if (narg == 0)
-		too_few_arguments();
-	for (i = 0;  i < narg;  i++)
-		check_type_integer(&vs_base[i]);
-	if (narg == 1) {
-		if (number_minusp(vs_base[0]))
-			vs_base[0] = number_negate(vs_base[0]);
-		return;
-	}
-	for (i = 1;  i < narg;  i++) {
-	  if (number_zerop(vs_base[0]) ||
-	      number_zerop(vs_base[i]))
-	    vs_base[0]=small_fixnum(0);
-	  else {
-	    t = number_times(vs_base[0], vs_base[i]);
-	    vs_push(t);
-	    g = get_gcd(vs_base[0], vs_base[i]);
-	    vs_push(g);
-	    vs_base[0] = number_divide(t, g);
-	    vs_popp;
-	    vs_popp;
-	  }
-	}
-	if (number_minusp(vs_base[0]))
-		vs_base[0] = number_negate(vs_base[0]);
-	vs_top = vs_base+1;
 }
 
 void
