@@ -20,6 +20,9 @@ Boston, MA 02111-1307, USA.
 
    Geoff Voelker (voelker@cs.washington.edu)                         8-12-94
 */
+
+/* #include "gclincl.h" */
+
 #ifndef UNIXSAVE
 #include <config.h>
 #endif
@@ -105,9 +108,8 @@ void recreate_heap1()
   if (GetModuleFileName (NULL, executable_path, MAX_PATH) == 0) 
     {
       printf ("Failed to find path for executable.\n");
-      exit (1);
+      do_gcl_abort();
     }
-
     recreate_heap (executable_path);
   }
   heap_state = HEAP_LOADED;
@@ -154,7 +156,7 @@ _start (void)
       if (GetModuleFileName (NULL, executable_path, MAX_PATH) == 0) 
 	{
 	  printf ("Failed to find path for executable.\n");
-	  exit (1);
+	  do_gcl_abort();
 	}
 
 #if 1
@@ -206,13 +208,17 @@ _start (void)
 }
 #endif /* UNIXSAVE */
 
+#ifdef __CYGWIN__
+#include <sys/cygwin.h>
+#endif
+
 /* Dump out .data and .bss sections into a new executable.  */
 void
 unexec (char *new_name, char *old_name, void *start_data, void *start_bss,
 	void *entry_address)
 {
-#ifdef __CYGWIN32__
-  file_data in_file, out_file;
+#ifdef __CYGWIN__
+  static file_data in_file, out_file;
   char out_filename[MAX_PATH], in_filename[MAX_PATH];
   char filename[MAX_PATH];
   unsigned long size;
@@ -235,13 +241,13 @@ unexec (char *new_name, char *old_name, void *start_data, void *start_bss,
   ptr = old_name + strlen (old_name) - 4;
   strcpy(filename, old_name);
   strcat(filename, (strcmp (ptr, ".exe") && strcmp (ptr, ".EXE"))?".exe":"");
-  cygwin_conv_to_full_win32_path(filename,in_filename);
+  cygwin_conv_path(CCP_POSIX_TO_WIN_A,filename,in_filename,sizeof(in_filename));
   ptr = new_name + strlen (new_name) - 4;
   strcpy(filename, new_name);
   strcat(filename, (strcmp (ptr, ".exe") && strcmp (ptr, ".EXE"))?".exe":"");
-  cygwin_conv_to_full_win32_path(filename,out_filename);
+  cygwin_conv_path(CCP_POSIX_TO_WIN_A,filename,out_filename,sizeof(out_filename));
 #else 
-  file_data in_file, out_file;
+  static file_data in_file, out_file;
   char out_filename[MAX_PATH], in_filename[MAX_PATH];
   unsigned long size;
   char *ptr;
@@ -270,8 +276,8 @@ unexec (char *new_name, char *old_name, void *start_data, void *start_bss,
   if (strcmp (ptr, ".exe") && strcmp (ptr, ".EXE")  )
     strcat (out_filename, ".exe");
 #endif
-  printf ("Dumping from %s\n", in_filename);
-  printf ("          to %s\n", out_filename);
+  /* printf ("Dumping from %s\n", in_filename); */
+  /* printf ("          to %s\n", out_filename); */
 
   /* We need to round off our heap to NT's allocation unit (64KB).  */
   round_heap (get_allocation_unit ());
@@ -279,9 +285,9 @@ unexec (char *new_name, char *old_name, void *start_data, void *start_bss,
   /* Open the undumped executable file.  */
   if (!open_input_file (&in_file, in_filename))
     {
-      printf ("Failed to open %s (%ld)...bailing.\n", 
-	      in_filename, GetLastError ());
-      exit (1);
+      printf ("Failed to open %s (%u)...bailing.\n",
+	      in_filename, (unsigned)GetLastError ());
+      do_gcl_abort();
     }
 
   /* Get the interesting section info, like start and size of .bss...  */
@@ -289,19 +295,20 @@ unexec (char *new_name, char *old_name, void *start_data, void *start_bss,
 
   /* The size of the dumped executable is the size of the original
      executable plus the size of the heap and the size of the .bss section.  */
-  heap_index_in_executable = (unsigned long)
-    round_to_next ((unsigned char *) in_file.size, get_allocation_unit ());
+  if (heap_index_in_executable==UNINIT_LONG)
+    heap_index_in_executable = (unsigned long)
+      round_to_next ((unsigned char *) in_file.size, get_allocation_unit ());
   /* from lisp we know what to use */
 #ifdef IN_UNIXSAVE
-  data_region_end = round_to_next(core_end,0x10000);
+  data_region_end = round_to_next((unsigned char *)core_end,0x10000);
   real_data_region_end = data_region_end;
 #endif  
   size = heap_index_in_executable + get_committed_heap_size () + bss_size;
   if (!open_output_file (&out_file, out_filename, size))
     {
-      printf ("Failed to open %s (%ld)...bailing.\n", 
-	      out_filename, GetLastError ());
-      exit (1);
+      printf ("Failed to open %s (%u)...bailing.\n",
+	      out_filename, (unsigned)GetLastError ());
+      do_gcl_abort();
     }
 
   /* Set the flag (before dumping).  */
@@ -322,7 +329,11 @@ unexec (char *new_name, char *old_name, void *start_data, void *start_bss,
     dos_header = (PIMAGE_DOS_HEADER) out_file.file_base;
     nt_header = (PIMAGE_NT_HEADERS) ((char *) dos_header + dos_header->e_lfanew);
 
-    
+ 
+    nt_header->OptionalHeader.SizeOfStackReserve=0x800000;
+    /* nt_header->OptionalHeader.SizeOfHeapReserve=0x80000000; */
+    /* nt_header->OptionalHeader.SizeOfHeapCommit=0x80000000; */
+   
     nt_header->OptionalHeader.CheckSum = 0;
 //    nt_header->FileHeader.TimeDateStamp = time (NULL);
 //    dos_header->e_cp = size / 512;
@@ -359,7 +370,7 @@ open_input_file (file_data *p_file, char *filename)
 
   file = CreateFile (filename, GENERIC_READ, FILE_SHARE_READ, NULL,
 		     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
-  if (file == INVALID_HANDLE_VALUE) 
+  if (file == INVALID_HANDLE_VALUE)
     return FALSE;
 
   size = GetFileSize (file, &upper_size);
@@ -444,7 +455,7 @@ get_bss_info_from_map_file (file_data *p_infile, PUCHAR *p_bss_start,
     {
       printf ("Failed to open map file %s, error %d...bailing out.\n",
 	      map_filename, GetLastError ());
-      exit (-1);
+      do_gcl_abort();
     }
 
   while (fgets (buffer, sizeof (buffer), map))
@@ -455,7 +466,7 @@ get_bss_info_from_map_file (file_data *p_infile, PUCHAR *p_bss_start,
       if (n != 2)
 	{
 	  printf ("Failed to scan the .bss section line:\n%s", buffer);
-	  exit (-1);
+	  do_gcl_abort();
 	}
       break;
     }
@@ -484,7 +495,7 @@ find_section (char * name, IMAGE_NT_HEADERS * nt_header)
 
   for (i = 0; i < nt_header->FileHeader.NumberOfSections; i++)
     {
-      if (strcmp (section->Name, name) == 0)
+      if (strcmp ((char *)section->Name, name) == 0)
 	return section;
       section++;
     }
@@ -526,7 +537,7 @@ get_section_info (file_data *p_infile)
   if (dos_header->e_magic != IMAGE_DOS_SIGNATURE) 
     {
       printf ("Unknown EXE header in %s...bailing.\n", p_infile->name);
-      exit (1);
+      do_gcl_abort();
     }
   nt_header = (PIMAGE_NT_HEADERS) (((unsigned long) dos_header) + 
 				   dos_header->e_lfanew);
@@ -534,14 +545,14 @@ get_section_info (file_data *p_infile)
     {
       printf ("Failed to find IMAGE_NT_HEADER in %s...bailing.\n", 
 	     p_infile->name);
-      exit (1);
+      do_gcl_abort();
     }
 
   /* Check the NT header signature ...  */
   if (nt_header->Signature != IMAGE_NT_SIGNATURE) 
     {
-      printf ("Invalid IMAGE_NT_SIGNATURE 0x%lx in %s...bailing.\n",
-	      nt_header->Signature, p_infile->name);
+      printf ("Invalid IMAGE_NT_SIGNATURE 0x%x in %s...bailing.\n",
+	      (int)nt_header->Signature, p_infile->name);
     }
 
   /* Flip through the sections for .data and .bss ...  */
@@ -583,11 +594,11 @@ get_section_info (file_data *p_infile)
 #else
 #define DATA_SECTION ".data"
 #endif      
-      if (!strcmp (section->Name, DATA_SECTION)) 
+      if (!strcmp ((char *)section->Name, DATA_SECTION)) 
 	{
 	  /* The Emacs initialized data section.  */
 	  data_section = section;
-	  ptr = (char *) nt_header->OptionalHeader.ImageBase +
+	  ptr = (unsigned char *) nt_header->OptionalHeader.ImageBase +
 	    section->VirtualAddress;
 	  data_start_va = ptr;
 	  data_start_file = section->PointerToRawData;
@@ -627,7 +638,7 @@ get_section_info (file_data *p_infile)
    bss system data on the Alpha).  However, in practice this doesn't
    seem to matter, since presumably the system libraries always
    reinitialize their bss variables.  */
-  bss_start = min (my_begbss, my_begbss_static);
+  bss_start = (unsigned char *)min (my_begbss, my_begbss_static);
   bss_size = max ((char *)my_endbss, (char *) my_endbss_static) - (char *) bss_start;
 
 #endif
@@ -644,32 +655,32 @@ copy_executable_and_dump_data_section (file_data *p_infile,
   unsigned long size, index;
   
   /* Get a pointer to where the raw data should go in the executable file.  */
-  data_file = (char *) p_outfile->file_base + data_start_file;
+  data_file = (unsigned char *) p_outfile->file_base + data_start_file;
 
   /* Get a pointer to the raw data in our address space.  */
   data_va = data_start_va;
     
-  size = (DWORD) data_file - (DWORD) p_outfile->file_base;
-  printf ("Copying executable up to data section...\n");
-  printf ("\t0x%08x Offset in input file.\n", 0);
-  printf ("\t0x%08x Offset in output file.\n", 0);
-  printf ("\t0x%08lx Size in bytes.\n", size);
+  size = (unsigned long) data_file - (unsigned long) p_outfile->file_base;
+  /* printf ("Copying executable up to data section...\n"); */
+  /* printf ("\t0x%08x Offset in input file.\n", 0); */
+  /* printf ("\t0x%08x Offset in output file.\n", 0); */
+  /* printf ("\t0x%08lx Size in bytes.\n", size); */
   memcpy (p_outfile->file_base, p_infile->file_base, size);
   
   size = data_size;
-  printf ("Dumping .data section...\n");
-  printf ("\t0x%p Address in process.\n", data_va);
-  printf ("\t0x%08x Offset in output file.\n", 
-	  data_file - p_outfile->file_base);
-  printf ("\t0x%08lx Size in bytes.\n", size);
+  /* printf ("Dumping .data section...\n"); */
+  /* printf ("\t0x%p Address in process.\n", data_va); */
+  /* printf ("\t0x%08x Offset in output file.\n",  */
+  /* 	  data_file - p_outfile->file_base); */
+  /* printf ("\t0x%08lx Size in bytes.\n", size); */
   memcpy (data_file, data_va, size);
   
-  index = (DWORD) data_file + size - (DWORD) p_outfile->file_base;
+  index = (unsigned long) data_file + size - (unsigned long) p_outfile->file_base;
   size = p_infile->size - index;
-  printf ("Copying rest of executable...\n");
-  printf ("\t0x%08lx Offset in input file.\n", index);
-  printf ("\t0x%08lx Offset in output file.\n", index);
-  printf ("\t0x%08lx Size in bytes.\n", size);
+  /* printf ("Copying rest of executable...\n"); */
+  /* printf ("\t0x%08lx Offset in input file.\n", index); */
+  /* printf ("\t0x%08lx Offset in output file.\n", index); */
+  /* printf ("\t0x%08lx Size in bytes.\n", size); */
   memcpy ((char *) p_outfile->file_base + index, 
 	  (char *) p_infile->file_base + index, size);
 }
@@ -680,27 +691,27 @@ dump_bss_and_heap (file_data *p_infile, file_data *p_outfile)
     unsigned char *heap_data, *bss_data;
     unsigned long size, index;
 
-    printf ("Dumping heap into executable...\n");
+    /* printf ("Dumping heap into executable...\n"); */
 
     index = heap_index_in_executable;
     size = get_committed_heap_size ();
     heap_data = get_heap_start ();
 
-    printf ("\t0x%p Heap start in process.\n", heap_data);
-    printf ("\t0x%08lx Heap offset in executable.\n", index);
-    printf ("\t0x%08lx Heap size in bytes.\n", size);
+    /* printf ("\t0x%p Heap start in process.\n", heap_data); */
+    /* printf ("\t0x%08lx Heap offset in executable.\n", index); */
+    /* printf ("\t0x%08lx Heap size in bytes.\n", size); */
 
     memcpy ((PUCHAR) p_outfile->file_base + index, heap_data, size);
 
-    printf ("Dumping .bss into executable...\n");
+    /* printf ("Dumping .bss into executable...\n"); */
     
     index += size;
     size = bss_size;
     bss_data = bss_start;
     
-    printf ("\t0x%p BSS start in process.\n", bss_data);
-    printf ("\t0x%08lx BSS offset in executable.\n", index);
-    printf ("\t0x%08lx BSS size in bytes.\n", size);
+    /* printf ("\t0x%p BSS start in process.\n", bss_data); */
+    /* printf ("\t0x%08lx BSS offset in executable.\n", index); */
+    /* printf ("\t0x%08lx BSS size in bytes.\n", size); */
     memcpy ((char *) p_outfile->file_base + index, bss_data, size);
 }
 
@@ -721,7 +732,7 @@ read_in_bss (char *filename)
   if (file == INVALID_HANDLE_VALUE) 
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
 
   /* Seek to where the .bss section is tucked away after the heap...  */
@@ -729,7 +740,7 @@ read_in_bss (char *filename)
   if (SetFilePointer (file, index, NULL, FILE_BEGIN) == 0xFFFFFFFF) 
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
 
   
@@ -738,7 +749,7 @@ read_in_bss (char *filename)
   if (!ReadFile (file, bss_start, bss_size, &n_read, (void *)NULL))
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
 
   CloseHandle (file);
@@ -759,7 +770,7 @@ map_in_heap (char *filename)
   if (file == INVALID_HANDLE_VALUE) 
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
   
   size = GetFileSize (file, &upper_size);
@@ -768,11 +779,11 @@ map_in_heap (char *filename)
   if (!file_mapping) 
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
     
   size = get_committed_heap_size ();
-  file_base = MapViewOfFileEx (file_mapping, FILE_MAP_COPY, 0, 
+  file_base = MapViewOfFileEx (file_mapping, FILE_MAP_ALL_ACCESS, 0,
 			       heap_index_in_executable, size,
 			       get_heap_start ());
   if (file_base != 0) 
@@ -786,10 +797,10 @@ map_in_heap (char *filename)
   CloseHandle (file_mapping);
 
   if (VirtualAlloc (get_heap_start (), get_committed_heap_size (),
-		    MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE) == NULL)
+		    MEM_COMMIT, PAGE_EXECUTE_READWRITE) == NULL)
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
 
   /* Seek to the location of the heap data in the executable.  */
@@ -797,7 +808,7 @@ map_in_heap (char *filename)
   if (SetFilePointer (file, i, NULL, FILE_BEGIN) == 0xFFFFFFFF)
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
 
   /* Read in the data.  */
@@ -805,7 +816,7 @@ map_in_heap (char *filename)
 		 get_committed_heap_size (), &n_read, (void *)NULL))
     {
       i = GetLastError ();
-      exit (1);
+      do_gcl_abort();
     }
 
   CloseHandle (file);
@@ -914,6 +925,24 @@ get_data_end (void)
   return data_region_end;
 }
 
+void *
+probe_base(void *base,unsigned long try,unsigned long inc,unsigned long c) {
+  void *r;
+  if (!(r=VirtualAlloc(base,try,MEM_RESERVE,PAGE_NOACCESS)))
+    return probe_base(base+inc,try,inc,c+1);
+  VirtualFree (r, 0, MEM_RELEASE);
+  return !c || inc<2 ? base : probe_base(base-inc,try,inc>>1,c+1);
+}
+
+unsigned long
+probe_heap_size(void *base,unsigned long try,unsigned long inc) {
+  void *r;
+  if (!(r=VirtualAlloc(base,try,MEM_RESERVE,PAGE_NOACCESS)))
+    return inc<2 ? try-inc : probe_heap_size(base,try-inc,inc>>1);
+  VirtualFree (r, 0, MEM_RELEASE);
+  return probe_heap_size(base,try+inc,inc);
+}
+
 static char *
 allocate_heap (void)
 {
@@ -953,41 +982,28 @@ allocate_heap (void)
      the region below the 256MB line for our malloc arena - 229MB is
      still a pretty decent arena to play in!  */
 
-  unsigned long base = DBEGIN;   /*  27MB */
-  /*   unsigned long base = 0x01B00000; */  /*  27MB */
-  unsigned long end  = 1 << VALBITS; /* 256MB */
-  void *ptr = NULL;
+  void *base,*ptr;
+  unsigned long min=PAGESIZE,inc=(1UL<<31);
 
-#define NTHEAP_PROBE_BASE 0
-#if NTHEAP_PROBE_BASE /* This is never normally defined */
-  /* Macros in gbc.c depend on DBEGIN being divisible by 32 */
-  /* Try various addresses looking for one the kernel will let us have.  */
-  while (!ptr && (base < end))
-    {
-      reserved_heap_size = end - base;
-      ptr = VirtualAlloc ((void *) base,
-			  get_reserved_heap_size (),
-			  MEM_RESERVE,
-			  PAGE_NOACCESS);
-      base += 0x00100000;  /* 1MB increment */
-      DBEGIN = (DBEGIN_TY) ptr;
-    }
+#if defined(__CYGWIN__)
+  ptr=my_endbss;
 #else
-  reserved_heap_size = end - base;
-  ptr = VirtualAlloc ((void *) base,
-		      get_reserved_heap_size (),
-		      MEM_RESERVE,
-		      PAGE_NOACCESS);
-  DBEGIN = (DBEGIN_TY) ptr;
+  ptr=(void *)0x5000000;
 #endif
+  base=probe_base(ptr,min,(unsigned long)my_endbss,0);
+  reserved_heap_size=probe_heap_size(base,inc+min,inc);
+  ptr = VirtualAlloc ((void *) base,get_reserved_heap_size (),MEM_RESERVE,PAGE_NOACCESS);
+  /* printf("probe results: %lu at %p\n",reserved_heap_size,ptr); */
+
+  DBEGIN = (DBEGIN_TY) ptr;
 
   return ptr;
-}
 
+}
 
 /* Emulate Unix sbrk.  */
 void *
-sbrk (unsigned long increment)
+sbrk (ptrdiff_t increment)
 {
   void *result;
   long size = (long) increment;
@@ -995,7 +1011,7 @@ sbrk (unsigned long increment)
   /* Allocate our heap if we haven't done so already.  */
   if (data_region_base == UNINIT_PTR) 
     {
-      data_region_base = allocate_heap ();
+      data_region_base = (unsigned char *)allocate_heap ();
       if (!data_region_base)
 	return NULL;
 
@@ -1004,7 +1020,7 @@ sbrk (unsigned long increment)
       if (((unsigned long) data_region_base & ~VALMASK) != 0) 
 	{
 	  printf ("Error: The heap was allocated in upper memory.\n");
-	  exit (1);
+	  do_gcl_abort();
 	}
 
       data_region_end = data_region_base;
@@ -1052,7 +1068,7 @@ sbrk (unsigned long increment)
 
       /* Commit more of our heap. */
       if (VirtualAlloc (data_region_end, size, MEM_COMMIT,
-			PAGE_READWRITE) == NULL)
+			PAGE_EXECUTE_READWRITE) == NULL)
 	return NULL;
       data_region_end += size;
 
@@ -1065,22 +1081,27 @@ sbrk (unsigned long increment)
   return result;
 }
 
+#ifdef __CYGWIN__
+/* Emulate Unix getpagesize.  */
+int getpagesize (void) { return 4096; }
+#endif
+
 /* Recreate the heap from the data that was dumped to the executable.
    EXECUTABLE_PATH tells us where to find the executable.  */
 void
-recreate_heap (char *executable_path)
-{
+recreate_heap (char *executable_path) {
+
   unsigned char *tmp;
 
   /* First reserve the upper part of our heap.  (We reserve first
      because there have been problems in the past where doing the
      mapping first has loaded DLLs into the VA space of our heap.)  */
-  tmp = VirtualAlloc ((void *) get_heap_end (),
-		      get_reserved_heap_size () - get_committed_heap_size (),
+  tmp = VirtualAlloc ((void *) get_heap_start (),
+		      get_reserved_heap_size (),
 		      MEM_RESERVE,
 		      PAGE_NOACCESS);
   if (!tmp)
-    exit (1);
+    do_gcl_abort();
 
   /* We read in the data for the .bss section from the executable
      first and map in the heap from the executable second to prevent
@@ -1092,6 +1113,7 @@ recreate_heap (char *executable_path)
 
   /* Update system version information to match current system.  */
   cache_system_info ();
+
 }
 
 /* Round the heap up to the given alignment.  */
@@ -1133,6 +1155,7 @@ _heap_term (void)
 #ifdef UNIXSAVE
 BOOL ctrl_c_handler (unsigned long type)
 {
+  extern void sigint(void);
   sigint();
   return 0;
 
