@@ -17,10 +17,18 @@ License for more details.
 
 #ifdef HAVE_XDR
 
+#ifdef DARWIN
+#undef __LP64__ /*Apple header declaration bug workaround for xdr_long*/
+#endif
+
 #ifdef AIX3
 #include <sys/select.h>
 #endif
+#ifdef __CYGWIN__
+#include <rpc/xdr.h>
+#else  /* __CYGWIN__ */
 #include <rpc/rpc.h>
+#endif /* __CYGWIN__ */
 
 extern short aet_sizes[];
 static object
@@ -28,7 +36,7 @@ FFN(siGxdr_open)(f)
      object f;
 { XDR *xdrs;
   object ar= alloc_simple_string(sizeof(XDR));
-  array_allocself(ar,1,0);
+  array_allocself(ar,1,OBJNULL);
   xdrs= (XDR *) ar->a.a_self;
   if (f->sm.sm_fp == 0) FEerror("stream not ok for xdr io",0);
   xdrstdio_create(xdrs, f->sm.sm_fp,
@@ -47,13 +55,16 @@ FFN(siGxdr_write)(object str,object elt) {
 
   switch (type_of(elt)) {
   case t_fixnum:
-    if(!xdr_long(xdrp,&fix(elt))) goto error;
+    {
+      fixnum e=fix(elt);
+      if(xdr_long(xdrp,(long *)&e)) goto error;
+    }
     break;
   case t_longfloat:
-    if(!xdr_double(xdrp,&lf(elt))) goto error;
+    if(xdr_double(xdrp,&lf(elt))) goto error;
     break;
   case t_shortfloat:
-    if(!xdr_float(xdrp,&sf(elt))) goto error;
+    if(xdr_float(xdrp,&sf(elt))) goto error;
     break;
   case t_vector:
     
@@ -79,7 +90,7 @@ FFN(siGxdr_write)(object str,object elt) {
       u_int tmp=elt->v.v_fillp;
       if (tmp!=elt->v.v_fillp)
 	goto error;
-      if(!xdr_array(xdrp,(char **)&elt->v.v_self,
+      if(xdr_array(xdrp,(void *)&elt->v.v_self,
 		    &tmp,
 		    elt->v.v_dim,
 		    aet_sizes[elt->v.v_elttype],
@@ -93,7 +104,7 @@ FFN(siGxdr_write)(object str,object elt) {
   }
   return elt;
  error:
-  FEerror("bad xdr read",0);
+  FEerror("bad xdr write",0);
   return elt;
 }
 
@@ -106,16 +117,16 @@ FFN(siGxdr_read)(object str,object elt) {
   switch (type_of(elt)) { 
   case t_fixnum:
     {fixnum l;
-    if(!xdr_long(xdrp,&l)) goto error;
+      if(xdr_long(xdrp,(long *)&l)) goto error;
     return make_fixnum(l);}
     break;
   case t_longfloat:
     {double x;
-    if(!xdr_double(xdrp,&x)) goto error;
+    if(xdr_double(xdrp,&x)) goto error;
     return make_longfloat(x);}
   case t_shortfloat:
     {float x;
-    if(!xdr_float(xdrp,&x)) goto error;
+    if(xdr_float(xdrp,&x)) goto error;
     return make_shortfloat(x);}
   case t_vector:
     switch(elt->v.v_elttype) {
@@ -141,7 +152,7 @@ FFN(siGxdr_read)(object str,object elt) {
       u_int tmp=elt->v.v_fillp;
       if (tmp!=elt->v.v_fillp)
 	goto error;
-      if(!xdr_array(xdrp,(char **)&elt->v.v_self,
+      if(xdr_array(xdrp,(void *)&elt->v.v_self,
 		    &tmp,
 		    elt->v.v_dim,
 		    aet_sizes[elt->v.v_elttype],
