@@ -1,62 +1,9 @@
-#include <string.h>
-#include <stdlib.h>
-#include "../h/include.h"
+#define FLAVOR ""
 
-extern object user_init();
-
-
-void gcl_init_or_load1 (void (*)(void),char *);
-#define init_or_load(fn,file) do {extern void fn(void); gcl_init_or_load1(fn,file);} \
-  while(0)
-
-/* #define mjoin(a,b) a ## b */
-/* #define Mjoin(a,b) mjoin(a,b) */
-
-#define ar_init(a) do {\
-  char b[200];\
-  \
-  if (snprintf(b,sizeof(b),"ar x %-*.*slibgcl.a %s.o",\
-		sSAsystem_directoryA->s.s_dbind->st.st_fillp,\
-		sSAsystem_directoryA->s.s_dbind->st.st_fillp,\
-		sSAsystem_directoryA->s.s_dbind->st.st_self,#a)<=0)\
-    error("Cannot unpack module " #a "o\n");\
-  if (system(b)) \
-    error("Cannot run ar command to unpack module " #a ".o\n");\
-  init_or_load(Mjoin(init_,a),#a ".o");\
-  if (unlink(#a ".o"))\
-    error("Cannot unlink " #a ".o\n");\
-} while(0)
-
-#define ar_check_init(a,b) do {\
-   object t;\
-   \
-   for (t=b->s.s_dbind;!endp(t) && type_of(t->c.c_car)==t_string && strcmp(#a,t->c.c_car->st.st_self);t=t->c.c_cdr);\
-   if (endp(t))\
-     ar_init(a);\
-} while(0)
-
-
-static void
-load1(x)
-     char *x;
-{printf("loading %s\n",x);
- fflush(stdout);
- load(x);}
-
-#define lsp_init(a) do {\
-   char b[200];\
-   \
-   if (snprintf(b,sizeof(b),"%-*.*s%s",\
-		sSAsystem_directoryA->s.s_dbind->st.st_fillp,\
-		sSAsystem_directoryA->s.s_dbind->st.st_fillp,\
-		sSAsystem_directoryA->s.s_dbind->st.st_self,a)<=0)\
-     error("Cannot append system directory\n");\
-   load1(b);\
-} while(0)
+#include "sys.c"
 
 void
-gcl_init_init()
-{
+gcl_init_init() {
 
   build_symbol_table();
 
@@ -72,22 +19,21 @@ gcl_init_init()
 }
 
 void
-gcl_init_system(object no_init)
-{
+gcl_init_system(object no_init) {
 
   if (type_of(no_init)!=t_symbol)
     error("Supplied no_init is not of type symbol\n");
 
+  ar_check_init(gcl_arraylib,no_init);
   ar_check_init(gcl_predlib,no_init);
   ar_check_init(gcl_setf,no_init);
-  ar_check_init(gcl_arraylib,no_init);
   ar_check_init(gcl_assert,no_init);
   ar_check_init(gcl_defstruct,no_init);
+  ar_check_init(gcl_restart,no_init);
   ar_check_init(gcl_describe,no_init);
 #ifdef HAVE_JAPI_H
   ar_check_init(gcl_japi,no_init);
 #endif
-  ar_check_init(gcl_iolib,no_init);
   ar_check_init(gcl_listlib,no_init);
   ar_check_init(gcl_mislib,no_init);
   ar_check_init(gcl_numlib,no_init);
@@ -101,8 +47,24 @@ gcl_init_system(object no_init)
   ar_check_init(gcl_loop,no_init);
   ar_check_init(gcl_defpackage,no_init);
   ar_check_init(gcl_make_defpackage,no_init);
+  ar_check_init(gcl_sharp,no_init);
 
+  ar_check_init(gcl_sharp_uv,no_init);
+  ar_check_init(gcl_namestring,no_init);
+  ar_check_init(gcl_logical_pathname_translations,no_init);
+  ar_check_init(gcl_make_pathname,no_init);
+  ar_check_init(gcl_parse_namestring,no_init);
+  ar_check_init(gcl_translate_pathname,no_init);
+  ar_check_init(gcl_directory,no_init);
+  ar_check_init(gcl_merge_pathnames,no_init);
+  ar_check_init(gcl_truename,no_init);
+  ar_check_init(gcl_rename_file,no_init);
+  ar_check_init(gcl_wild_pathname_p,no_init);
+  ar_check_init(gcl_pathname_match_p,no_init);
 	
+  ar_check_init(gcl_iolib,no_init);
+  ar_check_init(gcl_fpe,no_init);
+
   ar_check_init(gcl_cmpinline,no_init);
   ar_check_init(gcl_cmputil,no_init);
 
@@ -131,7 +93,25 @@ gcl_init_system(object no_init)
   ar_check_init(gcl_cmpvar,no_init);
   ar_check_init(gcl_cmpvs,no_init);
   ar_check_init(gcl_cmpwt,no_init);
+  ar_check_init(gcl_cmpmain,no_init);
 
+#ifdef HAVE_XGCL
+  lsp_init("../xgcl-2/package.lisp");
+  ar_check_init(gcl_Xlib,no_init);
+  ar_check_init(gcl_Xutil,no_init);
+  ar_check_init(gcl_X,no_init);
+  ar_check_init(gcl_XAtom,no_init);
+  ar_check_init(gcl_defentry_events,no_init);
+  ar_check_init(gcl_Xstruct,no_init);
+  ar_check_init(gcl_XStruct_l_3,no_init);
+  ar_check_init(gcl_general,no_init);
+  ar_check_init(gcl_keysymdef,no_init);
+  ar_check_init(gcl_X10,no_init);
+  ar_check_init(gcl_Xinit,no_init);
+  ar_check_init(gcl_dwtrans,no_init);
+  ar_check_init(gcl_tohtml,no_init);
+  ar_check_init(gcl_index,no_init);
+#endif
   
 }
 
