@@ -1,7 +1,7 @@
-/* Events.c           Hiep Huu Nguyen                         27 Aug 92 */
+/* Events.c           Hiep Huu Nguyen            27 Jun 06 */
 
 /*; Copyright (c) 1994 Hiep Huu Nguyen and The University of Texas at Austin.
-
+; edited 27 Aug 92; 12 Aug 2002; 23 Jun 06 by GSN; 27 Jun 06 by GSN
 ; See the files gnu.license and dec.copyright .
 
 ; This program is free software; you can redistribute it and/or modify
@@ -21,14 +21,13 @@
 ; Some of the files that interface to the Xlib are adapted from DEC/MIT files.
 ; See the file dec.copyright for details. */
 
+#include <stdlib.h>
 #include <X11/Xlib.h>
 
-#include "include.h"
+/********* XKeyEvent functions *****/
 
-/********* XKeyEvent funcions *****/
-
-int  make_XKeyEvent (){
-          return ((int) calloc(1, sizeof(XKeyEvent)));
+long  make_XKeyEvent (){
+          return ((long) calloc(1, sizeof(XKeyEvent)));
 }
 
 int  XKeyEvent_same_screen(i)
@@ -174,17 +173,17 @@ int j;
           i->window = j;
 }
 
-Display *XKeyEvent_display(i)
+long  XKeyEvent_display(i)
 XKeyEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XKeyEvent_display(i, j)
 XKeyEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XKeyEvent_send_event(i)
@@ -227,10 +226,10 @@ int j;
 }
 
 
-/********* XButtonEvent funcions *****/
+/********* XButtonEvent functions *****/
 
-int  make_XButtonEvent (){
-          return ((int) calloc(1, sizeof(XButtonEvent)));
+long  make_XButtonEvent (){
+          return ((long) calloc(1, sizeof(XButtonEvent)));
 }
 
 int  XButtonEvent_same_screen(i)
@@ -376,17 +375,17 @@ int j;
           i->window = j;
 }
 
-Display *XButtonEvent_display(i)
+long  XButtonEvent_display(i)
 XButtonEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XButtonEvent_display(i, j)
 XButtonEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XButtonEvent_send_event(i)
@@ -429,10 +428,10 @@ int j;
 }
 
 
-/********* XMotionEvent funcions *****/
+/********* XMotionEvent functions *****/
 
-int  make_XMotionEvent (){
-          return ((int) calloc(1, sizeof(XMotionEvent)));
+long  make_XMotionEvent (){
+          return ((long) calloc(1, sizeof(XMotionEvent)));
 }
 
 int  XMotionEvent_same_screen(i)
@@ -578,17 +577,17 @@ int j;
           i->window = j;
 }
 
-Display *XMotionEvent_display(i)
+long  XMotionEvent_display(i)
 XMotionEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XMotionEvent_display(i, j)
 XMotionEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XMotionEvent_send_event(i)
@@ -631,10 +630,10 @@ int j;
 }
 
 
-/********* XCrossingEvent funcions *****/
+/********* XCrossingEvent functions *****/
 
-int  make_XCrossingEvent (){
-          return ((int) calloc(1, sizeof(XCrossingEvent)));
+long  make_XCrossingEvent (){
+          return ((long) calloc(1, sizeof(XCrossingEvent)));
 }
 
 int  XCrossingEvent_state(i)
@@ -806,17 +805,17 @@ int j;
           i->window = j;
 }
 
-Display *XCrossingEvent_display(i)
+long  XCrossingEvent_display(i)
 XCrossingEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XCrossingEvent_display(i, j)
 XCrossingEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XCrossingEvent_send_event(i)
@@ -859,10 +858,10 @@ int j;
 }
 
 
-/********* XFocusChangeEvent funcions *****/
+/********* XFocusChangeEvent functions *****/
 
-int  make_XFocusChangeEvent (){
-          return ((int) calloc(1, sizeof(XFocusChangeEvent)));
+long  make_XFocusChangeEvent (){
+          return ((long) calloc(1, sizeof(XFocusChangeEvent)));
 }
 
 int  XFocusChangeEvent_detail(i)
@@ -904,17 +903,17 @@ int j;
           i->window = j;
 }
 
-Display *XFocusChangeEvent_display(i)
+long  XFocusChangeEvent_display(i)
 XFocusChangeEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XFocusChangeEvent_display(i, j)
 XFocusChangeEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XFocusChangeEvent_send_event(i)
@@ -957,10 +956,10 @@ int j;
 }
 
 
-/********* XKeymapEvent funcions *****/
+/********* XKeymapEvent functions *****/
 
-int  make_XKeymapEvent (){
-          return ((int) calloc(1, sizeof(XKeymapEvent)));
+long  make_XKeymapEvent (){
+          return ((long) calloc(1, sizeof(XKeymapEvent)));
 }
 
 char* XKeymapEvent_key_vector(i)
@@ -981,17 +980,17 @@ int j;
           i->window = j;
 }
 
-Display * XKeymapEvent_display(i)
+long  XKeymapEvent_display(i)
 XKeymapEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XKeymapEvent_display(i, j)
 XKeymapEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XKeymapEvent_send_event(i)
@@ -1034,10 +1033,10 @@ int j;
 }
 
 
-/********* XExposeEvent funcions *****/
+/********* XExposeEvent functions *****/
 
-int  make_XExposeEvent (){
-          return ((int) calloc(1, sizeof(XExposeEvent)));
+long  make_XExposeEvent (){
+          return ((long) calloc(1, sizeof(XExposeEvent)));
 }
 
 int  XExposeEvent_count(i)
@@ -1118,17 +1117,17 @@ int j;
           i->window = j;
 }
 
-Display *XExposeEvent_display(i)
+long  XExposeEvent_display(i)
 XExposeEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XExposeEvent_display(i, j)
 XExposeEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XExposeEvent_send_event(i)
@@ -1171,10 +1170,10 @@ int j;
 }
 
 
-/********* XGraphicsExposeEvent funcions *****/
+/********* XGraphicsExposeEvent functions *****/
 
-int  make_XGraphicsExposeEvent (){
-          return ((int) calloc(1, sizeof(XGraphicsExposeEvent)));
+long  make_XGraphicsExposeEvent (){
+          return ((long) calloc(1, sizeof(XGraphicsExposeEvent)));
 }
 
 int  XGraphicsExposeEvent_minor_code(i)
@@ -1281,17 +1280,17 @@ Drawable j;
           i->drawable = j;
 }
 
-Display * XGraphicsExposeEvent_display(i)
+long  XGraphicsExposeEvent_display(i)
 XGraphicsExposeEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XGraphicsExposeEvent_display(i, j)
 XGraphicsExposeEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XGraphicsExposeEvent_send_event(i)
@@ -1334,10 +1333,10 @@ int j;
 }
 
 
-/********* XNoExposeEvent funcions *****/
+/********* XNoExposeEvent functions *****/
 
-int  make_XNoExposeEvent (){
-          return ((int) calloc(1, sizeof(XNoExposeEvent)));
+long  make_XNoExposeEvent (){
+          return ((long) calloc(1, sizeof(XNoExposeEvent)));
 }
 
 int  XNoExposeEvent_minor_code(i)
@@ -1379,17 +1378,17 @@ Drawable j;
           i->drawable = j;
 }
 
-Display *XNoExposeEvent_display(i)
+long  XNoExposeEvent_display(i)
 XNoExposeEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XNoExposeEvent_display(i, j)
 XNoExposeEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XNoExposeEvent_send_event(i)
@@ -1432,10 +1431,10 @@ int j;
 }
 
 
-/********* XVisibilityEvent funcions *****/
+/********* XVisibilityEvent functions *****/
 
-int  make_XVisibilityEvent (){
-          return ((int) calloc(1, sizeof(XVisibilityEvent)));
+long  make_XVisibilityEvent (){
+          return ((long) calloc(1, sizeof(XVisibilityEvent)));
 }
 
 int  XVisibilityEvent_state(i)
@@ -1464,17 +1463,17 @@ int j;
           i->window = j;
 }
 
-Display *XVisibilityEvent_display(i)
+long  XVisibilityEvent_display(i)
 XVisibilityEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XVisibilityEvent_display(i, j)
 XVisibilityEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XVisibilityEvent_send_event(i)
@@ -1517,10 +1516,10 @@ int j;
 }
 
 
-/********* XCreateWindowEvent funcions *****/
+/********* XCreateWindowEvent functions *****/
 
-int  make_XCreateWindowEvent (){
-          return ((int) calloc(1, sizeof(XCreateWindowEvent)));
+long  make_XCreateWindowEvent (){
+          return ((long) calloc(1, sizeof(XCreateWindowEvent)));
 }
 
 int  XCreateWindowEvent_override_redirect(i)
@@ -1627,17 +1626,17 @@ int j;
           i->parent = j;
 }
 
-Display *XCreateWindowEvent_display(i)
+long  XCreateWindowEvent_display(i)
 XCreateWindowEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XCreateWindowEvent_display(i, j)
 XCreateWindowEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XCreateWindowEvent_send_event(i)
@@ -1680,10 +1679,10 @@ int j;
 }
 
 
-/********* XDestroyWindowEvent funcions *****/
+/********* XDestroyWindowEvent functions *****/
 
-int  make_XDestroyWindowEvent (){
-          return ((int) calloc(1, sizeof(XDestroyWindowEvent)));
+long  make_XDestroyWindowEvent (){
+          return ((long) calloc(1, sizeof(XDestroyWindowEvent)));
 }
 
 int  XDestroyWindowEvent_window(i)
@@ -1712,17 +1711,17 @@ int j;
           i->event = j;
 }
 
-Display *XDestroyWindowEvent_display(i)
+long  XDestroyWindowEvent_display(i)
 XDestroyWindowEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XDestroyWindowEvent_display(i, j)
 XDestroyWindowEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XDestroyWindowEvent_send_event(i)
@@ -1765,10 +1764,10 @@ int j;
 }
 
 
-/********* XUnmapEvent funcions *****/
+/********* XUnmapEvent functions *****/
 
-int  make_XUnmapEvent (){
-          return ((int) calloc(1, sizeof(XUnmapEvent)));
+long  make_XUnmapEvent (){
+          return ((long) calloc(1, sizeof(XUnmapEvent)));
 }
 
 int  XUnmapEvent_from_configure(i)
@@ -1810,17 +1809,17 @@ int j;
           i->event = j;
 }
 
-Display *XUnmapEvent_display(i)
+long  XUnmapEvent_display(i)
 XUnmapEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XUnmapEvent_display(i, j)
 XUnmapEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XUnmapEvent_send_event(i)
@@ -1863,10 +1862,10 @@ int j;
 }
 
 
-/********* XMapEvent funcions *****/
+/********* XMapEvent functions *****/
 
-int  make_XMapEvent (){
-          return ((int) calloc(1, sizeof(XMapEvent)));
+long  make_XMapEvent (){
+          return ((long) calloc(1, sizeof(XMapEvent)));
 }
 
 int  XMapEvent_override_redirect(i)
@@ -1908,17 +1907,17 @@ int j;
           i->event = j;
 }
 
-Display *XMapEvent_display(i)
+long  XMapEvent_display(i)
 XMapEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XMapEvent_display(i, j)
 XMapEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XMapEvent_send_event(i)
@@ -1961,10 +1960,10 @@ int j;
 }
 
 
-/********* XMapRequestEvent funcions *****/
+/********* XMapRequestEvent functions *****/
 
-int  make_XMapRequestEvent (){
-          return ((int) calloc(1, sizeof(XMapRequestEvent)));
+long  make_XMapRequestEvent (){
+          return ((long) calloc(1, sizeof(XMapRequestEvent)));
 }
 
 int  XMapRequestEvent_window(i)
@@ -1993,17 +1992,17 @@ int j;
           i->parent = j;
 }
 
-Display *XMapRequestEvent_display(i)
+long  XMapRequestEvent_display(i)
 XMapRequestEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XMapRequestEvent_display(i, j)
 XMapRequestEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XMapRequestEvent_send_event(i)
@@ -2046,10 +2045,10 @@ int j;
 }
 
 
-/********* XReparentEvent funcions *****/
+/********* XReparentEvent functions *****/
 
-int  make_XReparentEvent (){
-          return ((int) calloc(1, sizeof(XReparentEvent)));
+long  make_XReparentEvent (){
+          return ((long) calloc(1, sizeof(XReparentEvent)));
 }
 
 int  XReparentEvent_override_redirect(i)
@@ -2130,17 +2129,17 @@ int j;
           i->event = j;
 }
 
-Display *XReparentEvent_display(i)
+long  XReparentEvent_display(i)
 XReparentEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XReparentEvent_display(i, j)
 XReparentEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XReparentEvent_send_event(i)
@@ -2183,10 +2182,10 @@ int j;
 }
 
 
-/********* XConfigureEvent funcions *****/
+/********* XConfigureEvent functions *****/
 
-int  make_XConfigureEvent (){
-          return ((int) calloc(1, sizeof(XConfigureEvent)));
+long  make_XConfigureEvent (){
+          return ((long) calloc(1, sizeof(XConfigureEvent)));
 }
 
 int  XConfigureEvent_override_redirect(i)
@@ -2306,17 +2305,17 @@ int j;
           i->event = j;
 }
 
-Display *XConfigureEvent_display(i)
+long  XConfigureEvent_display(i)
 XConfigureEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XConfigureEvent_display(i, j)
 XConfigureEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XConfigureEvent_send_event(i)
@@ -2359,10 +2358,10 @@ int j;
 }
 
 
-/********* XGravityEvent funcions *****/
+/********* XGravityEvent functions *****/
 
-int  make_XGravityEvent (){
-          return ((int) calloc(1, sizeof(XGravityEvent)));
+long  make_XGravityEvent (){
+          return ((long) calloc(1, sizeof(XGravityEvent)));
 }
 
 int  XGravityEvent_y(i)
@@ -2417,17 +2416,17 @@ int j;
           i->event = j;
 }
 
-Display *XGravityEvent_display(i)
+long  XGravityEvent_display(i)
 XGravityEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XGravityEvent_display(i, j)
 XGravityEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XGravityEvent_send_event(i)
@@ -2470,10 +2469,10 @@ int j;
 }
 
 
-/********* XResizeRequestEvent funcions *****/
+/********* XResizeRequestEvent functions *****/
 
-int  make_XResizeRequestEvent (){
-          return ((int) calloc(1, sizeof(XResizeRequestEvent)));
+long  make_XResizeRequestEvent (){
+          return ((long) calloc(1, sizeof(XResizeRequestEvent)));
 }
 
 int  XResizeRequestEvent_height(i)
@@ -2515,17 +2514,17 @@ int j;
           i->window = j;
 }
 
-Display *XResizeRequestEvent_display(i)
+long  XResizeRequestEvent_display(i)
 XResizeRequestEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XResizeRequestEvent_display(i, j)
 XResizeRequestEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XResizeRequestEvent_send_event(i)
@@ -2568,10 +2567,10 @@ int j;
 }
 
 
-/********* XConfigureRequestEvent funcions *****/
+/********* XConfigureRequestEvent functions *****/
 
-int  make_XConfigureRequestEvent (){
-          return ((int) calloc(1, sizeof(XConfigureRequestEvent)));
+long  make_XConfigureRequestEvent (){
+          return ((long) calloc(1, sizeof(XConfigureRequestEvent)));
 }
 
 int  XConfigureRequestEvent_value_mask(i)
@@ -2704,17 +2703,17 @@ int j;
           i->parent = j;
 }
 
-Display *XConfigureRequestEvent_display(i)
+long  XConfigureRequestEvent_display(i)
 XConfigureRequestEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XConfigureRequestEvent_display(i, j)
 XConfigureRequestEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XConfigureRequestEvent_send_event(i)
@@ -2757,10 +2756,10 @@ int j;
 }
 
 
-/********* XCirculateEvent funcions *****/
+/********* XCirculateEvent functions *****/
 
-int  make_XCirculateEvent (){
-          return ((int) calloc(1, sizeof(XCirculateEvent)));
+long  make_XCirculateEvent (){
+          return ((long) calloc(1, sizeof(XCirculateEvent)));
 }
 
 int  XCirculateEvent_place(i)
@@ -2802,17 +2801,17 @@ int j;
           i->event = j;
 }
 
-Display *XCirculateEvent_display(i)
+long  XCirculateEvent_display(i)
 XCirculateEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XCirculateEvent_display(i, j)
 XCirculateEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XCirculateEvent_send_event(i)
@@ -2855,10 +2854,10 @@ int j;
 }
 
 
-/********* XCirculateRequestEvent funcions *****/
+/********* XCirculateRequestEvent functions *****/
 
-int  make_XCirculateRequestEvent (){
-          return ((int) calloc(1, sizeof(XCirculateRequestEvent)));
+long  make_XCirculateRequestEvent (){
+          return ((long) calloc(1, sizeof(XCirculateRequestEvent)));
 }
 
 int  XCirculateRequestEvent_place(i)
@@ -2900,17 +2899,17 @@ int j;
           i->parent = j;
 }
 
-Display *XCirculateRequestEvent_display(i)
+long  XCirculateRequestEvent_display(i)
 XCirculateRequestEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XCirculateRequestEvent_display(i, j)
 XCirculateRequestEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XCirculateRequestEvent_send_event(i)
@@ -2953,10 +2952,10 @@ int j;
 }
 
 
-/********* XPropertyEvent funcions *****/
+/********* XPropertyEvent functions *****/
 
-int  make_XPropertyEvent (){
-          return ((int) calloc(1, sizeof(XPropertyEvent)));
+long  make_XPropertyEvent (){
+          return ((long) calloc(1, sizeof(XPropertyEvent)));
 }
 
 int  XPropertyEvent_state(i)
@@ -3011,17 +3010,17 @@ int j;
           i->window = j;
 }
 
-Display *XPropertyEvent_display(i)
+long  XPropertyEvent_display(i)
 XPropertyEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XPropertyEvent_display(i, j)
 XPropertyEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XPropertyEvent_send_event(i)
@@ -3064,10 +3063,10 @@ int j;
 }
 
 
-/********* XSelectionClearEvent funcions *****/
+/********* XSelectionClearEvent functions *****/
 
-int  make_XSelectionClearEvent (){
-          return ((int) calloc(1, sizeof(XSelectionClearEvent)));
+long  make_XSelectionClearEvent (){
+          return ((long) calloc(1, sizeof(XSelectionClearEvent)));
 }
 
 int  XSelectionClearEvent_time(i)
@@ -3109,17 +3108,17 @@ int j;
           i->window = j;
 }
 
-Display *XSelectionClearEvent_display(i)
+long  XSelectionClearEvent_display(i)
 XSelectionClearEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XSelectionClearEvent_display(i, j)
 XSelectionClearEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XSelectionClearEvent_send_event(i)
@@ -3162,10 +3161,10 @@ int j;
 }
 
 
-/********* XSelectionRequestEvent funcions *****/
+/********* XSelectionRequestEvent functions *****/
 
-int  make_XSelectionRequestEvent (){
-          return ((int) calloc(1, sizeof(XSelectionRequestEvent)));
+long  make_XSelectionRequestEvent (){
+          return ((long) calloc(1, sizeof(XSelectionRequestEvent)));
 }
 
 int  XSelectionRequestEvent_time(i)
@@ -3246,17 +3245,17 @@ int j;
           i->owner = j;
 }
 
-Display *XSelectionRequestEvent_display(i)
+long  XSelectionRequestEvent_display(i)
 XSelectionRequestEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XSelectionRequestEvent_display(i, j)
 XSelectionRequestEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XSelectionRequestEvent_send_event(i)
@@ -3299,10 +3298,10 @@ int j;
 }
 
 
-/********* XSelectionEvent funcions *****/
+/********* XSelectionEvent functions *****/
 
-int  make_XSelectionEvent (){
-          return ((int) calloc(1, sizeof(XSelectionEvent)));
+long  make_XSelectionEvent (){
+          return ((long) calloc(1, sizeof(XSelectionEvent)));
 }
 
 int  XSelectionEvent_time(i)
@@ -3370,17 +3369,17 @@ int j;
           i->requestor = j;
 }
 
-Display *XSelectionEvent_display(i)
+long  XSelectionEvent_display(i)
 XSelectionEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XSelectionEvent_display(i, j)
 XSelectionEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XSelectionEvent_send_event(i)
@@ -3423,10 +3422,10 @@ int j;
 }
 
 
-/********* XColormapEvent funcions *****/
+/********* XColormapEvent functions *****/
 
-int  make_XColormapEvent (){
-          return ((int) calloc(1, sizeof(XColormapEvent)));
+long  make_XColormapEvent (){
+          return ((long) calloc(1, sizeof(XColormapEvent)));
 }
 
 int  XColormapEvent_state(i)
@@ -3481,17 +3480,17 @@ int j;
           i->window = j;
 }
 
-Display *XColormapEvent_display(i)
+long  XColormapEvent_display(i)
 XColormapEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XColormapEvent_display(i, j)
 XColormapEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XColormapEvent_send_event(i)
@@ -3534,10 +3533,10 @@ int j;
 }
 
 
-/********* XClientMessageEvent funcions *****/
+/********* XClientMessageEvent functions *****/
 
-int  make_XClientMessageEvent (){
-          return ((int) calloc(1, sizeof(XClientMessageEvent)));
+long  make_XClientMessageEvent (){
+          return ((long) calloc(1, sizeof(XClientMessageEvent)));
 }
 
 int  XClientMessageEvent_format(i)
@@ -3580,17 +3579,17 @@ int j;
           i->window = j;
 }
 
-Display *XClientMessageEvent_display(i)
+long  XClientMessageEvent_display(i)
 XClientMessageEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XClientMessageEvent_display(i, j)
 XClientMessageEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XClientMessageEvent_send_event(i)
@@ -3633,10 +3632,10 @@ int j;
 }
 
 
-/********* XMappingEvent funcions *****/
+/********* XMappingEvent functions *****/
 
-int  make_XMappingEvent (){
-          return ((int) calloc(1, sizeof(XMappingEvent)));
+long  make_XMappingEvent (){
+          return ((long) calloc(1, sizeof(XMappingEvent)));
 }
 
 int  XMappingEvent_count(i)
@@ -3691,17 +3690,17 @@ int j;
           i->window = j;
 }
 
-Display *XMappingEvent_display(i)
+long  XMappingEvent_display(i)
 XMappingEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XMappingEvent_display(i, j)
 XMappingEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XMappingEvent_send_event(i)
@@ -3744,10 +3743,10 @@ int j;
 }
 
 
-/********* XErrorEvent funcions *****/
+/********* XErrorEvent functions *****/
 
-int  make_XErrorEvent (){
-          return ((int) calloc(1, sizeof(XErrorEvent)));
+long  make_XErrorEvent (){
+          return ((long) calloc(1, sizeof(XErrorEvent)));
 }
 
 char XErrorEvent_minor_code(i)
@@ -3815,17 +3814,17 @@ int j;
           i->resourceid = j;
 }
 
-Display *XErrorEvent_display(i)
+long  XErrorEvent_display(i)
 XErrorEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XErrorEvent_display(i, j)
 XErrorEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XErrorEvent_type(i)
@@ -3842,10 +3841,10 @@ int j;
 }
 
 
-/********* XAnyEvent funcions *****/
+/********* XAnyEvent functions *****/
 
-int  make_XAnyEvent (){
-          return ((int) calloc(1, sizeof(XAnyEvent)));
+long  make_XAnyEvent (){
+          return ((long) calloc(1, sizeof(XAnyEvent)));
 }
 
 int  XAnyEvent_window(i)
@@ -3861,17 +3860,17 @@ int j;
           i->window = j;
 }
 
-Display *XAnyEvent_display(i)
+long  XAnyEvent_display(i)
 XAnyEvent* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_XAnyEvent_display(i, j)
 XAnyEvent* i;
-Display *j;
+long j;
 {
-          i->display = j;
+          i->display = (Display *) j;
 }
 
 int  XAnyEvent_send_event(i)
@@ -3914,8 +3913,8 @@ int j;
 }
 
 
-/********* XEvent funcions *****/
+/********* XEvent functions *****/
 
-int  make_XEvent (){
-          return ((int) calloc(1, sizeof(XEvent)));
+long  make_XEvent (){
+          return ((long) calloc(1, sizeof(XEvent)));
 }
