@@ -1,7 +1,7 @@
-/* XStruct-2.c           Hiep Huu Nguyen                         27 Aug 92 */
+/* XStruct-2.c           Hiep Huu Nguyen         27 Jun 06 */
 
 /* ; Copyright (c) 1994 Hiep Huu Nguyen and The University of Texas at Austin.
-
+; edited 27 Aug 92; 12 Aug 02 by G. Novak; 24 Jun 06 by GSN
 ; See the files gnu.license and dec.copyright .
 
 ; This program is free software; you can redistribute it and/or modify
@@ -22,15 +22,15 @@
 ; See the file dec.copyright for details. */
 
 
-/********* _XQEvent funcions *****/
+/********* _XQEvent functions *****/
 #define NEED_EVENTS
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xlibint.h>
 
 
-int  make__XQEvent (){
-          return ((int) calloc(1, sizeof(_XQEvent)));
+long  make__XQEvent (){
+          return ((long) calloc(1, sizeof(_XQEvent)));
 }
 
 XEvent _XQEvent_event(i)
@@ -46,24 +46,24 @@ XEvent j;
           i->event = j;
 }
 
-_XQEvent *_XQEvent_next(i)
+long _XQEvent_next(i)
 _XQEvent* i;
 {
-          return(i->next);
+          return((long) i->next);
 }
 
 void set__XQEvent_next(i, j)
 _XQEvent* i;
-_XQEvent *j;
+long j;
 {
-          i->next = j;
+          i->next = (struct _XSQEvent *) j;
 }
 
 
-/********* XCharStruct funcions *****/
+/********* XCharStruct functions *****/
 
-int  make_XCharStruct (){
-          return ((int) calloc(1, sizeof(XCharStruct)));
+long  make_XCharStruct (){
+          return ((long) calloc(1, sizeof(XCharStruct)));
 }
 
 int  XCharStruct_attributes(i)
@@ -145,10 +145,10 @@ int j;
 }
 
 
-/********* XFontProp funcions *****/
+/********* XFontProp functions *****/
 
-int  make_XFontProp (){
-          return ((int) calloc(1, sizeof(XFontProp)));
+long  make_XFontProp (){
+          return ((long) calloc(1, sizeof(XFontProp)));
 }
 
 int  XFontProp_card32(i)
@@ -178,10 +178,10 @@ int j;
 }
 
 
-/********* XFontStruct funcions *****/
+/********* XFontStruct functions *****/
 
-int  make_XFontStruct (){
-          return ((int) calloc(1, sizeof(XFontStruct)));
+long  make_XFontStruct (){
+          return ((long) calloc(1, sizeof(XFontStruct)));
 }
 
 int  XFontStruct_descent(i)
@@ -210,28 +210,28 @@ int j;
           i->ascent = j;
 }
 
-XCharStruct *XFontStruct_per_char(i)
+long  XFontStruct_per_char(i)
 XFontStruct* i;
 {
-          return(i->per_char);
+          return((long) i->per_char);
 }
 
 void set_XFontStruct_per_char(i, j)
 XFontStruct* i;
-XCharStruct *j;
+long j;
 {
-          i->per_char = j;
+          i->per_char = (XCharStruct *) j;
 }
 
-XCharStruct *XFontStruct_max_bounds(i)
+long XFontStruct_max_bounds(i)
 XFontStruct* i;
 {
-          return(&i->max_bounds);
+          return((long) &i->max_bounds);
 }
-XCharStruct *XFontStruct_min_bounds(i)
+long XFontStruct_min_bounds(i)
 XFontStruct* i;
 {
-          return(&i->min_bounds);
+          return((long) &i->min_bounds);
 }
 void set_XFontStruct_max_bounds(i, j)
 XFontStruct* i;
@@ -246,17 +246,17 @@ XCharStruct j;
           i->min_bounds = j;
 }
 
-XFontProp *XFontStruct_properties(i)
+long  XFontStruct_properties(i)
 XFontStruct* i;
 {
-          return(i->properties);
+          return((long) i->properties);
 }
 
 void set_XFontStruct_properties(i, j)
 XFontStruct* i;
-XFontProp *j;
+long j;
 {
-          i->properties = j;
+          i->properties = (XFontProp *) j;
 }
 
 int  XFontStruct_n_properties(i)
@@ -376,24 +376,24 @@ int j;
           i->fid = j;
 }
 
-XExtData * XFontStruct_ext_data(i)
+long  XFontStruct_ext_data(i)
 XFontStruct* i;
 {
-          return(i->ext_data);
+          return((long) i->ext_data);
 }
 
 void set_XFontStruct_ext_data(i, j)
 XFontStruct* i;
-XExtData *j;
+long j;
 {
-          i->ext_data = j;
+          i->ext_data = (XExtData *) j;
 }
 
 
-/********* XTextItem funcions *****/
+/********* XTextItem functions *****/
 
-int  make_XTextItem (){
-          return ((int) calloc(1, sizeof(XTextItem)));
+long  make_XTextItem (){
+          return ((long) calloc(1, sizeof(XTextItem)));
 }
 
 int  XTextItem_font(i)
@@ -435,24 +435,24 @@ int j;
           i->nchars = j;
 }
 
-char * XTextItem_chars(i)
+long  XTextItem_chars(i)
 XTextItem* i;
 {
-          return(i->chars);
+          return((long) i->chars);
 }
 
 void set_XTextItem_chars(i, j)
 XTextItem* i;
-char *j;
+long j;
 {
-          i->chars = j;
+          i->chars = (char *) j;
 }
 
 
-/********* XChar2b funcions *****/
+/********* XChar2b functions *****/
 
-int  make_XChar2b (){
-          return ((int) calloc(1, sizeof(XChar2b)));
+long  make_XChar2b (){
+          return ((long) calloc(1, sizeof(XChar2b)));
 }
 
 char XChar2b_byte2(i)
@@ -482,10 +482,10 @@ char j;
 }
 
 
-/********* XTextItem16 funcions *****/
+/********* XTextItem16 functions *****/
 
-int  make_XTextItem16 (){
-          return ((int) calloc(1, sizeof(XTextItem16)));
+long  make_XTextItem16 (){
+          return ((long) calloc(1, sizeof(XTextItem16)));
 }
 
 int  XTextItem16_font(i)
@@ -527,76 +527,76 @@ int j;
           i->nchars = j;
 }
 
-XChar2b * XTextItem16_chars(i)
+long  XTextItem16_chars(i)
 XTextItem16* i;
 {
-          return(i->chars);
+          return((long) i->chars);
 }
 
 void set_XTextItem16_chars(i, j)
 XTextItem16* i;
-XChar2b *j;
+long j;
 {
-          i->chars = j;
+          i->chars = (XChar2b *) j;
 }
 
 
-/********* XEDataObject funcions *****/
+/********* XEDataObject functions *****/
 
-int  make_XEDataObject (){
-          return ((int) calloc(1, sizeof(XEDataObject)));
+long  make_XEDataObject (){
+          return ((long) calloc(1, sizeof(XEDataObject)));
 }
 
-XFontStruct *XEDataObject_font(i)
+long  XEDataObject_font(i)
 XEDataObject* i;
 {
-          return(i->font);
+          return((long) i->font);
 }
 
 void set_XEDataObject_font(i, j)
 XEDataObject* i;
-XFontStruct *j;
+long j;
 {
-          i->font = j;
+          i->font = (XFontStruct *) j;
 }
 
-ScreenFormat *XEDataObject_pixmap_format(i)
+long  XEDataObject_pixmap_format(i)
 XEDataObject* i;
 {
-          return(i->pixmap_format);
+          return((long) i->pixmap_format);
 }
 
 void set_XEDataObject_pixmap_format(i, j)
 XEDataObject* i;
-ScreenFormat *j;
+long j;
 {
-          i->pixmap_format = j;
+          i->pixmap_format = (ScreenFormat *) j;
 }
 
-Screen *XEDataObject_screen(i)
+long  XEDataObject_screen(i)
 XEDataObject* i;
 {
-          return(i->screen);
+          return((long) i->screen);
 }
 
 void set_XEDataObject_screen(i, j)
 XEDataObject* i;
-Screen *j;
+long j;
 {
-          i->screen = j;
+          i->screen = (Screen *) j;
 }
 
-Visual *XEDataObject_visual(i)
+long  XEDataObject_visual(i)
 XEDataObject* i;
 {
-          return(i->visual);
+          return((long) i->visual);
 }
 
 void set_XEDataObject_visual(i, j)
 XEDataObject* i;
-Visual *j;
+long j;
 {
-          i->visual = j;
+          i->visual = (Visual *) j;
 }
 
 GC   XEDataObject_gc(i)
@@ -613,10 +613,10 @@ GC j;
 }
 
 
-/********* XSizeHints funcions *****/
+/********* XSizeHints functions *****/
 
-int  make_XSizeHints (){
-          return ((int) calloc(1, sizeof(XSizeHints)));
+long  make_XSizeHints (){
+          return ((long) calloc(1, sizeof(XSizeHints)));
 }
 
 int  XSizeHints_win_gravity(i)
@@ -856,10 +856,10 @@ int	j;
 }
 
 
-/********* XWMHints funcions *****/
+/********* XWMHints functions *****/
 
-int  make_XWMHints (){
-          return ((int) calloc(1, sizeof(XWMHints)));
+long  make_XWMHints (){
+          return ((long) calloc(1, sizeof(XWMHints)));
 }
 
 int  XWMHints_window_group(i)
@@ -980,10 +980,10 @@ int j;
 }
 
 
-/********* XTextProperty funcions *****/
+/********* XTextProperty functions *****/
 
-int  make_XTextProperty (){
-          return ((int) calloc(1, sizeof(XTextProperty)));
+long  make_XTextProperty (){
+          return ((long) calloc(1, sizeof(XTextProperty)));
 }
 
 int  XTextProperty_nitems(i)
@@ -1025,24 +1025,24 @@ int j;
           i->encoding = j;
 }
 
-unsigned char *XTextProperty_value(i)
+long  XTextProperty_value(i)
 XTextProperty* i;
 {
-          return(i->value);
+          return((long) i->value);
 }
 
 void set_XTextProperty_value(i, j)
 XTextProperty* i;
-unsigned char *j;
+long j;
 {
-          i->value = j;
+          i->value = (unsigned char *) j;
 }
 
 
-/********* XIconSize funcions *****/
+/********* XIconSize functions *****/
 
-int  make_XIconSize (){
-          return ((int) calloc(1, sizeof(XIconSize)));
+long  make_XIconSize (){
+          return ((long) calloc(1, sizeof(XIconSize)));
 }
 
 int  XIconSize_height_inc(i)
@@ -1124,43 +1124,43 @@ int j;
 }
 
 
-/********* XClassHint funcions *****/
+/********* XClassHint functions *****/
 
-int  make_XClassHint (){
-          return ((int) calloc(1, sizeof(XClassHint)));
+long  make_XClassHint (){
+          return ((long) calloc(1, sizeof(XClassHint)));
 }
 
-char *XClassHint_res_class(i)
+long  XClassHint_res_class(i)
 XClassHint* i;
 {
-          return(i->res_class);
+          return((long) i->res_class);
 }
 
 void set_XClassHint_res_class(i, j)
 XClassHint* i;
-char *j;
+long j;
 {
-          i->res_class = j;
+          i->res_class = (char *) j;
 }
 
-char *XClassHint_res_name(i)
+long  XClassHint_res_name(i)
 XClassHint* i;
 {
-          return(i->res_name);
+          return((long) i->res_name);
 }
 
 void set_XClassHint_res_name(i, j)
 XClassHint* i;
-char *j;
+long j;
 {
-          i->res_name = j;
+          i->res_name = (char *) j;
 }
 
 
-/********* XComposeStatus funcions *****/
+/********* XComposeStatus functions *****/
 
-int  make_XComposeStatus (){
-          return ((int) calloc(1, sizeof(XComposeStatus)));
+long  make_XComposeStatus (){
+          return ((long) calloc(1, sizeof(XComposeStatus)));
 }
 
 int  XComposeStatus_chars_matched(i)
@@ -1176,24 +1176,24 @@ int j;
           i->chars_matched = j;
 }
 
-XPointer  XComposeStatus_compose_ptr(i)
+long  XComposeStatus_compose_ptr(i)
 XComposeStatus* i;
 {
-          return(i->compose_ptr);
+          return((long) i->compose_ptr);
 }
 
 void set_XComposeStatus_compose_ptr(i, j)
 XComposeStatus* i;
-XPointer j;
+long j;
 {
-          i->compose_ptr = j;
+          i->compose_ptr = (XPointer) j;
 }
 
 
-/********* XVisualInfo funcions *****/
+/********* XVisualInfo functions *****/
 
-int  make_XVisualInfo (){
-          return ((int) calloc(1, sizeof(XVisualInfo)));
+long  make_XVisualInfo (){
+          return ((long) calloc(1, sizeof(XVisualInfo)));
 }
 
 int  XVisualInfo_bits_per_rgb(i)
@@ -1313,24 +1313,24 @@ int j;
           i->visualid = j;
 }
 
-Visual *XVisualInfo_visual(i)
+long  XVisualInfo_visual(i)
 XVisualInfo* i;
 {
-          return(i->visual);
+          return((long) i->visual);
 }
 
 void set_XVisualInfo_visual(i, j)
 XVisualInfo* i;
-Visual *j;
+long j;
 {
-          i->visual = j;
+          i->visual = (Visual *) j;
 }
 
 
-/********* XStandardColormap funcions *****/
+/********* XStandardColormap functions *****/
 
-int  make_XStandardColormap (){
-          return ((int) calloc(1, sizeof(XStandardColormap)));
+long  make_XStandardColormap (){
+          return ((long) calloc(1, sizeof(XStandardColormap)));
 }
 
 int  XStandardColormap_killid(i)
