@@ -1,7 +1,7 @@
-/* XStruct-4.c           Hiep Huu Nguyen                         27 Aug 92 */
+/* XStruct-4.c           Hiep Huu Nguyen       27 Jun 06 */
 
 /* ; Copyright (c) 1994 Hiep Huu Nguyen and The University of Texas at Austin.
-
+; edited 27 Aug 92; 12 Aug 2002 by G. Novak; 24 Jun 06 by GSN
 ; See the files gnu.license and dec.copyright .
 
 ; This program is free software; you can redistribute it and/or modify
@@ -22,15 +22,15 @@
 ; See the file dec.copyright for details. */
 
 #include <stdlib.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
-#include "include.h"
 
-/********* XExtCodes funcions *****/
+/********* XExtCodes functions *****/
 
-int  make_XExtCodes (){
-          return ((int) calloc(1, sizeof(XExtCodes)));
+long  make_XExtCodes (){
+          return ((long) calloc(1, sizeof(XExtCodes)));
 }
 
 int  XExtCodes_first_error(i)
@@ -86,10 +86,10 @@ int j;
 }
 
 
-/********* XPixmapFormatValues funcions *****/
+/********* XPixmapFormatValues functions *****/
 
-int  make_XPixmapFormatValues (){
-          return ((int) calloc(1, sizeof(XPixmapFormatValues)));
+long  make_XPixmapFormatValues (){
+          return ((long) calloc(1, sizeof(XPixmapFormatValues)));
 }
 
 int  XPixmapFormatValues_scanline_pad(i)
@@ -132,10 +132,10 @@ int j;
 }
 
 
-/********* XGCValues funcions *****/
+/********* XGCValues functions *****/
 
-int  make_XGCValues (){
-          return ((int) calloc(1, sizeof(XGCValues)));
+long  make_XGCValues (){
+          return ((long) calloc(1, sizeof(XGCValues)));
 }
 
 char XGCValues_dashes(i)
@@ -438,7 +438,7 @@ int j;
 }
 
 
-/********* GC funcions *****
+/********* GC functions *****
 
 int  make_GC (){
           GC i;
@@ -525,10 +525,10 @@ int j;
 
 */
 
-/********* Visual funcions *****/
+/********* Visual functions *****/
 
-int  make_Visual (){
-          return ((int) calloc(1, sizeof(Visual)));
+long  make_Visual (){
+          return ((long) calloc(1, sizeof(Visual)));
 }
 
 int  Visual_map_entries(i)
@@ -622,37 +622,37 @@ int j;
           i->visualid = j;
 }
 
-XExtData *Visual_ext_data(i)
+long  Visual_ext_data(i)
 Visual* i;
 {
-          return(i->ext_data);
+          return((long) i->ext_data);
 }
 
 void set_Visual_ext_data(i, j)
 Visual* i;
-XExtData *j;
+long j;
 {
-          i->ext_data = j;
+          i->ext_data = (XExtData *) j;
 }
 
 
-/********* Depth funcions *****/
+/********* Depth functions *****/
 
-int  make_Depth (){
-          return ((int) calloc(1, sizeof(Depth)));
+long  make_Depth (){
+          return ((long) calloc(1, sizeof(Depth)));
 }
 
-Visual *Depth_visuals(i)
+long  Depth_visuals(i)
 Depth* i;
 {
-          return(i->visuals);
+          return((long) i->visuals);
 }
 
 void set_Depth_visuals(i, j)
 Depth* i;
-Visual *j;
+long j;
 {
-          i->visuals = j;
+          i->visuals = (Visual *) j;
 }
 
 int  Depth_nvisuals(i)
@@ -682,10 +682,10 @@ int j;
 }
 
 
-/********* Screen funcions *****/
+/********* Screen functions *****/
 
-int  make_Screen (){
-          return ((int) calloc(1, sizeof(Screen)));
+long  make_Screen (){
+          return ((long) calloc(1, sizeof(Screen)));
 }
 
 int  Screen_root_input_mask(i)
@@ -792,30 +792,30 @@ int j;
           i->cmap = j;
 }
 
-GC  Screen_default_gc(i)
+long  Screen_default_gc(i)
 Screen* i;
 {
-          return(i->default_gc);
+          return((long) i->default_gc);
 }
 
 void set_Screen_default_gc(i, j)
 Screen* i;
-GC j;
+long j;
 {
-          i->default_gc = j;
+          i->default_gc = (GC) j;
 }
 
-Visual *Screen_root_visual(i)
+long  Screen_root_visual(i)
 Screen* i;
 {
-          return(i->root_visual);
+          return((long) i->root_visual);
 }
 
 void set_Screen_root_visual(i, j)
 Screen* i;
-Visual *j;
+long j;
 {
-          i->root_visual = j;
+          i->root_visual = (Visual *) j;
 }
 
 int  Screen_root_depth(i)
@@ -831,17 +831,17 @@ int j;
           i->root_depth = j;
 }
 
-Depth *Screen_depths(i)
+long  Screen_depths(i)
 Screen* i;
 {
-          return(i->depths);
+          return((long) i->depths);
 }
 
 void set_Screen_depths(i, j)
 Screen* i;
-Depth *j;
+long j;
 {
-          i->depths = j;
+          i->depths = (Depth *) j;
 }
 
 int  Screen_ndepths(i)
@@ -922,37 +922,37 @@ int j;
           i->root = j;
 }
 
-Display *Screen_display(i)
+long Screen_display(i)
 Screen* i;
 {
-          return(i->display);
+          return((long) i->display);
 }
 
 void set_Screen_display(i, j)
 Screen* i;
-Display *j;
+long j;
 {
-           i->display = j;
+           i->display = (struct _XDisplay *) j;
 }
 
-XExtData *Screen_ext_data(i)
+long  Screen_ext_data(i)
 Screen* i;
 {
-          return(i->ext_data);
+          return((long) i->ext_data);
 }
 
 void set_Screen_ext_data(i, j)
 Screen* i;
-XExtData *j;
+long j;
 {
-          i->ext_data = j;
+          i->ext_data = (XExtData *) j;
 }
 
 
-/********* ScreenFormat funcions *****/
+/********* ScreenFormat functions *****/
 
-int  make_ScreenFormat (){
-          return ((int) calloc(1, sizeof(ScreenFormat)));
+long  make_ScreenFormat (){
+          return ((long) calloc(1, sizeof(ScreenFormat)));
 }
 
 int  ScreenFormat_scanline_pad(i)
@@ -994,24 +994,24 @@ int j;
           i->depth = j;
 }
 
-XExtData *ScreenFormat_ext_data(i)
+long  ScreenFormat_ext_data(i)
 ScreenFormat* i;
 {
-          return(i->ext_data);
+          return((long) i->ext_data);
 }
 
 void set_ScreenFormat_ext_data(i, j)
 ScreenFormat* i;
-XExtData *j;
+long j;
 {
-          i->ext_data = j;
+          i->ext_data = (XExtData *) j;
 }
 
 
-/********* XSetWindowAttributes funcions *****/
+/********* XSetWindowAttributes functions *****/
 
-int  make_XSetWindowAttributes (){
-          return ((int) calloc(1, sizeof(XSetWindowAttributes)));
+long  make_XSetWindowAttributes (){
+          return ((long) calloc(1, sizeof(XSetWindowAttributes)));
 }
 
 int  XSetWindowAttributes_cursor(i)
@@ -1210,23 +1210,23 @@ int j;
 }
 
 
-/********* XWindowAttributes funcions *****/
+/********* XWindowAttributes functions *****/
 
-int  make_XWindowAttributes (){
-          return ((int) calloc(1, sizeof(XWindowAttributes)));
+long  make_XWindowAttributes (){
+          return ((long) calloc(1, sizeof(XWindowAttributes)));
 }
 
-Screen *XWindowAttributes_screen(i)
+long  XWindowAttributes_screen(i)
 XWindowAttributes* i;
 {
-          return(i->screen);
+          return((long) i->screen);
 }
 
 void set_XWindowAttributes_screen(i, j)
 XWindowAttributes* i;
-Screen *j;
+long j;
 {
-          i->screen = j;
+          i->screen = (Screen *) j;
 }
 
 int  XWindowAttributes_override_redirect(i)
@@ -1424,17 +1424,17 @@ int j;
           i->root = j;
 }
 
-Visual *XWindowAttributes_visual(i)
+long  XWindowAttributes_visual(i)
 XWindowAttributes* i;
 {
-          return(i->visual);
+          return((long) i->visual);
 }
 
 void set_XWindowAttributes_visual(i, j)
 XWindowAttributes* i;
-Visual *j;
+long j;
 {
-          i->visual = j;
+          i->visual = (Visual *) j;
 }
 
 int  XWindowAttributes_depth(i)
@@ -1516,23 +1516,23 @@ int j;
 }
 
 
-/********* XHostAddress funcions *****/
+/********* XHostAddress functions *****/
 
-int  make_XHostAddress (){
-          return ((int) calloc(1, sizeof(XHostAddress)));
+long  make_XHostAddress (){
+          return ((long) calloc(1, sizeof(XHostAddress)));
 }
 
-char *XHostAddress_address(i)
+long  XHostAddress_address(i)
 XHostAddress* i;
 {
-          return(i->address);
+          return((long) i->address);
 }
 
 void set_XHostAddress_address(i, j)
 XHostAddress* i;
-char *j;
+long j;
 {
-          i->address = j;
+          i->address = (char *) j;
 }
 
 int  XHostAddress_length(i)
@@ -1562,23 +1562,23 @@ int j;
 }
 
 
-/********* XImage funcions *****/
+/********* XImage functions *****/
 
-int  make_XImage (){
-          return ((int) calloc(1, sizeof(XImage)));
+long  make_XImage (){
+          return ((long) calloc(1, sizeof(XImage)));
 }
 
-XPointer XImage_obdata(i)
+long  XImage_obdata(i)
 XImage* i;
 {
-          return(i->obdata);
+          return((long) i->obdata);
 }
 
 void set_XImage_obdata(i, j)
 XImage* i;
-XPointer j;
+long j;
 {
-          i->obdata = j;
+          i->obdata = (XPointer) j;
 }
 
 int  XImage_blue_mask(i)
@@ -1711,17 +1711,17 @@ int j;
           i->byte_order = j;
 }
 
-char *XImage_data(i)
+long  XImage_data(i)
 XImage* i;
 {
-          return(i->data);
+          return((long) i->data);
 }
 
 void set_XImage_data(i, j)
 XImage* i;
-char *j;
+long j;
 {
-          i->data = j;
+          i->data = (char *) j;
 }
 
 int  XImage_format(i)
@@ -1777,10 +1777,10 @@ int j;
 }
 
 
-/********* XWindowChanges funcions *****/
+/********* XWindowChanges functions *****/
 
-int  make_XWindowChanges (){
-          return ((int) calloc(1, sizeof(XWindowChanges)));
+long  make_XWindowChanges (){
+          return ((long) calloc(1, sizeof(XWindowChanges)));
 }
 
 int  XWindowChanges_stack_mode(i)
@@ -1875,10 +1875,10 @@ int j;
 }
 
 
-/********* XColor funcions *****/
+/********* XColor functions *****/
 
-int  make_XColor (){
-          return ((int) calloc(1, sizeof(XColor)));
+long  make_XColor (){
+          return ((long) calloc(1, sizeof(XColor)));
 }
 
 char XColor_pad(i)
@@ -1960,10 +1960,10 @@ int j;
 }
 
 
-/********* XSegment funcions *****/
+/********* XSegment functions *****/
 
-int  make_XSegment (){
-          return ((int) calloc(1, sizeof(XSegment)));
+long  make_XSegment (){
+          return ((long) calloc(1, sizeof(XSegment)));
 }
 
 int  XSegment_y2(i)
@@ -2019,10 +2019,10 @@ int j;
 }
 
 
-/********* XPoint funcions *****/
+/********* XPoint functions *****/
 
-int  make_XPoint (){
-          return ((int) calloc(1, sizeof(XPoint)));
+long  make_XPoint (){
+          return ((long) calloc(1, sizeof(XPoint)));
 }
 
 int  XPoint_y(i)
@@ -2052,10 +2052,10 @@ int j;
 }
 
 
-/********* XRectangle funcions *****/
+/********* XRectangle functions *****/
 
-int  make_XRectangle (){
-          return ((int) calloc(1, sizeof(XRectangle)));
+long  make_XRectangle (){
+          return ((long) calloc(1, sizeof(XRectangle)));
 }
 
 int  XRectangle_height(i)
@@ -2111,10 +2111,10 @@ int j;
 }
 
 
-/********* XArc funcions *****/
+/********* XArc functions *****/
 
-int  make_XArc (){
-          return ((int) calloc(1, sizeof(XArc)));
+long  make_XArc (){
+          return ((long) calloc(1, sizeof(XArc)));
 }
 
 int  XArc_angle2(i)
@@ -2196,10 +2196,10 @@ int j;
 }
 
 
-/********* XKeyboardControl funcions *****/
+/********* XKeyboardControl functions *****/
 
-int  make_XKeyboardControl (){
-          return ((int) calloc(1, sizeof(XKeyboardControl)));
+long  make_XKeyboardControl (){
+          return ((long) calloc(1, sizeof(XKeyboardControl)));
 }
 
 int  XKeyboardControl_auto_repeat_mode(i)
@@ -2307,10 +2307,10 @@ int j;
 }
 
 
-/********* XKeyboardState funcions *****/
+/********* XKeyboardState functions *****/
 
-int  make_XKeyboardState (){
-          return ((int) calloc(1, sizeof(XKeyboardState)));
+long  make_XKeyboardState (){
+          return ((long) calloc(1, sizeof(XKeyboardState)));
 }
 
 char *XKeyboardState_auto_repeats(i)
@@ -2405,10 +2405,10 @@ int j;
 }
 
 
-/********* XTimeCoord funcions *****/
+/********* XTimeCoord functions *****/
 
-int  make_XTimeCoord (){
-          return ((int) calloc(1, sizeof(XTimeCoord)));
+long  make_XTimeCoord (){
+          return ((long) calloc(1, sizeof(XTimeCoord)));
 }
 
 int  XTimeCoord_y(i)
@@ -2451,23 +2451,23 @@ int j;
 }
 
 
-/********* XModifierKeymap funcions *****/
+/********* XModifierKeymap functions *****/
 
-int  make_XModifierKeymap (){
-          return ((int) calloc(1, sizeof(XModifierKeymap)));
+long  make_XModifierKeymap (){
+          return ((long) calloc(1, sizeof(XModifierKeymap)));
 }
 
-KeyCode *XModifierKeymap_modifiermap(i)
+long  XModifierKeymap_modifiermap(i)
 XModifierKeymap* i;
 {
-          return(i->modifiermap);
+          return((long) i->modifiermap);
 }
 
 void set_XModifierKeymap_modifiermap(i, j)
 XModifierKeymap* i;
-KeyCode *j;
+long j;
 {
-          i->modifiermap = j;
+          i->modifiermap = (KeyCode *) j;
 }
 
 int  XModifierKeymap_max_keypermod(i)
